@@ -63,7 +63,7 @@ static bool thread_group_in_exec(Task* t) {
     return false;
   }
   for (Task* tt : t->thread_group()->task_set()) {
-    if (tt == t) {
+    if (tt == t || t->already_exited()) {
       continue;
     }
     RecordTask* rt = static_cast<RecordTask*>(tt);
@@ -277,28 +277,6 @@ remote_code_ptr AddressSpace::find_syscall_instruction(Task* t) {
           .as_int());
 }
 
-static string find_rr_page_file(Task* t) {
-  string path = resource_path() + "share/rr/rr_page_";
-  switch (t->arch()) {
-    case x86:
-      path += "32";
-      break;
-    case x86_64:
-      path += "64";
-      break;
-    case aarch64:
-      path += "arm64";
-      break;
-    default:
-      ASSERT(t, false) << "Unknown architecture";
-      return path;
-  }
-  if (!t->session().is_recording()) {
-    path += "_replay";
-  }
-  return path;
-}
-
 void AddressSpace::map_rr_page(AutoRemoteSyscalls& remote) {
   int prot = PROT_EXEC | PROT_READ;
   int flags = MAP_PRIVATE | MAP_FIXED;
@@ -307,25 +285,57 @@ void AddressSpace::map_rr_page(AutoRemoteSyscalls& remote) {
   Task* t = remote.task();
   SupportedArch arch = t->arch();
 
-  string path = find_rr_page_file(t);
+  const char *fname;
+  switch (t->arch()) {
+    case x86_64:
+    case aarch64:
+      fname = RRPAGE_LIB_FILENAME;
+      break;
+    case x86:
+#if defined(__x86_64__)
+      fname = RRPAGE_LIB_FILENAME_32;
+#else
+      fname = RRPAGE_LIB_FILENAME;
+#endif
+      break;
+  }
+
+  string path = find_helper_library(fname);
+  if (path.empty()) {
+    FATAL() << "Failed to locate " << fname;
+  }
+  path += fname;
+  size_t offset_pages = t->session().is_recording() ?
+    RRPAGE_RECORD_PAGE_OFFSET : RRPAGE_REPLAY_PAGE_OFFSET;
+  size_t offset_bytes = offset_pages*rr_page_size();
 
   {
     ScopedFd page(path.c_str(), O_RDONLY);
-    ASSERT(t, page.is_open()) << "Failed to open rr_page file " << path;
+    ASSERT(t, page.is_open()) << "Failed to open rrpage library " << path;
     long child_fd = remote.send_fd(page.get());
     ASSERT(t, child_fd >= 0);
+    if (t->session().is_recording()) {
+      remote.infallible_mmap_syscall(rr_page_start() - offset_bytes, offset_bytes, prot, flags,
+                                    child_fd, 0);
+    }
     remote.infallible_mmap_syscall(rr_page_start(), rr_page_size(), prot, flags,
-                                   child_fd, 0);
+                                   child_fd, offset_pages);
 
     struct stat fstat = t->stat_fd(child_fd);
     file_name = t->file_name_of_fd(child_fd);
 
     remote.infallible_syscall(syscall_number_for_close(arch), child_fd);
 
-    map(t, rr_page_start(), rr_page_size(), prot, flags, 0, file_name,
+    map(t, rr_page_start(), rr_page_size(), prot, flags,
+        offset_pages * page_size(), file_name,
         fstat.st_dev, fstat.st_ino);
+    mapping_flags_of(rr_page_start()) = Mapping::IS_RR_PAGE;
+    if (t->session().is_recording()) {
+      map(t, rr_page_start() - offset_bytes, offset_bytes, prot, flags,
+          0, file_name,
+          fstat.st_dev, fstat.st_ino);
+    }
   }
-  mapping_flags_of(rr_page_start()) = Mapping::IS_RR_PAGE;
 
   if (t->session().is_recording()) {
     // brk() will not have been called yet so the brk area is empty.
@@ -482,9 +492,21 @@ vector<AddressSpace::SyscallType> AddressSpace::rr_page_syscalls() {
   return result;
 }
 
-void AddressSpace::save_auxv(Task* t) { saved_auxv_ = read_auxv(t); }
+void AddressSpace::save_auxv(Task* t) {
+  saved_auxv_ = read_auxv(t);
+  save_interpreter_base(t, saved_auxv());
+}
 
-void AddressSpace::read_mm_map(Task* t, struct prctl_mm_map* map) {
+void AddressSpace::save_interpreter_base(Task* t, std::vector<uint8_t> auxv) {
+  saved_interpreter_base_ = read_interpreter_base(auxv);
+  save_ld_path(t, saved_interpreter_base());
+}
+
+void AddressSpace::save_ld_path(Task* t, remote_ptr<void> interpreter_base) {
+  saved_ld_path_ = read_ld_path(t, interpreter_base);
+}
+
+void AddressSpace::read_mm_map(Task* t, NativeArch::prctl_mm_map* map) {
   char buf[PATH_MAX+1024];
   {
     string proc_stat = t->proc_stat_path();
@@ -501,7 +523,7 @@ void AddressSpace::read_mm_map(Task* t, struct prctl_mm_map* map) {
   // We don't change /proc/pid/exe, since we're unlikely to have CAP_SYS_ADMIN
   map->exe_fd = -1;
   // auxv is restored separately
-  map->auxv = NULL;
+  map->auxv.val = 0;
   map->auxv_size = 0;
   // All of these fields of /proc/pid/stat, we don't use (currently)
   char state;
@@ -1011,10 +1033,11 @@ bool AddressSpace::add_breakpoint(remote_code_ptr addr, BreakpointType type) {
     ssize_t bkpt_size = bkpt_instruction_length(arch());
     // Grab a random task from the VM so we can use its
     // read/write_mem() helpers.
-    Task* t = *task_set().begin();
-    if (bkpt_size !=
-        t->read_bytes_fallible(addr.to_data_ptr<uint8_t>(),
-                               bkpt_size, overwritten_data)) {
+    Task* t = first_running_task();
+    if (!t ||
+        bkpt_size !=
+            t->read_bytes_fallible(addr.to_data_ptr<uint8_t>(),
+                                   bkpt_size, overwritten_data)) {
       return false;
     }
     t->write_bytes_helper(addr.to_data_ptr<uint8_t>(), bkpt_size,
@@ -1040,19 +1063,23 @@ void AddressSpace::remove_all_breakpoints() {
 void AddressSpace::suspend_breakpoint_at(remote_code_ptr addr) {
   auto it = breakpoints.find(addr);
   if (it != breakpoints.end()) {
-    Task* t = *task_set().begin();
-    t->write_bytes_helper(addr.to_data_ptr<uint8_t>(),
-      bkpt_instruction_length(arch()), it->second.overwritten_data);
+    Task* t = first_running_task();
+    if (t) {
+      t->write_bytes_helper(addr.to_data_ptr<uint8_t>(),
+        bkpt_instruction_length(arch()), it->second.overwritten_data);
+    }
   }
 }
 
 void AddressSpace::restore_breakpoint_at(remote_code_ptr addr) {
   auto it = breakpoints.find(addr);
   if (it != breakpoints.end()) {
-    Task* t = *task_set().begin();
-    t->write_bytes_helper(addr.to_data_ptr<uint8_t>(),
-      bkpt_instruction_length(arch()),
-      breakpoint_insn(arch()));
+    Task* t = first_running_task();
+    if (t) {
+      t->write_bytes_helper(addr.to_data_ptr<uint8_t>(),
+        bkpt_instruction_length(arch()),
+        breakpoint_insn(arch()));
+    }
   }
 }
 
@@ -1124,7 +1151,10 @@ bool AddressSpace::restore_watchpoints() {
 
 bool AddressSpace::update_watchpoint_value(const MemoryRange& range,
                                            Watchpoint& watchpoint) {
-  Task* t = *task_set().begin();
+  Task* t = first_running_task();
+  if (!t) {
+    return false;
+  }
   bool valid = true;
   vector<uint8_t> value_bytes = watchpoint.value_bytes;
   for (size_t i = 0; i < value_bytes.size(); ++i) {
@@ -1357,6 +1387,10 @@ static string strip_deleted(const string& s) {
   return s;
 }
 
+string KernelMapping::fsname_strip_deleted() const {
+  return strip_deleted(fsname_);
+}
+
 enum HandleHeap { TREAT_HEAP_AS_ANONYMOUS, RESPECT_HEAP };
 
 static bool normalized_file_names_equal(const KernelMapping& km1,
@@ -1621,7 +1655,7 @@ AddressSpace::AddressSpace(Task* t, const string& exe, uint32_t exec_count)
       stopping_breakpoint_table_entry_size_(0),
       first_run_event_(0) {
   // TODO: this is a workaround of
-  // https://github.com/mozilla/rr/issues/1113 .
+  // https://github.com/rr-debugger/rr/issues/1113 .
   if (session_->done_initial_exec()) {
     populate_address_space(t);
     DEBUG_ASSERT(!vdso_start_addr.is_null());
@@ -1657,6 +1691,8 @@ AddressSpace::AddressSpace(Session* session, const AddressSpace& o,
       stopping_breakpoint_table_(o.stopping_breakpoint_table_),
       stopping_breakpoint_table_entry_size_(o.stopping_breakpoint_table_entry_size_),
       saved_auxv_(o.saved_auxv_),
+      saved_interpreter_base_(o.saved_interpreter_base_),
+      saved_ld_path_(o.saved_ld_path_),
       first_run_event_(0) {
   for (auto& m : mem) {
     // The original address space continues to have exclusive ownership of
@@ -1928,7 +1964,10 @@ void AddressSpace::destroy_breakpoint(BreakpointMap::const_iterator it) {
   if (task_set().empty()) {
     return;
   }
-  Task* t = *task_set().begin();
+  Task* t = first_running_task();
+  if (!t) {
+    return;
+  }
   auto ptr = it->first.to_data_ptr<uint8_t>();
   auto data = it->second.overwritten_data;
   if (bkpt_instruction_length(arch()) == 1) {
@@ -2108,6 +2147,14 @@ static MemoryRange adjust_range_for_stack_growth(const KernelMapping& km) {
   return MemoryRange(start, km.end());
 }
 
+static bool overlaps_asan_usage(const MemoryRange& r) {
+  MemoryRange asan_shadow(remote_ptr<void>((uintptr_t)0x00007fff7000LL),
+                          remote_ptr<void>((uintptr_t)0x10007fff8000LL));
+  MemoryRange asan_allocator_reserved(remote_ptr<void>((uintptr_t)0x600000000000LL),
+                                      remote_ptr<void>((uintptr_t)0x640000000000LL));
+  return r.intersects(asan_shadow) || r.intersects(asan_allocator_reserved);
+}
+
 // Choose a 4TB range to exclude from random mappings. This makes room for
 // advanced trace analysis tools that require a large address range in tracees
 // that is never mapped.
@@ -2117,40 +2164,64 @@ static MemoryRange choose_global_exclusion_range() {
   }
 
   const uint64_t range_size = uint64_t(4)*1024*1024*1024*1024;
-  int bits = random_addr_bits(x86_64);
-  uint64_t r = ((uint64_t)(uint32_t)random() << 32) | (uint32_t)random();
-  uint64_t r_addr = r & ((uint64_t(1) << bits) - 1);
-  r_addr = min(r_addr, (uint64_t(1) << bits) - range_size);
-  remote_ptr<void> addr = floor_page_size(remote_ptr<void>(r_addr));
-  return MemoryRange(addr, (uintptr_t)range_size);
+  while (true) {
+    int bits = random_addr_bits(x86_64);
+    uint64_t r = ((uint64_t)(uint32_t)random() << 32) | (uint32_t)random();
+    uint64_t r_addr = r & ((uint64_t(1) << bits) - 1);
+    r_addr = min(r_addr, (uint64_t(1) << bits) - range_size);
+    remote_ptr<void> addr = floor_page_size(remote_ptr<void>(r_addr));
+    MemoryRange ret(addr, (uintptr_t)range_size);
+    if (!overlaps_asan_usage(ret)) {
+      return ret;
+    }
+  }
+}
+
+MemoryRange AddressSpace::get_global_exclusion_range() {
+  static MemoryRange global_exclusion_range = choose_global_exclusion_range();
+  return global_exclusion_range;
 }
 
 remote_ptr<void> AddressSpace::chaos_mode_find_free_memory(RecordTask* t,
-                                                           size_t len) {
-  static MemoryRange global_exclusion_range = choose_global_exclusion_range();
+                                                           size_t len, remote_ptr<void> hint) {
+  MemoryRange global_exclusion_range = get_global_exclusion_range();
+  // NB: Above RR_PAGE_ADDR is probably not free anyways, but if it somehow is
+  // don't hand it out again.
+  static MemoryRange rrpage_so_range = MemoryRange(RR_PAGE_ADDR - page_size(), RR_PAGE_ADDR + page_size());
+
+  // Ignore the hint half the time.
+  if (hint && (random() & 1)) {
+    hint = nullptr;
+  }
 
   int bits = random_addr_bits(t->arch());
   uint64_t addr_space_limit = uint64_t(1) << bits;
   while (true) {
     remote_ptr<void> addr;
-    // Half the time, try to allocate at a completely random address. The other
-    // half of the time, we'll try to allocate immediately before or after a
-    // randomly chosen existing mapping.
-    if (random() % 2) {
-      // Some of these addresses will not be mappable. That's fine, the
-      // kernel will fall back to a valid address if the hint is not valid.
-      uint64_t r = ((uint64_t)(uint32_t)random() << 32) | (uint32_t)random();
-      addr = floor_page_size(remote_ptr<void>(r & (addr_space_limit - 1)));
+    if (hint) {
+      addr = hint;
+      // Don't try using the hint again.
+      hint = nullptr;
     } else {
-      ASSERT(t, !mem.empty());
-      int map_index = random() % mem.size();
-      int map_count = 0;
-      for (const auto& m : maps()) {
-        if (map_count == map_index) {
-          addr = m.map.start();
-          break;
+      // Half the time, try to allocate at a completely random address. The other
+      // half of the time, we'll try to allocate immediately before or after a
+      // randomly chosen existing mapping.
+      if (random() % 2) {
+        // Some of these addresses will not be mappable. That's fine, the
+        // kernel will fall back to a valid address if the hint is not valid.
+        uint64_t r = ((uint64_t)(uint32_t)random() << 32) | (uint32_t)random();
+        addr = floor_page_size(remote_ptr<void>(r & (addr_space_limit - 1)));
+      } else {
+        ASSERT(t, !mem.empty());
+        int map_index = random() % mem.size();
+        int map_count = 0;
+        for (const auto& m : maps()) {
+          if (map_count == map_index) {
+            addr = m.map.start();
+            break;
+          }
+          ++map_count;
         }
-        ++map_count;
       }
     }
 
@@ -2181,16 +2252,15 @@ remote_ptr<void> AddressSpace::chaos_mode_find_free_memory(RecordTask* t,
       continue;
     }
     MemoryRange r(addr, ceil_page_size(len));
+    if (r.intersects(rrpage_so_range)) {
+      continue;
+    }
     if (r.intersects(global_exclusion_range)) {
       continue;
     }
     if (t->session().asan_active() && sizeof(size_t) == 8) {
       LOG(debug) << "Checking ASAN shadow";
-      MemoryRange asan_shadow(remote_ptr<void>((uintptr_t)0x00007fff7000LL),
-                              remote_ptr<void>((uintptr_t)0x10007fff8000LL));
-      MemoryRange asan_allocator_reserved(remote_ptr<void>((uintptr_t)0x600000000000LL),
-                                          remote_ptr<void>((uintptr_t)0x640000000000LL));
-      if (r.intersects(asan_shadow) || r.intersects(asan_allocator_reserved)) {
+      if (overlaps_asan_usage(r)) {
         continue;
       }
     }
@@ -2264,13 +2334,7 @@ void AddressSpace::fd_tables_changed() {
   }
   DEBUG_ASSERT(task_set().size() != 0);
   uint8_t fdt_uniform = true;
-  RecordTask* rt = nullptr;
-  // Find a suitable task
-  for (auto* t : task_set()) {
-    if (!t->is_dying()) {
-      rt = static_cast<RecordTask*>(t);
-    }
-  }
+  RecordTask* rt = static_cast<RecordTask*>(first_running_task());
   if (!rt) {
     return;
   }
