@@ -29,8 +29,6 @@
 #include "remote_code_ptr.h"
 #include "util.h"
 
-struct prctl_mm_map;
-
 namespace rr {
 
 class AutoRemoteSyscalls;
@@ -132,6 +130,7 @@ public:
   }
 
   const std::string& fsname() const { return fsname_; }
+  std::string fsname_strip_deleted() const;
   dev_t device() const { return device_; }
   ino_t inode() const { return inode_; }
   int prot() const { return prot_; }
@@ -728,9 +727,10 @@ public:
     SupportedArch arch, remote_code_ptr ip);
 
   /**
-   * Return a pointer to 8 bytes of 0xFF
+   * Return a pointer to 8 bytes of 0xFF.
+   * (Currently only set during record / not part of the ABI)
    */
-  static remote_ptr<uint8_t> rr_page_ff_bytes() { return RR_PAGE_FF_BYTES; }
+  static remote_ptr<uint8_t> rr_page_record_ff_bytes() { return RR_PAGE_FF_BYTES; }
 
   /**
    * Locate a syscall instruction in t's VDSO.
@@ -752,7 +752,13 @@ public:
   const std::vector<uint8_t>& saved_auxv() { return saved_auxv_; }
   void save_auxv(Task* t);
 
-  void read_mm_map(Task* t, struct prctl_mm_map* map);
+  remote_ptr<void> saved_interpreter_base() { return saved_interpreter_base_; }
+  void save_interpreter_base(Task* t, std::vector<uint8_t> auxv);
+
+  std::string saved_ld_path() { return saved_ld_path_;}
+  void save_ld_path(Task* t, remote_ptr<void>);
+
+  void read_mm_map(Task* t, NativeArch::prctl_mm_map* map);
 
   /**
    * Reads the /proc/<pid>/maps entry for a specific address. Does no caching.
@@ -769,7 +775,7 @@ public:
 
   static uint32_t chaos_mode_min_stack_size() { return 8 * 1024 * 1024; }
 
-  remote_ptr<void> chaos_mode_find_free_memory(RecordTask* t, size_t len);
+  remote_ptr<void> chaos_mode_find_free_memory(RecordTask* t, size_t len, remote_ptr<void> hint);
   remote_ptr<void> find_free_memory(
       size_t len, remote_ptr<void> after = remote_ptr<void>());
 
@@ -821,6 +827,12 @@ public:
   int stopping_breakpoint_table_entry_size() { return stopping_breakpoint_table_entry_size_; }
 
   // Also sets brk_ptr.
+  enum {
+    RRVDSO_PAGE_OFFSET = 2,
+    RRPAGE_RECORD_PAGE_OFFSET = 3,
+    RRPAGE_REPLAY_PAGE_OFFSET = 4
+  };
+
   void map_rr_page(AutoRemoteSyscalls& remote);
   void unmap_all_but_rr_page(AutoRemoteSyscalls& remote);
 
@@ -838,6 +850,8 @@ public:
    */
   void fd_tables_changed();
 
+  static MemoryRange get_global_exclusion_range();
+
 private:
   struct Breakpoint;
   typedef std::map<remote_code_ptr, Breakpoint> BreakpointMap;
@@ -1123,6 +1137,8 @@ private:
   int stopping_breakpoint_table_entry_size_;
 
   std::vector<uint8_t> saved_auxv_;
+  remote_ptr<void> saved_interpreter_base_;
+  std::string saved_ld_path_;
 
   /**
    * The time of the first event that ran code for a task in this address space.
