@@ -82,11 +82,17 @@ AutoRemoteSyscalls::AutoRemoteSyscalls(Task* t,
       initial_regs(t->regs()),
       initial_ip(t->ip()),
       initial_sp(t->regs().sp()),
+      initial_at_seccomp(t->ptrace_event() == PTRACE_EVENT_SECCOMP),
       restore_wait_status(t->status()),
       new_tid_(-1),
       scratch_mem_was_mapped(false),
       use_singlestep_path(false),
       enable_mem_params_(enable_mem_params) {
+  if (initial_at_seccomp) {
+    // This should only ever happen during recording - we don't use the
+    // seccomp traps during replay.
+    ASSERT(t, t->session().is_recording());
+  }
   // We support two paths for syscalls:
   // -- a fast path using a privileged untraced syscall and PTRACE_SINGLESTEP.
   // This only requires a single task-wait.
@@ -107,6 +113,11 @@ AutoRemoteSyscalls::AutoRemoteSyscalls(Task* t,
 }
 
 void AutoRemoteSyscalls::setup_path(bool enable_singlestep_path) {
+#if defined(__aarch64__)
+  // XXXkhuey this fast path doesn't work on AArch64 yet, go slow instead
+  enable_singlestep_path = false;
+#endif
+
   if (!replaced_bytes.empty()) {
     // XXX what to do here to clean up if the task died unexpectedly?
     t->write_mem(remote_ptr<uint8_t>(initial_regs.ip().to_data_ptr<uint8_t>()),
@@ -206,6 +217,20 @@ void AutoRemoteSyscalls::restore_state_to(Task* t) {
   regs.set_sp(initial_sp);
   // Restore stomped registers.
   t->set_regs(regs);
+  // If we were sitting at a seccomp trap, try to get back there by resuming
+  // here. Since the original register contents caused a seccomp trap,
+  // re-running the syscall with the same registers should put us right back
+  // to this same seccomp trap.
+  if (initial_at_seccomp && t->ptrace_event() != PTRACE_EVENT_SECCOMP) {
+    RecordTask* rt = static_cast<RecordTask*>(t);
+    while (true) {
+      rt->resume_execution(RESUME_CONT, RESUME_WAIT, RESUME_NO_TICKS);
+      if (rt->ptrace_event())
+        break;
+      rt->stash_sig();
+    }
+    ASSERT(rt, rt->ptrace_event() == PTRACE_EVENT_SECCOMP);
+  }
   t->set_status(restore_wait_status);
 }
 
@@ -278,7 +303,11 @@ long AutoRemoteSyscalls::syscall_base(int syscallno, Registers& callregs) {
       ASSERT(t, false) << "Unexpected status " << t->status();
     }
   } else {
-    t->enter_syscall();
+    if (initial_at_seccomp && t->ptrace_event() == PTRACE_EVENT_SECCOMP) {
+      LOG(debug) << "Skipping enter_syscall - already at seccomp stop";
+    } else {
+      t->enter_syscall();
+    }
     LOG(debug) << "Used enter_syscall; status=" << t->status();
     // proceed to syscall exit
     t->resume_execution(RESUME_SYSCALL, RESUME_WAIT, RESUME_NO_TICKS);
@@ -579,6 +608,10 @@ int64_t AutoRemoteSyscalls::infallible_lseek_syscall(int fd, int64_t offset,
 }
 
 void AutoRemoteSyscalls::check_syscall_result(long ret, int syscallno, bool allow_death) {
+  if (word_size(t->arch()) == 4) {
+    // Sign-extend ret because it can be a 32-bit negative errno
+    ret = (int)ret;
+  }
   if (allow_death && ret == -ESRCH) {
     return;
   }
