@@ -45,14 +45,23 @@ template <typename D> struct  __attribute__((packed)) Dwarf5CompilationUnitHeade
   uint8_t unit_type;
   uint8_t address_size;
   typename D::Offset debug_abbrev_offset;
+
+  void install_dwo_id(DwarfCompilationUnit* unit) const {
+    unit->set_dwo_id(0);
+  }
+};
+
+template <typename D> struct  __attribute__((packed)) Dwarf5SkeletonSplitCompilationUnitHeader {
+  typedef D Size;
+  typename D::CompilationUnitPreamble preamble;
+  uint16_t version;
+  uint8_t unit_type;
+  uint8_t address_size;
+  typename D::Offset debug_abbrev_offset;
   uint64_t dwo_id;
 
   void install_dwo_id(DwarfCompilationUnit* unit) const {
-    if (version == 5 && (unit_type == DW_UT_skeleton || unit_type == DW_UT_split_compile)) {
-      unit->set_dwo_id(dwo_id);
-    } else {
-      unit->set_dwo_id(0);
-    }
+    unit->set_dwo_id(dwo_id);
   }
 };
 
@@ -263,9 +272,20 @@ static size_t form_size(DWForm form, size_t address_size, size_t dwarf_size, Dwa
     case DW_FORM_strx2: return 2;
     case DW_FORM_strx3: return 3;
     case DW_FORM_strx4: return 4;
+    case DW_FORM_string: {
+      auto before = span->size();
+      DwarfSpan a_span(*span);
+      a_span.read_null_terminated_string(ok);
+      if (!ok) {
+        return 0;
+      }
+      return before - a_span.size();
+    }
     case DW_FORM_sec_offset: return dwarf_size;
     case DW_FORM_flag_present: return 0;
     case DW_FORM_implicit_const: return 0;
+    case DW_FORM_strp_sup: return dwarf_size;
+    case DW_FORM_GNU_strp_alt: return dwarf_size;
     default:
       LOG(warn) << "form " << form << " not supported!";
       *ok = false;
@@ -353,6 +373,14 @@ static const char* decode_string(const DwarfCompilationUnit& cu, DwarfSpan span,
       }
       return debug_strs.debug_str.subspan(offset).read_null_terminated_string(ok);
     }
+    case DW_FORM_strp_sup:
+    case DW_FORM_GNU_strp_alt: {
+      uint64_t offset = decode_unsigned_literal(span, ok);
+      if (!*ok) {
+        return nullptr;
+      }
+      return debug_strs.debug_str_sup.subspan(offset).read_null_terminated_string(ok);
+    }
     case DW_FORM_line_strp: {
       uint64_t offset = decode_unsigned_literal(span, ok);
       if (!*ok) {
@@ -375,6 +403,8 @@ static const char* decode_string(const DwarfCompilationUnit& cu, DwarfSpan span,
       }
       return debug_strs.debug_str.subspan(offset).read_null_terminated_string(ok);
     }
+    case DW_FORM_string:
+      return span.read_null_terminated_string(ok);
     default:
       LOG(warn) << "Unknown string form " << form;
       *ok = false;
@@ -433,7 +463,15 @@ template <typename D> void DwarfCompilationUnit::init_size(DwarfSpan* debug_info
   if (2 <= h->version && h->version <= 4) {
     init<Dwarf4CompilationUnitHeader<D>>(debug_info, abbrevs, ok);
   } else if (h->version == 5) {
-    init<Dwarf5CompilationUnitHeader<D>>(debug_info, abbrevs, ok);
+    auto hh = DwarfSpan(*debug_info).read<Dwarf5CompilationUnitHeader<D>>(ok);
+    if (!ok) {
+      return;
+    }
+    if (hh->unit_type == DW_UT_skeleton || hh->unit_type == DW_UT_split_compile) {
+      init<Dwarf5SkeletonSplitCompilationUnitHeader<D>>(debug_info, abbrevs, ok);
+    } else {
+      init<Dwarf5CompilationUnitHeader<D>>(debug_info, abbrevs, ok);
+    }
   } else {
     LOG(warn) << "Unknown compilation unit version " << h->version;
     *ok = false;
@@ -443,7 +481,7 @@ template <typename D> void DwarfCompilationUnit::init_size(DwarfSpan* debug_info
 template <typename H> void DwarfCompilationUnit::init(DwarfSpan* debug_info, DwarfAbbrevs& abbrevs, bool* ok) {
   DwarfSpan span(*debug_info);
   auto h = span.read<H>(ok);
-  if (!ok) {
+  if (!*ok) {
     return;
   }
   uint64_t length = h->preamble.unit_length;
@@ -455,6 +493,9 @@ template <typename H> void DwarfCompilationUnit::init(DwarfSpan* debug_info, Dwa
   debug_info->consume(length + sizeof(h->preamble));
   DwarfAbbrevSet& abbrev_set = abbrevs.lookup(h->debug_abbrev_offset);
   die_ = make_unique<DwarfDIE>(span, abbrev_set, sizeof(typename H::Size::Offset), h->address_size, ok);
+  if (!*ok) {
+    return;
+  }
   if (die_->tag() != DW_TAG_compile_unit &&
       die_->tag() != DW_TAG_partial_unit &&
       die_->tag() != DW_TAG_skeleton_unit) {
