@@ -734,7 +734,7 @@ bool GdbConnection::query(char* payload) {
       req.target = query_thread;
       req.mem().addr = strtoul(args, &args, 16);
       parser_assert(';' == *args++);
-      req.mem().len = strtoul(args, &args, 16);
+      req.mem().len = strtoull(args, &args, 16);
       parser_assert(';' == *args++);
       read_binary_data((const uint8_t*)args, inbuf.data() + packetend,
                        req.mem().data);
@@ -955,22 +955,22 @@ bool GdbConnection::process_vpacket(char* payload) {
     string event_str = decode_ascii_encoded_hex_str(arg1);
     char* endp;
     if (event_str[0] == 'c') {
-      int param = strtol(event_str.c_str() + 1, &endp, 0);
+      int64_t param = strtoll(event_str.c_str() + 1, &endp, 0);
       req.restart().type = RESTART_FROM_CHECKPOINT;
       req.restart().param_str = event_str.substr(1);
       req.restart().param = param;
       LOG(debug) << "next replayer restarting from checkpoint "
-                 << req.restart().param;
+                 << param;
     } else if (event_str[0] == 't') {
-      int param = strtol(event_str.c_str() + 1, &endp, 0);
+      int64_t param = strtoll(event_str.c_str() + 1, &endp, 0);
       req.restart().type = RESTART_FROM_TICKS;
       req.restart().param_str = event_str.substr(1);
       req.restart().param = param;
       LOG(debug) << "next replayer restarting from tick count "
-                 << req.restart().param;
+                 << param;
     } else {
       req.restart().type = RESTART_FROM_EVENT;
-      req.restart().param = strtol(event_str.c_str(), &endp, 0);
+      req.restart().param = strtoll(event_str.c_str(), &endp, 0);
       LOG(debug) << "next replayer advancing to event " << req.restart().param;
     }
     if (!endp || *endp != '\0') {
