@@ -68,7 +68,8 @@ enum CpuMicroarch {
   IntelKabylake,
   IntelCometlake,
   IntelIcelake,
-  LastIntel = IntelIcelake,
+  IntelTigerlake,
+  LastIntel = IntelTigerlake,
   FirstAMD,
   AMDF15R30 = FirstAMD,
   AMDZen,
@@ -123,6 +124,7 @@ struct PmuConfig {
 // See Intel 64 and IA32 Architectures Performance Monitoring Events.
 // See check_events from libpfm4.
 static const PmuConfig pmu_configs[] = {
+  { IntelTigerlake, "Intel Tigerlake", 0x5111c4, 0, 0, 0, 100, PMU_TICKS_RCB },
   { IntelIcelake, "Intel Icelake", 0x5111c4, 0, 0, 0, 100, PMU_TICKS_RCB },
   { IntelCometlake, "Intel Cometlake", 0x5101c4, 0, 0x5301cb, 0, 100, PMU_TICKS_RCB },
   { IntelKabylake, "Intel Kabylake", 0x5101c4, 0, 0x5301cb, 0, 100, PMU_TICKS_RCB },
@@ -208,12 +210,12 @@ static ScopedFd start_counter(pid_t tid, int group_fd,
   }
   if (0 >= fd) {
     if (errno == EACCES) {
-      FATAL() << "Permission denied to use 'perf_event_open'; are perf events "
-                 "enabled? Try 'perf record'.";
+      CLEAN_FATAL() << "Permission denied to use 'perf_event_open'; are hardware perf events "
+                 "available? See https://github.com/rr-debugger/rr/wiki/Will-rr-work-on-my-system";
     }
     if (errno == ENOENT) {
-      FATAL() << "Unable to open performance counter with 'perf_event_open'; "
-                 "are perf events enabled? Try 'perf record'.";
+      CLEAN_FATAL() << "Unable to open performance counter with 'perf_event_open'; "
+                 "are hardware perf events available? See https://github.com/rr-debugger/rr/wiki/Will-rr-work-on-my-system";
     }
     FATAL() << "Failed to initialize counter";
   }
@@ -610,9 +612,21 @@ Ticks PerfCounters::read_ticks(Task* t) {
       uint64_t minus_measure_val = read_counter(fd_minus_ticks_measure);
       interrupt_val -= minus_measure_val;
     }
-    ASSERT(t, !counting_period || interrupt_val <= adjusted_counting_period)
-        << "Detected " << interrupt_val << " ticks, expected no more than "
-        << adjusted_counting_period;
+    if (t->session().is_recording()) {
+      if (counting_period && interrupt_val > adjusted_counting_period) {
+        LOG(warn) << "Recorded ticks of " << interrupt_val
+          << " overshot requested ticks target by " << interrupt_val - counting_period
+          << " ticks.\n"
+           "On AMD systems this is known to occur occasionally for unknown reasons.\n"
+           "Recording should continue normally. Please report any unexpected rr failures\n"
+           "received after this warning, any conditions that reliably reproduce it,\n"
+           "or sightings of this warning on non-AMD systems.";
+      }
+    } else {
+      ASSERT(t, !counting_period || interrupt_val <= adjusted_counting_period)
+          << "Detected " << interrupt_val << " ticks, expected no more than "
+          << adjusted_counting_period;
+    }
     return interrupt_val;
   }
 
