@@ -66,6 +66,9 @@ static CpuMicroarch compute_cpu_microarch() {
       return IntelGoldmont;
     case 0x706e0:
       return IntelIcelake;
+    case 0x806c0:
+    case 0x806d0:
+      return IntelTigerlake;
     case 0x806e0:
     case 0x906e0:
       return IntelKabylake;
@@ -84,8 +87,15 @@ static CpuMicroarch compute_cpu_microarch() {
     case 0x70f10: // Matisse (Zen 2) (UNTESTED)
       if (ext_family == 8) {
         return AMDZen;
+      } else if (ext_family == 3) {
+        return AMDF15R30;
       }
       break;
+    case 0x20f10: // Vermeer (Zen 3)
+    case 0x50f00: // Cezanne (Zen 3)
+      if (ext_family == 0xa) {
+        return AMDZen;
+      }
     default:
       break;
   }
@@ -268,7 +278,7 @@ static void check_for_zen_speclockmap() {
   // thread, but not more). When the optimization is disabled, the
   // perf counter for retired lock instructions of type SpecLockMapCommit
   // (on PMC 0x25) stays at 0.
-  // See more details at https://github.com/mozilla/rr/issues/2034.
+  // See more details at https://github.com/rr-debugger/rr/issues/2034.
   struct perf_event_attr attr;
   // 0x25 == RETIRED_LOCK_INSTRUCTIONS - Counts the number of retired locked instructions
   // + 0x08 == SPECLOCKMAPCOMMIT
@@ -287,7 +297,7 @@ static void check_for_zen_speclockmap() {
       fprintf(stderr,
               "On Zen CPUs, rr will not work reliably unless you disable the "
               "hardware SpecLockMap optimization.\nFor instructions on how to "
-              "do this, see https://github.com/mozilla/rr/wiki/Zen\n");
+              "do this, see https://github.com/rr-debugger/rr/wiki/Zen\n");
     }
   }
 }
