@@ -25,7 +25,7 @@ ProcMemMonitor::ProcMemMonitor(Task* t, const string& pathname) {
     if (!*end) {
       Task* target = t->session().find_task(tid);
       if (target) {
-        tuid = target->tuid();
+        auid = target->vm()->uid();
       }
     }
   }
@@ -40,7 +40,7 @@ void ProcMemMonitor::did_write(Task* t, const std::vector<Range>& ranges,
 
   // In prior versions of rr, we recorded this directly into the trace.
   // If so, there's nothing to do here.
-  if (t->session().is_replaying() && t->session().as_replay()->explicit_proc_mem()) {
+  if (t->session().is_replaying() && t->session().as_replay()->has_trace_quirk(TraceReader::ExplicitProcMem)) {
     return;
   }
 
@@ -49,19 +49,28 @@ void ProcMemMonitor::did_write(Task* t, const std::vector<Range>& ranges,
     return;
   }
 
-  auto* target = static_cast<ReplayTask*>(t->session().find_task(tuid));
+  auto* target = t->session().find_address_space(auid);
   if (!target) {
     return;
   }
 
+  ReplayTask* task = static_cast<ReplayTask*>(target->first_running_task());
+  if (!task) {
+    return;
+  }
+
   for (auto& r : ranges) {
     auto bytes = t->read_mem(r.data.cast<uint8_t>(), r.length);
     remote_ptr<uint8_t> target_addr = offset;
-    target->write_mem(target_addr, bytes.data(), r.length);
-    target->vm()->maybe_update_breakpoints(target, target_addr,
-                                           r.length);
+    task->write_mem(target_addr, bytes.data(), r.length);
+    target->maybe_update_breakpoints(task, target_addr,
+                                     r.length);
     offset += r.length;
   }
 }
 
+bool ProcMemMonitor::target_is_vm(AddressSpace *vm) {
+  return auid == vm->uid();
+}
+
 } // namespace rr
