@@ -81,11 +81,14 @@ RecordCommand RecordCommand::singleton(
     "                             tracee. There can be any number of these.\n"
     "  -w, --wait                 Wait for all child processes to exit, not\n"
     "                             just the initial process.\n"
-    "  --nested=ignore            Directly start child process when running\n"
-    "                             under nested rr recording, instead of\n"
-    "                             raising an error.\n"
-    "  --nested=detach            When nested, start a separate recording\n"
-    "                             session, instead of raising an error\n"
+    "  --nested=<value>           Control behavior when run inside an outer\n"
+    "                             rr recording. Default: exit with error\n"
+    "  --nested=ignore            Directly start child process so it's part\n"
+    "                             of the outer recording\n"
+    "  --nested=detach            Start a separate recording session.\n"
+    "                             Must not share memory with the outer.\n"
+    "  --nested=release           Run the child without recording it.\n"
+    "                             Must not share memory with the outer.\n"
     "  --setuid-sudo              If running under sudo, pretend to be the\n"
     "                             user that ran sudo rather than root. This\n"
     "                             allows recording setuid/setcap binaries.\n"
@@ -94,7 +97,7 @@ RecordCommand RecordCommand::singleton(
     "  --stap-sdt                 Enables the use of SystemTap statically-\n"
     "                             defined tracepoints\n"
     "  --asan                     Override heuristics and always enable ASAN\n"
-    "                             compatibility.");
+    "                             compatibility.\n");
 
 struct RecordFlags {
   vector<string> extra_env;
@@ -321,6 +324,8 @@ static bool parse_record_arg(vector<string>& args, RecordFlags& flags) {
         flags.nested = NESTED_IGNORE;
       } else if (opt.value == "detach") {
         flags.nested = NESTED_DETACH;
+      } else if (opt.value == "release") {
+        flags.nested = NESTED_RELEASE;
       } else {
         LOG(warn) << "Unknown nesting behavior `" << opt.value << "`";
         flags.nested = NESTED_ERROR;
@@ -518,12 +523,26 @@ static void handle_SIGTERM(__attribute__((unused)) int sig) {
   }
 }
 
+/**
+ * Something segfaulted - this is probably a bug in rr. Try to at least
+ * give a stacktrace.
+ */
+static void handle_SIGSEGV(__attribute__((unused)) int sig) {
+  static const char msg[] =
+    "rr itself crashed (SIGSEGV). This shouldn't happen!\n";
+  write_all(STDERR_FILENO, msg, sizeof(msg) - 1);
+  notifying_abort();
+}
+
 static void install_signal_handlers(void) {
   struct sigaction sa;
   memset(&sa, 0, sizeof(sa));
   sa.sa_handler = handle_SIGTERM;
   sigaction(SIGTERM, &sa, nullptr);
 
+  sa.sa_handler = handle_SIGSEGV;
+  sigaction(SIGSEGV, &sa, nullptr);
+
   sa.sa_handler = SIG_IGN;
   sigaction(SIGHUP, &sa, nullptr);
   sigaction(SIGINT, &sa, nullptr);
@@ -563,7 +582,7 @@ static RecordSession* static_session;
 // later.
 void force_close_record_session() {
   if (static_session) {
-    static_session->terminate_recording();
+    static_session->close_trace_writer(TraceWriter::CLOSE_ERROR);
   }
 }
 
@@ -615,7 +634,7 @@ static WaitStatus record(const vector<string>& args, const RecordFlags& flags) {
       flags.use_syscall_buffer, flags.syscallbuf_desched_sig,
       flags.bind_cpu, flags.output_trace_dir,
       flags.trace_id.get(),
-      flags.stap_sdt, flags.unmap_vdso);
+      flags.stap_sdt, flags.unmap_vdso, flags.asan);
   setup_session_from_flags(*session, flags);
 
   static_session = session.get();
@@ -637,6 +656,7 @@ static WaitStatus record(const vector<string>& args, const RecordFlags& flags) {
   install_signal_handlers();
 
   RecordSession::RecordResult step_result;
+  bool did_term_detached_tasks = false;
   do {
     bool done_initial_exec = session->done_initial_exec();
     step_result = session->record_step();
@@ -644,9 +664,16 @@ static WaitStatus record(const vector<string>& args, const RecordFlags& flags) {
     if (!done_initial_exec && session->done_initial_exec() && flags.output_trace_dir.empty()) {
       session->trace_writer().make_latest_trace();
     }
-  } while (step_result.status == RecordSession::STEP_CONTINUE && !term_requested);
+    if (term_requested) {
+      session->terminate_tracees();
+      if (!did_term_detached_tasks) {
+        session->term_detached_tasks();
+        did_term_detached_tasks = true;
+      }
+    }
+  } while (step_result.status == RecordSession::STEP_CONTINUE);
 
-  session->terminate_recording();
+  session->close_trace_writer(TraceWriter::CLOSE_OK);
   static_session = nullptr;
 
   switch (step_result.status) {
@@ -685,14 +712,6 @@ static void exec_child(vector<string>& args) {
   // Never returns!
 }
 
-static void reset_cpu_affinity() {
-  cpu_set_t mask;
-  memset(&mask, 0xFF, sizeof(mask));
-  sched_setaffinity(0, sizeof(mask), &mask);
-  // Best effort - if we fail, that's not yet fatal (though
-  // we might be running a bit slower)
-}
-
 static void reset_uid_sudo() {
   // Let's change our uids now. We do keep capabilities though, since that's
   // the point of the exercise. The first exec will reset both the keepcaps,
@@ -733,23 +752,31 @@ int RecordCommand::run(vector<string>& args) {
   }
 
   if (running_under_rr()) {
-    if (flags.nested == NESTED_IGNORE) {
-      exec_child(args);
-    } else if (flags.nested == NESTED_DETACH) {
-      int ret = syscall(SYS_rrcall_detach_teleport, 0, 0, 0, 0, 0, 0);
-      if (ret < 0) {
-        FATAL() << "Failed to detach from parent rr";
+    switch (flags.nested) {
+      case NESTED_IGNORE:
+        exec_child(args);
+        return 1;
+      case NESTED_DETACH:
+      case NESTED_RELEASE: {
+        int ret = syscall(SYS_rrcall_detach_teleport, (uintptr_t)0, (uintptr_t)0,
+          (uintptr_t)0, (uintptr_t)0, (uintptr_t)0, (uintptr_t)0);
+        if (ret < 0) {
+          FATAL() << "Failed to detach from parent rr";
+        }
+        if (running_under_rr(false)) {
+          FATAL() << "Detaching from parent rr did not work";
+        }
+        if (flags.nested == NESTED_RELEASE) {
+          exec_child(args);
+          return 1;
+        }
+        break;
       }
-      if (running_under_rr(false)) {
-        FATAL() << "Detaching from parent rr did not work";
-      }
-      reset_cpu_affinity();
-      // Fall through
-    } else {
-      fprintf(stderr, "rr: cannot run rr recording under rr. Exiting.\n"
-                      "Use `rr record --nested=ignore` to start the child "
-                      "process directly.\n");
-      return 1;
+      default:
+        fprintf(stderr, "rr: cannot run rr recording under rr. Exiting.\n"
+                        "Use `rr record --nested=ignore` to start the child "
+                        "process directly.\n");
+        return 1;
     }
   }
 
