@@ -119,10 +119,9 @@ public:
   RecordResult record_step();
 
   /**
-   * Flush buffers and write a termination record to the trace. Don't call
-   * record_step() after this.
+   * SIGKILL all tracees.
    */
-  void terminate_recording();
+  void terminate_tracees();
 
   /**
    * Close trace output without flushing syscall buffers or writing
@@ -152,7 +151,8 @@ public:
 
   void set_enable_chaos(bool enable_chaos) {
     scheduler().set_enable_chaos(enable_chaos);
-    this->enable_chaos_ = enable_chaos;
+    enable_chaos_ = enable_chaos;
+    trace_out.set_chaos_mode(enable_chaos);
   }
   bool enable_chaos() const { return enable_chaos_; }
 
@@ -189,10 +189,9 @@ public:
   virtual TraceStream* trace_stream() override { return &trace_out; }
 
   /**
-   * Like kill_all_tasks, but makes sure to record the exits of the task we
-   * killed.
+   * Send SIGTERM to all detached tasks and wait for them to finish.
    */
-  void kill_all_record_tasks();
+  void term_detached_tasks();
 
 private:
   RecordSession(const std::string& exe_path,
@@ -229,7 +228,6 @@ private:
   void desched_state_changed(RecordTask* t);
   bool prepare_to_inject_signal(RecordTask* t, StepState* step_state);
   void task_continue(const StepState& step_state);
-  bool can_end();
 
   TraceWriter trace_out;
   Scheduler scheduler_;
