@@ -183,7 +183,6 @@ RecordTask::RecordTask(RecordSession& session, pid_t _tid, uint32_t serial,
       delay_syscallbuf_reset_for_seccomp_trap(false),
       prctl_seccomp_status(0),
       robust_futex_list_len(0),
-      own_namespace_rec_tid(0),
       termination_signal(0),
       tsc_mode(PR_TSC_ENABLE),
       cpuid_mode(1),
@@ -197,7 +196,9 @@ RecordTask::RecordTask(RecordSession& session, pid_t _tid, uint32_t serial,
       waiting_for_reap(false),
       waiting_for_zombie(false),
       waiting_for_ptrace_exit(false),
-      retry_syscall_patching(false) {
+      retry_syscall_patching(false),
+      sent_shutdown_kill(false),
+      tick_request_override((TicksRequest)0) {
   push_event(Event::sentinel());
   if (session.tasks().empty()) {
     // Initial tracee. It inherited its state from this process, so set it up.
@@ -237,7 +238,7 @@ RecordTask::~RecordTask() {
          (is_exit_syscall(ev().Syscall().number, ev().Syscall().regs.arch()) ||
           is_exit_group_syscall(ev().Syscall().number,
                                 ev().Syscall().regs.arch()))))) {
-    LOG(warn) << tid << " still has pending events.  From top down:";
+    LOG(info) << tid << " still has pending events.  From top down:";
     log_pending_events();
   }
 
@@ -252,9 +253,7 @@ RecordTask::~RecordTask() {
   }
 }
 
-void RecordTask::record_exit_event(int fatal_signo) {
-  // Task::destroy has already done PTRACE_DETACH so the task can complete
-  // exiting.
+void RecordTask::record_exit_event(int fatal_signo, WriteChildTid write_child_tid) {
   // The kernel explicitly only clears the futex if the address space is shared.
   // If the address space has no other users then the futex will not be cleared
   // even if it lives in shared memory which other tasks can read.
@@ -266,14 +265,24 @@ void RecordTask::record_exit_event(int fatal_signo) {
      as->has_mapping(tid_futex)) {
     int val = 0;
     record_local(tid_futex, &val);
+    if (write_child_tid == WRITE_CHILD_TID) {
+      // Write the memory now, otherwise the kernel will write it later and that can
+      // race with the execution of other threads if we don't wait for this
+      // thread to fully exit.
+      // This could fail since the address space might have gone away/been switched
+      // by execve.
+      bool ok = true;
+      write_mem(tid_futex, 0, &ok);
+      // The kernel will do an unconditional futex wake on that location so we don't
+      // need to do it.
+    }
   }
 
   // Write the exit event here so that the value recorded above is captured.
   // Don't flush syscallbuf. Whatever triggered the exit (syscall, signal)
   // should already have flushed it, if it was running. If it was blocked,
-  // then the syscallbuf would already have been flushed too. The exception
-  // is kill_all_tasks() in which case it's OK to just drop the last chunk of
-  // execution. Trying to flush syscallbuf for an exiting task could be bad,
+  // then the syscallbuf would already have been flushed too. Trying to flush
+  // syscallbuf for an exiting task could be bad,
   // e.g. it could be in the middle of syscallbuf code that's supposed to be
   // atomic. For the same reasons don't allow syscallbuf to be reset here.
   record_event(Event::exit(), DONT_FLUSH_SYSCALLBUF, DONT_RESET_SYSCALLBUF);
@@ -333,16 +342,6 @@ void RecordTask::post_wait_clone(Task* cloned_from, int flags) {
   update_own_namespace_tid();
 }
 
-static string exe_path(RecordTask* t) {
-  char proc_exe[PATH_MAX];
-  snprintf(proc_exe, sizeof(proc_exe), "/proc/%d/exe", t->tid);
-  char exe[PATH_MAX];
-  ssize_t ret = readlink(proc_exe, exe, sizeof(exe) - 1);
-  ASSERT(t, ret >= 0);
-  exe[ret] = 0;
-  return exe;
-}
-
 void RecordTask::post_exec() {
   // Change syscall number to execve *for the new arch*. If we don't do this,
   // and the arch changes, then the syscall number for execve in the old arch/
@@ -355,8 +354,7 @@ void RecordTask::post_exec() {
   ev().Syscall().set_arch(arch());
 
   // The signal mask is inherited across execve so we don't need to invalidate.
-  string exe_file = exe_path(this);
-  Task::post_exec(exe_file);
+  Task::post_exec(this->exe_path());
   if (emulated_ptracer) {
     ASSERT(this, !(emulated_ptracer->arch() == x86 && arch() == x86_64))
         << "We don't support a 32-bit process tracing a 64-bit process";
@@ -514,6 +512,17 @@ void RecordTask::init_buffers() { RR_ARCH_FUNCTION(init_buffers_arch, arch()); }
 
 template <typename Arch>
 void RecordTask::on_syscall_exit_arch(int syscallno, const Registers& regs) {
+  switch (syscallno) {
+    // These syscalls affect the sigmask even if they fail.
+    case Arch::epoll_pwait:
+    case Arch::pselect6:
+    case Arch::pselect6_time64:
+    case Arch::ppoll:
+    case Arch::ppoll_time64:
+      invalidate_sigmask();
+      break;
+  }
+
   if (regs.original_syscallno() == SECCOMP_MAGIC_SKIP_ORIGINAL_SYSCALLNO ||
       regs.syscall_failed()) {
     return;
@@ -594,7 +603,7 @@ void RecordTask::will_resume_execution(ResumeRequest, WaitRequest,
     // have much choice, signal injection won't work if we block the signal.
     // We leave rr signals unblocked. TIME_SLICE_SIGNAL has to be unblocked
     // because blocking it seems to cause problems for some hardware/kernel
-    // configurations (see https://github.com/mozilla/rr/issues/1979),
+    // configurations (see https://github.com/rr-debugger/rr/issues/1979),
     // causing them to stop counting events.
     sig_set_t sigset = ~session().rr_signal_mask();
     if (sig) {
@@ -756,7 +765,7 @@ void RecordTask::force_emulate_ptrace_stop(WaitStatus status) {
   // return that result immediately.
 }
 
-bool RecordTask::do_ptrace_exit_stop(WaitStatus exit_status) {
+void RecordTask::do_ptrace_exit_stop(WaitStatus exit_status) {
   // Notify ptracer of the exit if it's not going to receive it from the
   // kernel because it's not the parent. (The kernel has similar logic to
   // deliver two stops in this case.)
@@ -767,17 +776,24 @@ bool RecordTask::do_ptrace_exit_stop(WaitStatus exit_status) {
     emulated_stop_type = NOT_STOPPED;
     // This is a bit wrong; this is an exit stop, not a signal/ptrace stop.
     emulate_ptrace_stop(exit_status);
-    return true;
   }
-  return false;
 }
 
 void RecordTask::did_reach_zombie() {
-  if (!already_reaped() && may_reap()) {
-    reap();
-  }
   waiting_for_zombie = false;
-  if (!waiting_for_reap) {
+  // Remove from address-space and fds list since we really aren't associated
+  // with them anymore (and we can't be used to operate on them)
+  as->erase_task(this);
+  fds->erase_task(this);
+
+  if (!already_reaped()) {
+    if (may_reap()) {
+      reap();
+    } else {
+      waiting_for_reap = true;
+    }
+  }
+  if ((already_reaped() || !waiting_for_reap) && !emulated_stop_pending) {
     delete this;
   }
 }
@@ -1201,7 +1217,13 @@ sig_set_t RecordTask::read_sigmask_from_process() {
   // to be restored, not the kernel's current (internal) sigmask, which is what
   // /proc/.../status reports. Always go with what /proc/.../status reports. See
   // https://github.com/torvalds/linux/commit/fcfc2aa0185f4a731d05a21e9f359968fdfd02e7
-  if (!at_may_restart_syscall()) {
+  // XXXkhuey and yet that's not what we actually do here ...
+  if (at_interrupted_non_restartable_signal_modifying_syscall()) {
+    // Mark the sigmask as already invalid. The moment we exit the kernel and run more
+    // of the tracee the sigmask will change, so we need to keep refetching the
+    // sigmask until that happens.
+    invalidate_sigmask();
+  } else if (!at_may_restart_syscall()) {
     sig_set_t mask;
     long ret = fallible_ptrace(PTRACE_GETSIGMASK,
                                remote_ptr<void>(sizeof(sig_set_t)), &mask);
@@ -1217,9 +1239,10 @@ sig_set_t RecordTask::read_sigmask_from_process() {
 
 sig_set_t RecordTask::get_sigmask() {
   if (blocked_sigs_dirty) {
+    // Clear this first, read_sigmask_from_process might set it again.
+    blocked_sigs_dirty = false;
     blocked_sigs = read_sigmask_from_process();
     LOG(debug) << "Refreshed sigmask, now " << HEX(blocked_sigs);
-    blocked_sigs_dirty = false;
   }
   return blocked_sigs;
 }
@@ -1518,6 +1541,13 @@ bool RecordTask::at_may_restart_syscall() const {
           EV_SYSCALL_INTERRUPTION == prev_ev->type());
 }
 
+bool RecordTask::at_interrupted_non_restartable_signal_modifying_syscall() const {
+  auto r = regs();
+  // XXXkhuey io_uring_enter (not yet supported) can do this too.
+  return r.syscall_result_signed() == -EINTR &&
+    is_epoll_pwait_syscall(r.original_syscallno(), arch());
+}
+
 bool RecordTask::is_arm_desched_event_syscall() {
   return is_desched_event_syscall() && PERF_EVENT_IOC_ENABLE == regs().arg2();
 }
@@ -1557,13 +1587,16 @@ bool RecordTask::running_inside_desched() const {
   return false;
 }
 
-uint16_t RecordTask::get_ptrace_eventmsg_seccomp_data() {
+int RecordTask::get_ptrace_eventmsg_seccomp_data() {
   unsigned long data = 0;
   // in theory we could hit an assertion failure if the tracee suffers
   // a SIGKILL before we get here. But the SIGKILL would have to be
   // precisely timed between the generation of a PTRACE_EVENT_FORK/CLONE/
   // SYS_clone event, and us fetching the event message here.
-  xptrace(PTRACE_GETEVENTMSG, nullptr, &data);
+  if (fallible_ptrace(PTRACE_GETEVENTMSG, nullptr, &data) < 0) {
+    ASSERT(this, errno == ESRCH);
+    return -1;
+  }
   return data;
 }
 
@@ -1627,22 +1660,42 @@ void RecordTask::record_remote_writable(remote_ptr<void> addr,
 }
 
 ssize_t RecordTask::record_remote_fallible(remote_ptr<void> addr,
-                                          ssize_t num_bytes) {
-  ASSERT(this, num_bytes >= 0);
-
-  if (record_remote_by_local_map(addr, num_bytes)) {
-    return num_bytes;
-  }
-
+                                           uintptr_t num_bytes,
+                                           const std::vector<WriteHole>& holes) {
+  auto hole_iter = holes.begin();
+  uintptr_t offset = 0;
   vector<uint8_t> buf;
-  ssize_t nread = 0;
-  if (!addr.is_null()) {
-    buf.resize(num_bytes);
-    nread = read_bytes_fallible(addr, num_bytes, buf.data());
-    buf.resize(max<ssize_t>(0, nread));
+  while (offset < num_bytes) {
+    if (hole_iter != holes.end() && hole_iter->offset == offset) {
+      offset += hole_iter->size;
+      ++hole_iter;
+      continue;
+    }
+
+    uintptr_t bytes = min(uintptr_t(4*1024*1024), num_bytes - offset);
+    if (hole_iter != holes.end()) {
+      ASSERT(this, hole_iter->offset > offset);
+      bytes = min(bytes, uintptr_t(hole_iter->offset) - offset);
+    }
+    if (record_remote_by_local_map(addr + offset, bytes)) {
+      offset += bytes;
+      continue;
+    }
+
+    if (addr) {
+      buf.resize(bytes);
+      ssize_t nread = read_bytes_fallible(addr + offset, bytes, buf.data());
+      if (nread <= 0) {
+        break;
+      }
+      trace_writer().write_raw_data(buf.data(), nread);
+      offset += nread;
+    } else {
+      offset += bytes;
+    }
   }
-  trace_writer().write_raw(rec_tid, buf.data(), buf.size(), addr);
-  return nread;
+  trace_writer().write_raw_header(rec_tid, offset, addr, holes);
+  return offset;
 }
 
 void RecordTask::record_remote_even_if_null(remote_ptr<void> addr,
@@ -1681,7 +1734,7 @@ void RecordTask::log_pending_events() const {
   /* The event at depth 0 is the placeholder event, which isn't
    * useful to log.  Skip it. */
   for (auto it = pending_events.rbegin(); it != pending_events.rend(); ++it) {
-    it->log();
+    LOG(info) << *it;
   }
 }
 
@@ -1923,21 +1976,153 @@ void RecordTask::set_tid_and_update_serial(pid_t tid,
   own_namespace_rec_tid = own_namespace_tid;
 }
 
-bool RecordTask::waiting_for_pid_namespace_tasks_to_exit() const {
-  if (tg->tgid_own_namespace != 1 || thread_group()->task_set().size() > 1) {
+bool RecordTask::may_reap() {
+  if (emulated_stop_pending) {
+    LOG(debug) << "Declining to reap " << tid << "; emulated stop pending";
+    // Don't reap until the emulated ptrace stop has been processed.
     return false;
   }
-  // This is the last thread of pid-1 for its namespace. See if there
-  // are any other tasks in the pid namespace. We can just check if there
-  // are any threadgroup children of our threadgroup.
+  // Non thread-group-leaders may always be reaped
+  if (tid != real_tgid()) {
+    return true;
+  }
+  for (auto it : thread_group()->task_set()) {
+    if (&*it != this) {
+      LOG(debug) << "Declining to reap " << tid << "; leader of non-empty thread-group with active thread " << it->tid;
+      return false;
+    }
+  }
+  return true;
+}
+
+void RecordTask::reap() {
+  ASSERT(this, !was_reaped);
+  LOG(debug) << "Reaping " << tid;
+  siginfo_t info;
+  memset(&info, 0, sizeof(info));
+  int ret = waitid(P_PID, tid, &info, WEXITED | WNOHANG);
+  if (ret != 0) {
+    FATAL() << "Unexpected wait status for tid " << tid;
+  }
+  /* The sid_pid == 0 case here is the same as the case below where we're the
+   * group leader whose pid gets stolen.
+   */
+  DEBUG_ASSERT(info.si_pid == tid ||
+               info.si_pid == 0);
+  was_reaped = true;
+}
+
+bool RecordTask::try_wait() {
+  if (wait_unexpected_exit()) {
+    return true;
+  }
+
+  // Check if there is a status change for us
+  WaitStatus status;
+  siginfo_t info;
+  memset(&info, 0, sizeof(siginfo_t));
+  int ret = waitid(P_PID, tid, &info, WSTOPPED | WNOHANG);
+  ASSERT(this, 0 == ret || (-1 == ret && errno == ECHILD)) <<
+    "waitid(" << tid << ", WSTOPPED | NOHANG) failed with "
+                         << ret;
+  LOG(debug) << "waitid(" << tid << ", NOHANG) returns " << ret;
+  if (ret == 0 && info.si_pid == 0) {
+    return false;
+  }
+  if (ret == 0) {
+    status = WaitStatus(info);
+  } else if (ret == -1) {
+    ASSERT(this, errno == ECHILD);
+    // Either we died/are dying unexpectedly, or we were in exec and changed the tid.
+    // Try to differentiate the two situations by seeing if there is an exit
+    // notification ready for us to de-queue, in which case we synthesize an
+    // exit event (but don't actually reap the task, instead leaving that
+    // for the generic cleanup code).
+    int ret = waitid(P_PID, tid, &info, WEXITED | WNOWAIT | WNOHANG);
+    if (ret == 0) {
+      if (info.si_pid == tid) {
+        LOG(debug) << "Synthesizing PTRACE_EVENT_EXIT for zombie process in try_wait " << tid;
+        status = WaitStatus::for_ptrace_event(PTRACE_EVENT_EXIT);
+      } else {
+        // This can happen when the task is in zap_pid_ns_processes waiting for all tasks
+        // in the pid-namespace to exit. It's not in a signal stop, but it's also not
+        // ready to be reaped yet, yet we're still tracing it. Don't wait on this
+        // task, we should be able to reap it later.
+        ASSERT(this, info.si_pid == 0);
+        return false;
+      }
+    } else {
+      ASSERT(this, ret == -1 && errno == ECHILD) << "waitpid failed with " << ret;
+      return false;
+    }
+  }
+  did_waitpid(status);
+  return true;
+}
+
+static uint64_t read_pid_ns(const RecordTask* t) {
+  char buf[PATH_MAX];
+  sprintf(buf, "/proc/%d/ns/pid", t->tid);
+  char link[PATH_MAX];
+  int ret = readlink(buf, link, sizeof(link));
+  ASSERT(t, ret >= 0);
+  ASSERT(t, ret < (int)sizeof(link));
+  link[ret] = 0;
+  ASSERT(t, strncmp(link, "pid:[", 5) == 0);
+  char* end;
+  uint64_t result = strtoul(link + 5, &end, 10);
+  ASSERT(t, strcmp(end, "]") == 0);
+  return result;
+}
+
+bool RecordTask::waiting_for_pid_namespace_tasks_to_exit() const {
+  if (tg->tgid_own_namespace != 1) {
+    return false;
+  }
+  // This might be the last live thread for pid-1 in the pid namespace.
+  // Checking that it *is* the last live thread is tricky because other
+  // threads could unexpectedly die asynchronously :-(.
+  // See if there are any other tasks in the pid namespace.
+  // Note that due to setns there can be tasks in the pid namespace
+  // with parents outside the pid namespace other than our thread-group.
+
+  // If there are multiple threads in our threadgroup, they're in our
+  // pid namespace.
+  if (thread_group()->task_set().size() > 1) {
+    return true;
+  }
+  // If we have any child processes then those belong to our pid namespace
+  // (or a descendant).
   for (auto p : session().thread_group_map()) {
     if (p.second->parent() == tg.get()) {
       return true;
     }
   }
+  // If there are any other tasks in the pid namespace at least one must be
+  // directly in the namespace.
+  uint64_t pid_ns = read_pid_ns(this);
+  for (auto it : session().tasks()) {
+    auto rt = static_cast<RecordTask*>(it.second);
+    if (rt == this) {
+      continue;
+    }
+    if (read_pid_ns(rt) == pid_ns) {
+      return true;
+    }
+  }
   return false;
 }
 
+int RecordTask::process_depth() const {
+  int depth = 0;
+  ThreadGroup* tg = this->tg.get();
+  while (tg) {
+    ++depth;
+    tg = tg->parent();
+  }
+  return depth;
+}
+
 template <typename Arch>
 static void maybe_restore_original_syscall_registers_arch(RecordTask* t,
                                                           void* local_addr) {
