@@ -5,7 +5,6 @@
 #include "ReplaySession.h"
 
 #include <linux/futex.h>
-#include <sys/prctl.h>
 #include <syscall.h>
 
 #include <algorithm>
@@ -163,9 +162,22 @@ ReplaySession::ReplaySession(const std::string& dir, const Flags& flags)
       ticks_at_start_of_event(0),
       flags_(flags),
       trace_start_time(0) {
+  if (trace_in.required_forward_compatibility_version() > FORWARD_COMPATIBILITY_VERSION) {
+    CLEAN_FATAL()
+      << "This rr build is too old to replay the trace (we support forward compability version "
+      << FORWARD_COMPATIBILITY_VERSION << " but the trace needs " << trace_in.required_forward_compatibility_version();
+  }
+
   ticks_semantics_ = trace_in.ticks_semantics();
   rrcall_base_ = trace_in.rrcall_base();
 
+  if (!flags.redirect_stdio_file.empty()) {
+    tracee_output_fd_ = make_shared<ScopedFd>(flags.redirect_stdio_file.c_str(), O_CREAT | O_TRUNC | O_WRONLY, 0600);
+    if (!tracee_output_fd_->is_open()) {
+      FATAL() << "Can't open/create tracee output file " << flags.redirect_stdio_file;
+    }
+  }
+
   memset(&last_siginfo_, 0, sizeof(last_siginfo_));
   advance_to_next_trace_frame();
 
@@ -196,6 +208,7 @@ ReplaySession::ReplaySession(const std::string& dir, const Flags& flags)
 ReplaySession::ReplaySession(const ReplaySession& other)
     : Session(other),
       emu_fs(EmuFs::create()),
+      tracee_output_fd_(other.tracee_output_fd_),
       trace_in(other.trace_in),
       trace_frame(other.trace_frame),
       current_step(other.current_step),
@@ -422,6 +435,14 @@ bool ReplaySession::handle_unrecorded_cpuid_fault(
   Registers r = t->regs();
   const CPUIDRecord* rec = find_cpuid_record(records, r.ax(), r.cx());
   if (rec) {
+    if (rec->ecx_in == UINT32_MAX &&
+        (r.ax() == CPUID_AMD_CACHE_TOPOLOGY || r.ax() == CPUID_AMD_PLATFORM_QOS)) {
+      const CPUIDRecord* rec2 = find_cpuid_record(records, CPUID_GETVENDORSTRING, 0);
+      if (!rec2 || is_cpu_vendor_amd(rec2->out)) {
+        LOG(error) << "Can't find extended AMD CPUID records. Replay will likely fail. " <<
+          "Please re-record the trace with an up-to-date version of rr.";
+      }
+    }
     r.set_cpuid_output(rec->out.eax, rec->out.ebx, rec->out.ecx, rec->out.edx);
   } else {
     LOG(warn) << "Can't find CPUID record for request AX=" << HEX(r.ax())
@@ -598,7 +619,7 @@ Completion ReplaySession::enter_syscall(ReplayTask* t,
       // If we use the breakpoint optimization, we must get a SIGTRAP before
       // reaching a syscall, so cont_syscall_boundary must return INCOMPLETE.
       if (syscall_bp_vm) {
-        FATAL()
+        ASSERT(t, false)
             << "Expected syscall_bp_vm to be clear but it's " << syscall_bp_vm->leader_tid()
             << "'s address space with a breakpoint at " << syscall_bp_addr
             << " while we're at " << t->ip();
@@ -760,7 +781,8 @@ static void guard_unexpected_signal(ReplayTask* t) {
 
   if (t->stop_sig()) {
     ASSERT(t, false) << "Replay got unrecorded signal "
-                     << signal_name(t->stop_sig()) << " while awaiting signal";
+                     << signal_name(t->stop_sig()) << " while awaiting signal"
+                     << "\n" << t->get_siginfo();
   } else if (t->status().is_syscall()) {
     ASSERT(t, false) << "Replay got unrecorded syscall "
                      << syscall_name(t->regs().original_syscallno(), t->arch())
@@ -1085,8 +1107,15 @@ void ReplaySession::check_ticks_consistency(ReplayTask* t, const Event& ev) {
 }
 
 static bool treat_signal_event_as_deterministic(const SignalEvent& ev) {
-  return ev.deterministic == DETERMINISTIC_SIG &&
-    ev.siginfo.si_signo != SIGBUS;
+  if (ev.siginfo.si_signo == SIGBUS) {
+    return false;
+  }
+  if (ev.siginfo.si_signo == SIGSEGV && ev.siginfo.si_code == SEGV_PKUERR) {
+    // We don't set up memory protection key state, so pkey-triggered signals
+    // won't happen.
+    return false;
+  }
+  return ev.deterministic == DETERMINISTIC_SIG;
 }
 
 /**
@@ -1136,7 +1165,8 @@ Completion ReplaySession::emulate_deterministic_signal(
   }
   ASSERT(t, t->stop_sig() == sig)
       << "Replay got unrecorded signal " << signal_name(t->stop_sig())
-      << " (expecting " << signal_name(sig) << ")";
+      << " (expecting " << signal_name(sig) << ")"
+      << "\n" << t->get_siginfo();
   check_ticks_consistency(t, ev);
 
   if (EV_INSTRUCTION_TRAP == ev.type()) {
@@ -1304,7 +1334,8 @@ Completion ReplaySession::flush_syscallbuf(ReplayTask* t,
 
   Registers r = t->regs();
   ASSERT(t, t->stop_sig() == SIGSEGV && r.ip() == t->vm()->do_breakpoint_fault_addr())
-      << "Replay got unexpected signal (or none) " << t->stop_sig();
+      << "Replay got unexpected signal (or none) " << t->stop_sig()
+      << " ip " << r.ip() << " breakpoint_fault_addr " << t->vm()->do_breakpoint_fault_addr();
   r.set_ip(r.ip().increment_by_movrm_insn_length(t->arch()));
   t->set_regs(r);
 
@@ -1322,6 +1353,10 @@ Completion ReplaySession::patch_vsyscall(ReplayTask* t, const StepConstraints& c
   bool added = t->vm()->add_breakpoint(vsyscall_entry, BKPT_INTERNAL);
   ASSERT(t, added);
   auto complete = continue_or_step(t, constraints, ticks_request, RESUME_CONT);
+  TrapReasons reasons;
+  if (complete == COMPLETE && t->stop_sig() == SIGTRAP) {
+    reasons = t->compute_trap_reasons();
+  }
   t->vm()->remove_breakpoint(vsyscall_entry, BKPT_INTERNAL);
 
   if (complete == INCOMPLETE) {
@@ -1336,7 +1371,9 @@ Completion ReplaySession::patch_vsyscall(ReplayTask* t, const StepConstraints& c
 
   ASSERT(t, t->stop_sig() == SIGTRAP)
       << "Replay got unexpected signal (or none) " << t->stop_sig();
-  ASSERT(t, t->regs().ip().undo_executed_bkpt(t->arch()) == vsyscall_entry);
+  if (!reasons.breakpoint || t->ip().undo_executed_bkpt(t->arch()) != vsyscall_entry) {
+    return INCOMPLETE;
+  }
 
   t->apply_all_data_records_from_trace();
   Registers r = t->regs();
@@ -1494,8 +1531,14 @@ static void end_task(ReplayTask* t) {
   t->set_regs(r);
   // Enter the syscall.
   t->resume_execution(RESUME_CONT, RESUME_WAIT, RESUME_NO_TICKS);
-  ASSERT(t, t->ptrace_event() == PTRACE_EVENT_EXIT);
-  t->did_handle_ptrace_exit_event();
+  if (t->session().done_initial_exec()) {
+    ASSERT(t, t->ptrace_event() == PTRACE_EVENT_EXIT);
+    t->did_handle_ptrace_exit_event();
+  } else {
+    // If we never execed, the trace is totally hosed,
+    // just clean up.
+    t->did_kill();
+  }
   t->detach();
   delete t;
 }
@@ -1711,7 +1754,7 @@ ReplayResult ReplaySession::replay_step(const StepConstraints& constraints) {
       advance_to_next_trace_frame();
     }
     if (current_step.action == TSTEP_EXIT_TASK) {
-      result.break_status.task = t;
+      result.break_status.task_context = TaskContext(t);
       result.break_status.task_exit = true;
     }
     return result;
@@ -1720,7 +1763,7 @@ ReplayResult ReplaySession::replay_step(const StepConstraints& constraints) {
   fast_forward_status = FastForwardStatus();
 
   // Now we know |t| hasn't died, so save it in break_status.
-  result.break_status.task = t;
+  result.break_status.task_context = TaskContext(t);
 
   /* Advance towards fulfilling |current_step|. */
   if (try_one_trace_step(t, constraints) == INCOMPLETE) {
@@ -1754,7 +1797,11 @@ ReplayResult ReplaySession::replay_step(const StepConstraints& constraints) {
 
   // If try_one_trace_step set extra-registers already, the values it used from the frame
   // will already have FIP/FDP cleared if necessary. Clearing them again here is fine.
-  if (trace_reader().clear_fip_fdp()) {
+  if (trace_reader().clear_fip_fdp() &&
+      current_step.action != TSTEP_EXIT_TASK)
+      /* TSTEP_EXIT_TASK means the task object got already
+         deleted above in try_one_trace_step/exit_task/end_task. */
+  {
     const ExtraRegisters* maybe_extra = t->extra_regs_fallible();
     if (maybe_extra) {
       ExtraRegisters extra_registers = *maybe_extra;
@@ -1787,7 +1834,7 @@ ReplayResult ReplaySession::replay_step(const StepConstraints& constraints) {
       }
       break;
     case TSTEP_EXIT_TASK:
-      result.break_status.task = nullptr;
+      result.break_status.task_context = TaskContext();
       t = nullptr;
       DEBUG_ASSERT(!result.break_status.any_break());
       break;
