@@ -159,7 +159,10 @@ public:
    * at the beginning of this event.
    */
   void set_reverse_execution_barrier_event(FrameTime event) {
-    reverse_execution_barrier_event = event;
+    reverse_execution_barrier_event_ = event;
+  }
+  FrameTime reverse_execution_barrier_event() {
+    return reverse_execution_barrier_event_;
   }
 
   // State-changing APIs. These may alter state associated with
@@ -173,6 +176,11 @@ public:
     return seek_to_before_key(MarkKey(time, 0, ReplayStepKey()));
   }
 
+  /**
+   * Seek the timeline to just before tick count `ticks` during event `time`.
+   */
+  void seek_to_ticks(FrameTime time, Ticks ticks);
+
   /**
    * Reset the current session to the last checkpointed session before (or at)
    * the mark. Will return at the mark if this mark was explicitly checkpointed
@@ -498,7 +506,7 @@ private:
       watchpoints;
   bool breakpoints_applied;
 
-  FrameTime reverse_execution_barrier_event;
+  FrameTime reverse_execution_barrier_event_;
 
   /**
    * Checkpoints used to accelerate reverse execution.
