@@ -4,7 +4,6 @@
 
 #include <linux/limits.h>
 #include <linux/unistd.h>
-#include <sys/prctl.h>
 #include <syscall.h>
 #include <sys/wait.h>
 
@@ -267,7 +266,7 @@ string Session::read_spawned_task_error() const {
 BreakStatus Session::diagnose_debugger_trap(Task* t, RunCommand run_command) {
   assert_fully_initialized();
   BreakStatus break_status;
-  break_status.task = t;
+  break_status.task_context = TaskContext(t);
 
   int stop_sig = t->stop_sig();
   if (!stop_sig) {
@@ -320,12 +319,12 @@ BreakStatus Session::diagnose_debugger_trap(Task* t, RunCommand run_command) {
       BreakpointType retired_bp =
           t->vm()->get_breakpoint_type_for_retired_insn(t->ip());
       if (BKPT_USER == retired_bp) {
-        LOG(debug) << "hit debugger breakpoint at ip " << t->ip();
         // SW breakpoint: $ip is just past the
         // breakpoint instruction.  Move $ip back
         // right before it.
         t->move_ip_before_breakpoint();
         break_status.breakpoint_hit = true;
+        LOG(debug) << "hit debugger breakpoint at ip " << t->ip();
       }
     }
   }
