@@ -23,8 +23,11 @@ using namespace std;
 
 namespace rr {
 
+const char* DEBUGLINK = "debuglink";
+const char* DEBUGALTLINK = "debugaltlink";
+
 /// Prints JSON containing
-/// "relevant_binaries": an array of strings, trace-relative binary file names.
+/// "relevant_binaries": an array of strings, trace-relative binary file names (or build-ids, for explicit-sources).
 ///   These are ELF files in the trace that our collected data is relevant to.
 /// "external_debug_info": an array of objects, {"path":<path>, "build_id":<build-id>, "type":<type>}
 ///   These are ELF files in the filesystem that contain separate debuginfo. "build-id" is the
@@ -41,6 +44,8 @@ namespace rr {
 ///   These symlinks that exist in the filesystem that are relevant to the source file paths.
 /// "files": a map from VCS directory name to array of source files relative to that directory
 ///   An empty VCS directory name means files not under any VCS.
+/// "comp_dir_substitutions": a map from trace-relative binary file names (or build-ids, for explicit-sources) to
+/// the compilation-dir-override.
 class SourcesCommand : public Command {
 public:
   virtual int run(vector<string>& args) override;
@@ -57,7 +62,9 @@ SourcesCommand SourcesCommand::singleton(
     "  --substitute=LIBRARY=PATH  When searching for the source to LIBRARY,\n"
     "                             substitute PATH in place of the path stored\n"
     "                             in the library's DW_AT_comp_dir property\n"
-    "                             for all compilation units.\n");
+    "                             for all compilation units.\n"
+    "                             LIBRARY is the basename of the original file name,\n"
+    "                             e.g. libc-2.32.so\n");
 
 class ExplicitSourcesCommand : public Command {
 public:
@@ -77,14 +84,18 @@ ExplicitSourcesCommand ExplicitSourcesCommand::singleton(
     "  --substitute=LIBRARY=PATH  When searching for the source to LIBRARY,\n"
     "                             substitute PATH in place of the path stored\n"
     "                             in the library's DW_AT_comp_dir property\n"
-    "                             for all compilation units.\n");
+    "                             for all compilation units.\n"
+    "                             LIBRARY is the basename of the original file name,\n"
+    "                             e.g. libc-2.32.so\n");
 
-static void parent_dir(string& s) {
+static void dir_name(string& s) {
   size_t p = s.rfind('/');
-  if (p == string::npos) {
+  if (p == string::npos || (p == 0 && s.size() == 1)) {
     s.clear();
   } else if (p > 0) {
     s.resize(p);
+  } else {
+    s.resize(1);
   }
 }
 
@@ -95,28 +106,91 @@ static void base_name(string& s) {
   }
 }
 
+static bool is_absolute(string& s) {
+  return s[0] == '/';
+}
+
+static void prepend_path(const char* prefix, string& s) {
+  size_t len = strlen(prefix);
+  if (!len) {
+    return;
+  }
+  if (prefix[len - 1] == '/') {
+    s = string(prefix) + s;
+  } else {
+    s = string(prefix) + '/' + s;
+  }
+}
+
+struct DirExistsCache {
+  unordered_map<string, bool> cache;
+  bool dir_exists(const string& dir) {
+    auto it = cache.find(dir);
+    if (it != cache.end()) {
+      return it->second;
+    }
+    bool exists = access(dir.c_str(), F_OK) == 0;
+    cache.insert(make_pair(dir, exists));
+    return exists;
+  }
+};
+
+// Resolve a file name relative to a compilation directory and relative directory.
 // file_name cannot be null, but the others can be.
-static void resolve_file_name(const char* original_file_dir,
-                              const char* comp_dir, const char* rel_dir,
-                              const char* file_name, set<string>* file_names) {
-  const char* names[] = { original_file_dir, comp_dir, rel_dir, file_name };
-  ssize_t first_absolute = -1;
-  for (ssize_t i = 0; i < 4; ++i) {
-    if (names[i] && names[i][0] == '/') {
-      first_absolute = i;
+// Takes into accout the original file name as follows:
+// -- if comp_dir, rel_dir or file_name are absolute, or original_file_name is NULL,
+// then ignore original_file_name.
+// The result is just the result of combining comp_dir/rel_dir/file_name.
+// -- otherwise they're all relative to some build directory. We hypothesize
+// the build directory is some ancestor directory of original_file_name.
+// We try making comp_dir/rel_dir/file_name relative to each ancestor directory
+// of original_file_name, and if we find a file there, we return that name.
+//
+// If non-empty, `comp_dir_substution` should replace `original_comp_dir`
+// in `rel_dir` if `original_comp_dir` is a prefix of `rel_dir`.
+static string resolve_file_name(const char* original_file_name,
+                                const char* comp_dir,
+                                const char* original_comp_dir,
+                                const string& comp_dir_substitution,
+                                const char* rel_dir,
+                                const char* file_name,
+                                DirExistsCache& dir_exists_cache) {
+  string path = file_name;
+  if (is_absolute(path)) {
+    return path;
+  }
+  if (rel_dir) {
+    if (rel_dir[0] == '/' && !comp_dir_substitution.empty() && original_comp_dir &&
+        strncmp(rel_dir, original_comp_dir, strlen(original_comp_dir)) == 0) {
+      string rel = comp_dir_substitution + (rel_dir + strlen(original_comp_dir));
+      prepend_path(rel.c_str(), path);
+    } else {
+      prepend_path(rel_dir, path);
+    }
+    if (is_absolute(path)) {
+      return path;
     }
   }
-  string s = first_absolute >= 0 ? "" : "/";
-  for (size_t i = (first_absolute >= 0 ? first_absolute : 0); i < 4; ++i) {
-    if (!names[i]) {
-      continue;
+  if (comp_dir) {
+    prepend_path(comp_dir, path);
+    if (is_absolute(path)) {
+      return path;
+    }
+  }
+  if (!original_file_name) {
+    return path;
+  }
+  string original(original_file_name);
+  while (true) {
+    dir_name(original);
+    if (original.empty()) {
+      return path;
+    }
+    string candidate = original + "/" + path;
+    if (dir_exists_cache.dir_exists(candidate)) {
+      return candidate;
     }
-    if (!s.empty() && s.back() != '/') {
-      s.push_back('/');
-    }
-    s += names[i];
   }
-  file_names->insert(move(s));
 }
 
 struct DwoInfo {
@@ -124,34 +198,37 @@ struct DwoInfo {
   string trace_file;
   // Could be an empty string
   string comp_dir;
+  string full_path;
   uint64_t id;
 };
 
 static bool process_compilation_units(ElfFileReader& reader,
+                                      ElfFileReader* sup_reader,
                                       const string& trace_relative_name,
                                       const string& original_file_name,
                                       const string& comp_dir_substitution,
-                                      set<string>* file_names, vector<DwoInfo>* dwos) {
+                                      set<string>* file_names, vector<DwoInfo>* dwos,
+                                      DirExistsCache& dir_exists_cache) {
   DwarfSpan debug_info = reader.dwarf_section(".debug_info");
   DwarfSpan debug_abbrev = reader.dwarf_section(".debug_abbrev");
   DwarfSpan debug_str = reader.dwarf_section(".debug_str");
+  DwarfSpan debug_str_sup = sup_reader ? sup_reader->dwarf_section(".debug_str") : DwarfSpan();
   DwarfSpan debug_str_offsets = reader.dwarf_section(".debug_str_offsets");
   DwarfSpan debug_line = reader.dwarf_section(".debug_line");
   DwarfSpan debug_line_str = reader.dwarf_section(".debug_line_str");
   if (debug_info.empty() || debug_abbrev.empty() ||
-      debug_str.empty() || debug_line.empty())  {
+      (debug_str.empty() && debug_str_sup.empty()) ||
+      debug_line.empty())  {
     return false;
   }
 
   DebugStrSpans debug_strs = {
     debug_str,
+    debug_str_sup,
     debug_str_offsets,
     debug_line_str,
   };
 
-  string original_file_dir = original_file_name;
-  parent_dir(original_file_dir);
-
   DwarfAbbrevs abbrevs(debug_abbrev);
   do {
     bool ok = true;
@@ -168,11 +245,12 @@ static bool process_compilation_units(ElfFileReader& reader,
     } else {
       cu.set_str_offsets_base(0);
     }
+    const char* original_comp_dir = cu.die().string_attr(cu, DW_AT_comp_dir, debug_strs, &ok);;
     const char* comp_dir;
     if (!comp_dir_substitution.empty()) {
       comp_dir = comp_dir_substitution.c_str();
     } else {
-      comp_dir = cu.die().string_attr(cu, DW_AT_comp_dir, debug_strs, &ok);
+      comp_dir = original_comp_dir;
       if (!ok) {
         continue;
       }
@@ -197,11 +275,12 @@ static bool process_compilation_units(ElfFileReader& reader,
         }
       }
       if (has_dwo_id) {
+        string full_name = resolve_file_name(original_file_name.c_str(), comp_dir, original_comp_dir, comp_dir_substitution, nullptr, dwo_name, dir_exists_cache);
         string c;
         if (comp_dir) {
           c = comp_dir;
         }
-        dwos->push_back({ dwo_name, trace_relative_name, move(c), dwo_id });
+        dwos->push_back({ dwo_name, trace_relative_name, move(c), full_name, dwo_id });
       } else {
         LOG(warn) << "DW_AT_GNU_dwo_name but not DW_AT_GNU_dwo_id";
       }
@@ -211,7 +290,7 @@ static bool process_compilation_units(ElfFileReader& reader,
       continue;
     }
     if (source_file_name) {
-      resolve_file_name(original_file_dir.c_str(), comp_dir, nullptr, source_file_name, file_names);
+      file_names->insert(resolve_file_name(original_file_name.c_str(), comp_dir, original_comp_dir, comp_dir_substitution, nullptr, source_file_name, dir_exists_cache));
     }
     intptr_t stmt_list = cu.die().section_ptr_attr(DW_AT_stmt_list, &ok);
     if (stmt_list < 0 || !ok) {
@@ -227,7 +306,7 @@ static bool process_compilation_units(ElfFileReader& reader,
         continue;
       }
       const char* dir = lines.directories()[f.directory_index];
-      resolve_file_name(original_file_dir.c_str(), comp_dir, dir, f.file_name, file_names);
+      file_names->insert(resolve_file_name(original_file_name.c_str(), comp_dir, original_comp_dir, comp_dir_substitution, dir, f.file_name, dir_exists_cache));
     }
   } while (!debug_info.empty());
 
@@ -242,61 +321,212 @@ struct ExternalDebugInfo {
     if (path < other.path) {
       return true;
     }
-    if (path == other.path) {
+    if (path > other.path) {
       return false;
     }
     if (build_id < other.build_id) {
       return true;
     }
-    if (build_id == other.build_id) {
+    if (build_id > other.build_id) {
       return false;
     }
     return type < other.type;
   }
 };
 
-static bool try_auxiliary_file(ElfFileReader& trace_file_reader,
-                               const string& trace_relative_name,
-                               const string& original_file_name,
-                               set<string>* file_names, const string& aux_file_name,
-                               const char* file_type,
-                               vector<DwoInfo>* dwos,
-                               set<ExternalDebugInfo>* external_debug_info) {
+static unique_ptr<ElfFileReader>
+find_auxiliary_file(const string& original_file_name,
+                    const string& aux_file_name,
+                    string& full_file_name) {
   if (aux_file_name.empty()) {
-    return false;
+    return nullptr;
   }
-  string full_file_name;
+  ScopedFd fd;
   if (aux_file_name.c_str()[0] == '/') {
     full_file_name = aux_file_name;
+    fd = ScopedFd(full_file_name.c_str(), O_RDONLY);
+    if (!fd.is_open()) {
+      LOG(warn) << "Can't find external debuginfo file " << full_file_name;
+      return nullptr;
+    }
   } else {
+    // Skip first trying the current directory. That's unlikely to be correct.
+
+    // Try in the same directory as the original file.
     string original_file_dir = original_file_name;
-    parent_dir(original_file_dir);
+    dir_name(original_file_dir);
     full_file_name = original_file_dir + "/" + aux_file_name;
     normalize_file_name(full_file_name);
+    fd = ScopedFd(full_file_name.c_str(), O_RDONLY);
+    if (fd.is_open()) {
+      // Debian/Ubuntu built /lib/x86_64-linux-gnu/ld-2.31.so with a
+      // .gnu_debuglink of "ld-2.31.so", expecting it to be found at
+      // /usr/lib/debug/lib/x86_64-linux-gnu/ld-2.31.so. So we need to make
+      // sure we aren't using the binary file as its own debuginfo.
+      if (real_path(original_file_name) != real_path(full_file_name)) {
+        goto found;
+      }
+    }
+    LOG(info) << "Can't find external debuginfo file " << full_file_name;
+
+    // Next try in a subdirectory called .debug
+    full_file_name = original_file_dir + "/.debug/" + aux_file_name;
+    normalize_file_name(full_file_name);
+    fd = ScopedFd(full_file_name.c_str(), O_RDONLY);
+    if (fd.is_open()) {
+      goto found;
+    }
+    LOG(info) << "Can't find external debuginfo file " << full_file_name;
+
+    // Then try in /usr/lib/debug
+    full_file_name = "/usr/lib/debug/" + aux_file_name;
+    normalize_file_name(full_file_name);
+    fd = ScopedFd(full_file_name.c_str(), O_RDONLY);
+    if (fd.is_open()) {
+      goto found;
+    }
+    LOG(info) << "Can't find external debuginfo file " << full_file_name;
+
+    // Try in an appropriate subdirectory of /usr/lib/debug
+    full_file_name = "/usr/lib/debug" + original_file_dir + "/" + aux_file_name;
+    normalize_file_name(full_file_name);
+    fd = ScopedFd(full_file_name.c_str(), O_RDONLY);
+    if (fd.is_open()) {
+      goto found;
+    }
+    LOG(info) << "Can't find external debuginfo file " << full_file_name;
+
+    // If none of those worked, give up.
+    LOG(warn) << "Exhausted auxilliary debuginfo search locations for " << aux_file_name;
+    return nullptr;
   }
 
-  ScopedFd fd(full_file_name.c_str(), O_RDONLY);
-  if (!fd.is_open()) {
-    LOG(warn) << "Can't find external debuginfo file " << full_file_name;
-    return false;
-  }
+found:
   LOG(info) << "Examining external " << full_file_name;
-  ElfFileReader reader(fd);
-  if (!reader.ok()) {
+  auto reader = make_unique<ElfFileReader>(fd);
+  if (!reader->ok()) {
     LOG(warn) << "Not an ELF file!";
-    return false;
+    return nullptr;
   }
+  return reader;
+}
+
+static unique_ptr<ElfFileReader>
+find_auxiliary_file_by_buildid(ElfFileReader& trace_file_reader, string& full_file_name) {
+  string build_id = trace_file_reader.read_buildid();
+  if (build_id.empty()) {
+    LOG(warn) << "Main ELF binary has no build ID!";
+    return nullptr;
+  }
+  if (build_id.size() < 3) {
+    LOG(warn) << "Build ID is too short!";
+    return nullptr;
+  }
+
+  string path = "/usr/lib/debug/.build-id/" + build_id.substr(0, 2) + "/" + build_id.substr(2) + ".debug";
+  ScopedFd fd(path.c_str(), O_RDONLY);
+  if (!fd.is_open()) {
+    LOG(info) << "Can't find external debuginfo file " << path;
+    return nullptr;
+  }
+
+  LOG(info) << "Examining external by buildid " << path;
+  auto reader = make_unique<ElfFileReader>(fd);
+  if (!reader->ok()) {
+    LOG(warn) << "Not an ELF file!";
+    return nullptr;
+  }
+  full_file_name = path;
+  return reader;
+}
+
+// Traverse the compilation units of an auxiliary file to collect their source files
+static bool process_auxiliary_file(ElfFileReader& trace_file_reader,
+                                   ElfFileReader& aux_file_reader,
+                                   ElfFileReader* alt_file_reader,
+                                   const string& trace_relative_name,
+                                   const string& original_file_name,
+                                   set<string>* file_names,
+                                   const string& full_aux_file_name,
+                                   const char* file_type,
+                                   const map<string, string>& comp_dir_substitutions,
+                                   vector<DwoInfo>* dwos,
+                                   set<ExternalDebugInfo>* external_debug_info,
+                                   bool already_used_file,
+                                   DirExistsCache& dir_exists_cache) {
   string build_id = trace_file_reader.read_buildid();
   if (build_id.empty()) {
     LOG(warn) << "Main ELF binary has no build ID!";
     return false;
   }
-  if (!process_compilation_units(reader, trace_relative_name, original_file_name, {}, file_names, dwos)) {
-    LOG(warn) << "No debuginfo!";
-    return false;
+
+  bool did_work;
+  string original_name = original_file_name;
+  base_name(original_name);
+  auto it = comp_dir_substitutions.find(original_name);
+  if (it != comp_dir_substitutions.end()) {
+    LOG(debug) << "\tFound comp_dir substitution " << it->second;
+    did_work = process_compilation_units(aux_file_reader, alt_file_reader,
+                                         trace_relative_name, original_file_name,
+                                         it->second, file_names, dwos, dir_exists_cache);
+  } else {
+    LOG(debug) << "\tNo comp_dir substitution found";
+    did_work = process_compilation_units(aux_file_reader, alt_file_reader,
+                                         trace_relative_name, original_file_name,
+                                         {}, file_names, dwos, dir_exists_cache);
   }
-  external_debug_info->insert({ full_file_name, build_id, string(file_type) });
-  return true;
+
+  if (!did_work) {
+    LOG(warn) << "No debuginfo!";
+    /* If we've already used this file we need to insert it into the external_debug_info
+     * set even if it does not have any CUs of its own.
+     */
+    if (!already_used_file) {
+      return false;
+    }
+  }
+  external_debug_info->insert({ full_aux_file_name, build_id, string(file_type) });
+  return did_work;
+}
+
+static bool try_debuglink_file(ElfFileReader& trace_file_reader,
+                               const string& trace_relative_name,
+                               const string& original_file_name,
+                               set<string>* file_names, const string& aux_file_name,
+                               const map<string, string>& comp_dir_substitutions,
+                               vector<DwoInfo>* dwos,
+                               set<ExternalDebugInfo>* external_debug_info,
+                               DirExistsCache& dir_exists_cache) {
+  string full_file_name;
+  auto reader = find_auxiliary_file(original_file_name, aux_file_name,
+                                    full_file_name);
+  if (!reader) {
+    reader = find_auxiliary_file_by_buildid(trace_file_reader, full_file_name);
+    if (!reader) {
+      return false;
+    }
+  }
+
+  /* A debuglink file can have its own debugaltlink */
+  string full_altfile_name;
+  Debugaltlink debugaltlink = reader->read_debugaltlink();
+  auto altlink_reader = find_auxiliary_file(original_file_name, debugaltlink.file_name,
+                                            full_altfile_name);
+
+  bool has_source_files = process_auxiliary_file(trace_file_reader, *reader, altlink_reader.get(),
+                                                 trace_relative_name, original_file_name,
+                                                 file_names, full_file_name, DEBUGLINK,
+                                                 comp_dir_substitutions,
+                                                 dwos, external_debug_info, false, dir_exists_cache);
+
+  if (altlink_reader) {
+    has_source_files |= process_auxiliary_file(trace_file_reader, *altlink_reader, nullptr,
+                                               trace_relative_name, original_file_name,
+                                               file_names, full_altfile_name, DEBUGALTLINK,
+                                               comp_dir_substitutions,
+                                               dwos, external_debug_info, has_source_files, dir_exists_cache);
+  }
+  return has_source_files;
 }
 
 struct Symlink {
@@ -399,7 +629,7 @@ static string resolve_symlinks(const string& path,
       string target;
       if (buf[0] != '/') {
         target = base;
-        parent_dir(target);
+        dir_name(target);
         if (target.size() > 1) {
           target.push_back('/');
         }
@@ -468,11 +698,18 @@ static bool starts_with(const string& s, const string& prefix) {
   return strncmp(s.c_str(), prefix.c_str(), prefix.size()) == 0;
 }
 
+struct OutputCompDirSubstitution {
+  string trace_relative_name;
+  string substitution;
+};
+
 static int sources(const map<string, string>& binary_file_names, const map<string, string>& comp_dir_substitutions, bool is_explicit) {
   vector<string> relevant_binary_names;
   set<string> file_names;
   set<ExternalDebugInfo> external_debug_info;
   vector<DwoInfo> dwos;
+  vector<OutputCompDirSubstitution> output_comp_dir_substitutions;
+  DirExistsCache dir_exists_cache;
   for (auto& pair : binary_file_names) {
     string trace_relative_name = pair.first;
     string original_name = pair.second;
@@ -491,21 +728,46 @@ static int sources(const map<string, string>& binary_file_names, const map<strin
       base_name(trace_relative_name);
     }
     base_name(original_name);
+    Debugaltlink debugaltlink = reader.read_debugaltlink();
+
+    string full_altfile_name;
+    auto altlink_reader = find_auxiliary_file(pair.second, debugaltlink.file_name,
+                                              full_altfile_name);
+
     bool has_source_files;
+    LOG(debug) << "Looking for comp_dir substitutions for " << original_name;
     auto it = comp_dir_substitutions.find(original_name);
     if (it != comp_dir_substitutions.end()) {
-      has_source_files = process_compilation_units(reader, trace_relative_name, pair.second, it->second, &file_names, &dwos);
+      LOG(debug) << "\tFound comp_dir substitution " << it->second;
+      output_comp_dir_substitutions.push_back({ trace_relative_name, it->second });
+      has_source_files = process_compilation_units(reader, altlink_reader.get(),
+                                                   trace_relative_name, pair.second,
+                                                   it->second, &file_names, &dwos, dir_exists_cache);
     } else {
-      has_source_files = process_compilation_units(reader, trace_relative_name, pair.second, {}, &file_names, &dwos);
+      LOG(debug) << "\tNo comp_dir substitution found";
+      has_source_files = process_compilation_units(reader, altlink_reader.get(),
+                                                   trace_relative_name, pair.second,
+                                                   {}, &file_names, &dwos, dir_exists_cache);
     }
+    /* If the original binary had source files, force the inclusion of any debugaltlink
+     * file, even if it does not itself have compilation units (it may have relevant strings)
+     */
+    const bool original_had_source_files = has_source_files;
 
     Debuglink debuglink = reader.read_debuglink();
-    has_source_files |= try_auxiliary_file(reader, trace_relative_name, pair.second,
-      &file_names, debuglink.file_name, "debuglink", &dwos, &external_debug_info);
+    has_source_files |= try_debuglink_file(reader, trace_relative_name, pair.second,
+                                           &file_names, debuglink.file_name,
+                                           comp_dir_substitutions, &dwos,
+                                           &external_debug_info, dir_exists_cache);
 
-    Debugaltlink debugaltlink = reader.read_debugaltlink();
-    has_source_files |= try_auxiliary_file(reader, trace_relative_name, pair.second,
-      &file_names, debugaltlink.file_name, "debugaltlink", &dwos, &external_debug_info);
+    if (altlink_reader) {
+      has_source_files |= process_auxiliary_file(reader, *altlink_reader, nullptr,
+                                                 trace_relative_name, pair.second,
+                                                 &file_names, full_altfile_name,
+                                                 DEBUGALTLINK, comp_dir_substitutions,
+                                                 &dwos, &external_debug_info,
+                                                 original_had_source_files, dir_exists_cache);
+    }
 
     if (has_source_files) {
       relevant_binary_names.push_back(move(trace_relative_name));
@@ -530,10 +792,20 @@ static int sources(const map<string, string>& binary_file_names, const map<strin
     while (!vcs_stack.empty() && !starts_with(f, *vcs_stack.back())) {
       vcs_stack.pop_back();
     }
-    while (vcs_dir_iterator != vcs_dirs.end() && starts_with(f, *vcs_dir_iterator)) {
-      vcs_stack.push_back(&*vcs_dir_iterator);
-      vcs_dirs_vector.push_back(&*vcs_dir_iterator);
-      ++vcs_dir_iterator;
+    while (vcs_dir_iterator != vcs_dirs.end()) {
+      if (starts_with(f, *vcs_dir_iterator)) {
+        vcs_stack.push_back(&*vcs_dir_iterator);
+        vcs_dirs_vector.push_back(&*vcs_dir_iterator);
+        ++vcs_dir_iterator;
+        continue;
+      }
+      if (*vcs_dir_iterator < f) {
+        // Skip this VCS dir because all of its files must have been
+        // skipped (not found).
+        ++vcs_dir_iterator;
+        continue;
+      }
+      break;
     }
     if (vcs_stack.empty()) {
       if (!pushed_empty_string) {
@@ -553,6 +825,14 @@ static int sources(const map<string, string>& binary_file_names, const map<strin
            i == relevant_binary_names.size() - 1 ? "" : ",");
   }
   printf("  ],\n");
+  printf("  \"comp_dir_substitutions\":{\n");
+  for (size_t i = 0; i < output_comp_dir_substitutions.size(); ++i) {
+    auto& sub = output_comp_dir_substitutions[i];
+    printf("    \"%s\": \"%s\"%s\n", json_escape(sub.trace_relative_name).c_str(),
+           json_escape(sub.substitution).c_str(),
+           i == output_comp_dir_substitutions.size() - 1 ? "" : ",");
+  }
+  printf("  },\n");
   printf("  \"external_debug_info\":[\n");
   size_t index = 0;
   for (auto& ext : external_debug_info) {
@@ -567,8 +847,9 @@ static int sources(const map<string, string>& binary_file_names, const map<strin
   printf("  \"dwos\":[\n");
   index = 0;
   for (auto& d : dwos) {
-    printf("    { \"name\":\"%s\", \"trace_file\":\"%s\", ",
+    printf("    { \"name\":\"%s\", \"full_path\":\"%s\", \"trace_file\":\"%s\", ",
            json_escape(d.name).c_str(),
+           json_escape(d.full_path).c_str(),
            json_escape(d.trace_file).c_str());
     if (!d.comp_dir.empty()) {
       printf("\"comp_dir\":\"%s\", ", json_escape(d.comp_dir).c_str());
@@ -656,6 +937,7 @@ int SourcesCommand::run(vector<string>& args) {
   if (!files) {
     FATAL() << "Can't open trace dir";
   }
+  closedir(files);
 
   map<string, string> binary_file_names;
   while (true) {
