@@ -141,20 +141,6 @@ public:
    */
   void reenable_cpuid_tsc();
 
-  /**
-   * Linux requires the invariant that that all members of a thread group
-   * are reaped before the thread group leader. This determines whether or
-   * not we're allowed to attempt reaping this thread or whether doing so
-   * risks deadlock.
-   */
-  bool may_reap();
-
-  /**
-   * Reaps a task-exit notification, thus detaching us from the tracee.
-   * N.B.: If may_reap is false, this risks a deadlock.
-   */
-  void reap();
-
   /**
    * Wait for the task to exit, but do not reap/detach yet.
    */
@@ -217,6 +203,17 @@ public:
    */
   std::string proc_stat_path();
 
+  /**
+   * Return the path of /proc/<pid>/exe
+   */
+  std::string proc_exe_path();
+
+  /**
+   * Return the path of the executable (i.e. what
+   * /proc/<pid>/exe points to).
+   */
+  std::string exe_path();
+
   /**
    * Stat |fd| in the context of this task's fd table.
    */
@@ -379,9 +376,10 @@ public:
   virtual void did_wait() {}
   /**
    * Return the pid of the task in its own pid namespace.
-   * Only RecordTasks actually change pid namespaces.
+   * Only RecordTasks actually change pid namespaces, but
+   * this value is stored and present during replay too.
    */
-  virtual pid_t own_namespace_tid() { return tid; }
+  pid_t own_namespace_tid() { return own_namespace_rec_tid; }
 
   /**
    * Assuming ip() is just past a breakpoint instruction, adjust
@@ -665,11 +663,6 @@ public:
    * interrupt_after_elapsed == 0.0, the interrupt will happen immediately.
    */
   void wait(double interrupt_after_elapsed = -1);
-  /**
-   * Return true if the status of this has changed, but don't
-   * block.
-   */
-  bool try_wait();
   /**
    * Return true if an unexpected exit was already detected for this task and
    * it is ready to be reported.
@@ -722,6 +715,13 @@ public:
   uint64_t write_ranges(const std::vector<FileMonitor::Range>& ranges,
                         void* data, size_t size);
 
+  /**
+   * Writes zeroes to the given memory range.
+   * For efficiency tries using MADV_REMOVE via `remote`. Caches
+   * an AutoRemoteSyscalls in `*remote`.
+   */
+  void write_zeroes(std::unique_ptr<AutoRemoteSyscalls>* remote, remote_ptr<void> addr, size_t size);
+
   /**
    * Don't use these helpers directly; use the safer and more
    * convenient variants above.
@@ -743,6 +743,13 @@ public:
   void write_bytes_helper(remote_ptr<void> addr, ssize_t buf_size,
                           const void* buf, bool* ok = nullptr,
                           uint32_t flags = 0);
+  /**
+   * |flags| is bits from WriteFlags.
+   * Returns number of bytes written.
+   */
+  ssize_t write_bytes_helper_no_notifications(remote_ptr<void> addr, ssize_t buf_size,
+                                              const void* buf, bool* ok = nullptr,
+                                              uint32_t flags = 0);
 
   SupportedArch detect_syscall_arch();
 
@@ -834,6 +841,8 @@ public:
    * recording, it's synonymous with |tid|, and during replay
    * it's the tid that was recorded. */
   pid_t rec_tid;
+  /* This is the recorded tid of the tracee *in its own pid namespace*. */
+  pid_t own_namespace_rec_tid;
 
   size_t syscallbuf_size;
   /* Points at the tracee's mapping of the buffer. */
@@ -873,6 +882,7 @@ public:
     uint64_t cloned_file_data_offset;
     ThreadLocals thread_locals;
     pid_t rec_tid;
+    pid_t own_namespace_rec_tid;
     uint32_t serial;
     ThreadGroupUid tguid;
     int desched_fd_child;
@@ -924,7 +934,10 @@ public:
     return was_reaped;
   }
 
-  void os_exec_stub(SupportedArch arch);
+  void os_exec(SupportedArch arch, std::string filename);
+  void os_exec_stub(SupportedArch arch) {
+      os_exec(arch, find_exec_stub(arch));
+  }
 
   /**
    * Try to make the current task look exactly like some `other` task
