@@ -42,9 +42,8 @@ namespace rr {
 // version number doesn't track the rr version number, because changes
 // to the trace format will be rare.
 //
-// NB: if you *do* change the trace format for whatever reason, you
-// MUST increment this version number.  Otherwise users' old traces
-// will become unreplayable and they won't know why.
+// We don't plan to ever change this again. Instead, we use CapnpProto
+// to define the trace format in an extensible way (see rr_trace.capnp).
 //
 #define TRACE_VERSION 85
 
@@ -402,8 +401,7 @@ void TraceWriter::write_frame(RecordTask* t, const Event& ev,
   // Use an on-stack first segment that should be adequate for most cases. A
   // simple syscall event takes 320 bytes currently. The default Capnproto
   // implementation does a calloc(8192) for the first segment.
-  word buf[reasonable_frame_message_words];
-  memset(buf, 0, sizeof(buf));
+  word buf[reasonable_frame_message_words] = {};
   MallocMessageBuilder frame_msg(buf);
   trace::Frame::Builder frame = frame_msg.initRoot<trace::Frame>();
 
@@ -417,6 +415,11 @@ void TraceWriter::write_frame(RecordTask* t, const Event& ev,
     w.setTid(r.rec_tid);
     w.setAddr(r.addr.as_int());
     w.setSize(r.size);
+    auto holes = w.initHoles(r.holes.size());
+    for (size_t j = 0; j < r.holes.size(); ++j) {
+      holes[j].setOffset(r.holes[j].offset);
+      holes[j].setSize(r.holes[j].size);
+    }
   }
   raw_recs.clear();
   frame.setArch(to_trace_arch(t->arch()));
@@ -500,6 +503,12 @@ void TraceWriter::write_frame(RecordTask* t, const Event& ev,
           o.setDevice(opened.device);
           o.setInode(opened.inode);
         }
+      } else if (e.socket_addrs) {
+        auto addrs = data.initSocketAddrs();
+        auto localAddr = (*e.socket_addrs.get())[0];
+        auto remoteAddr = (*e.socket_addrs.get())[1];
+        addrs.setLocalAddr(Data::Reader(reinterpret_cast<uint8_t*>(&localAddr), sizeof(localAddr)));
+        addrs.setRemoteAddr(Data::Reader(reinterpret_cast<uint8_t*>(&remoteAddr), sizeof(remoteAddr)));
       }
       break;
     }
@@ -533,7 +542,14 @@ TraceFrame TraceReader::read_frame() {
   for (size_t i = 0; i < raw_recs.size(); ++i) {
     // Build list in reverse order so we can efficiently pull records from it
     auto w = mem_writes[raw_recs.size() - 1 - i];
-    raw_recs[i] = { w.getAddr(), (size_t)w.getSize(), i32_to_tid(w.getTid()) };
+    auto holes = w.getHoles();
+    vector<WriteHole> h;
+    h.resize(holes.size());
+    for (size_t j = 0; j < h.size(); ++j) {
+      const auto& hole = holes[j];
+      h[j] = { hole.getOffset(), hole.getSize() };
+    }
+    raw_recs[i] = { w.getAddr(), (size_t)w.getSize(), i32_to_tid(w.getTid()), h };
   }
 
   TraceFrame ret;
@@ -665,6 +681,21 @@ TraceFrame TraceReader::read_frame() {
           }
           break;
         }
+        case trace::Frame::Event::Syscall::Extra::SOCKET_ADDRS: {
+          auto addrs = data.getSocketAddrs();
+          syscall_ev.socket_addrs = make_shared<std::array<typename NativeArch::sockaddr_storage, 2>>();
+          Data::Reader local = addrs.getLocalAddr();
+          if (local.size() != sizeof(NativeArch::sockaddr_storage)) {
+            FATAL() << "Invalid sockaddr length";
+          }
+          memcpy(&(*syscall_ev.socket_addrs.get())[0], local.begin(), sizeof(NativeArch::sockaddr_storage));
+          Data::Reader remote = addrs.getRemoteAddr();
+          if (remote.size() != sizeof(NativeArch::sockaddr_storage)) {
+            FATAL() << "Invalid sockaddr length";
+          }
+          memcpy(&(*syscall_ev.socket_addrs.get())[1], remote.begin(), sizeof(NativeArch::sockaddr_storage));
+          break;
+        }
         default:
           FATAL() << "Unknown syscall type";
           break;
@@ -922,7 +953,7 @@ TraceWriter::RecordInTrace TraceWriter::write_mapped_region(
   } else if (starts_with(km.fsname(), "/SYSV")) {
     src.setTrace();
   } else if (origin == SYSCALL_MAPPING &&
-             (km.inode() == 0 || km.fsname() == "/dev/zero (deleted)")) {
+             (km.inode() == 0 || km.fsname_strip_deleted() == "/dev/zero")) {
     src.setZero();
   } else if (!starts_with(km.fsname(), "/")) {
     src.setTrace();
@@ -1152,10 +1183,14 @@ KernelMapping TraceReader::read_mapped_region(MappedData* data, bool* found,
                        map.getFileOffsetBytes());
 }
 
-void TraceWriter::write_raw(pid_t rec_tid, const void* d, size_t len,
-                            remote_ptr<void> addr) {
+void TraceWriter::write_raw_header(pid_t rec_tid, size_t total_len,
+                                   remote_ptr<void> addr,
+                                   const std::vector<WriteHole>& holes = std::vector<WriteHole>()) {
+  raw_recs.push_back({ addr, total_len, rec_tid, holes });
+}
+
+void TraceWriter::write_raw_data(const void* d, size_t len) {
   auto& data = writer(RAW_DATA);
-  raw_recs.push_back({ addr, len, rec_tid });
   data.write(d, len);
 }
 
@@ -1174,8 +1209,44 @@ bool TraceReader::read_raw_data_for_frame(RawData& d) {
   auto& rec = raw_recs[raw_recs.size() - 1];
   d.rec_tid = rec.rec_tid;
   d.addr = rec.addr;
+
   d.data.resize(rec.size);
-  reader(RAW_DATA).read((char*)d.data.data(), rec.size);
+  auto hole_iter = rec.holes.begin();
+  uintptr_t offset = 0;
+  while (offset < d.data.size()) {
+    uintptr_t end = rec.size;
+    if (hole_iter != rec.holes.end()) {
+      if (offset == hole_iter->offset) {
+        memset(d.data.data() + offset, 0, hole_iter->size);
+        offset += hole_iter->size;
+        ++hole_iter;
+        continue;
+      }
+      end = hole_iter->offset;
+    }
+    reader(RAW_DATA).read((char*)d.data.data() + offset, end - offset);
+    offset = end;
+  }
+
+  raw_recs.pop_back();
+  return true;
+}
+
+bool TraceReader::read_raw_data_for_frame_with_holes(RawDataWithHoles& d) {
+  if (raw_recs.empty()) {
+    return false;
+  }
+  auto& rec = raw_recs[raw_recs.size() - 1];
+  d.rec_tid = rec.rec_tid;
+  d.addr = rec.addr;
+  d.holes = move(rec.holes);
+  size_t data_size = rec.size;
+  for (auto& h : d.holes) {
+    data_size -= h.size;
+  }
+  d.data.resize(data_size);
+  reader(RAW_DATA).read((char*)d.data.data(), data_size);
+
   raw_recs.pop_back();
   return true;
 }
@@ -1185,7 +1256,11 @@ bool TraceReader::read_raw_data_metadata_for_frame(RawDataMetadata& d) {
     return false;
   }
   d = raw_recs[raw_recs.size() - 1];
-  reader(RAW_DATA).skip(d.size);
+  size_t data_size = d.size;
+  for (auto& h : d.holes) {
+    data_size -= h.size;
+  }
+  reader(RAW_DATA).skip(data_size);
   raw_recs.pop_back();
   return true;
 }
@@ -1318,6 +1393,7 @@ void TraceWriter::close(CloseStatus status, const TraceUuid* uuid) {
   header.setTicksSemantics(
     to_trace_ticks_semantics(PerfCounters::default_ticks_semantics()));
   header.setSyscallbufProtocolVersion(SYSCALLBUF_PROTOCOL_VERSION);
+  header.setRequiredForwardCompatibilityVersion(FORWARD_COMPATIBILITY_VERSION);
   header.setPreloadThreadLocalsRecorded(true);
   header.setRrcallBase(syscall_number_for_rrcall_init_preload(x86_64));
 
@@ -1335,7 +1411,11 @@ void TraceWriter::close(CloseStatus status, const TraceUuid* uuid) {
     x86data.setClearFipFdp(clear_fip_fdp_);
   }
 
-  header.setExplicitProcMem(false);
+  {
+    auto quirks = header.initQuirks();
+    quirks.setExplicitProcMem(false);
+    quirks.setSpecialLibrrpage(false);
+  }
   // Add a random UUID to the trace metadata. This lets tools identify a trace
   // easily.
   if (!uuid) {
@@ -1346,6 +1426,11 @@ void TraceWriter::close(CloseStatus status, const TraceUuid* uuid) {
     header.setUuid(Data::Reader(uuid->bytes, sizeof(TraceUuid::bytes)));
   }
   header.setOk(status == CLOSE_OK);
+  header.setChaosMode(chaos_mode ? trace::ChaosMode::KNOWN_TRUE : trace::ChaosMode::KNOWN_FALSE);
+  MemoryRange exclusion_range = AddressSpace::get_global_exclusion_range();
+  header.setExclusionRangeStart(exclusion_range.start().as_int());
+  header.setExclusionRangeEnd(exclusion_range.end().as_int());
+
   try {
     writePackedMessageToFd(version_fd, header_msg);
   } catch (...) {
@@ -1477,7 +1562,17 @@ TraceReader::TraceReader(const string& dir)
   preload_thread_locals_recorded_ = header.getPreloadThreadLocalsRecorded();
   ticks_semantics_ = from_trace_ticks_semantics(header.getTicksSemantics());
   rrcall_base_ = header.getRrcallBase();
-  explicit_proc_mem_ = header.getExplicitProcMem();
+  required_forward_compatibility_version_ = header.getRequiredForwardCompatibilityVersion();
+  quirks_ = 0;
+  {
+    auto quirks = header.getQuirks();
+    if (quirks.getExplicitProcMem()) {
+      quirks_ |= ExplicitProcMem;
+    }
+    if (quirks.getSpecialLibrrpage()) {
+      quirks_ |= SpecialLibRRpage;
+    }
+  }
   Data::Reader uuid = header.getUuid();
   uuid_ = unique_ptr<TraceUuid>(new TraceUuid());
   if (uuid.size() != sizeof(uuid_->bytes)) {
@@ -1501,6 +1596,23 @@ TraceReader::TraceReader(const string& dir)
     clear_fip_fdp_ = x86data.getClearFipFdp();
   }
 
+  switch (header.getChaosMode()) {
+    case trace::ChaosMode::UNKNOWN:
+      chaos_mode_known_ = false;
+      chaos_mode_ = false;
+      break;
+    case trace::ChaosMode::KNOWN_TRUE:
+      chaos_mode_known_ = true;
+      chaos_mode_ = true;
+      break;
+    case trace::ChaosMode::KNOWN_FALSE:
+      chaos_mode_known_ = true;
+      chaos_mode_ = false;
+      break;
+  }
+  exclusion_range_ = MemoryRange(remote_ptr<void>(header.getExclusionRangeStart()),
+                                 remote_ptr<void>(header.getExclusionRangeEnd()));
+
   // Set the global time at 0, so that when we tick it for the first
   // event, it matches the initial global time at recording, 1.
   global_time = 0;
@@ -1526,8 +1638,12 @@ TraceReader::TraceReader(const TraceReader& other)
   preload_thread_locals_recorded_ = other.preload_thread_locals_recorded_;
   rrcall_base_ = other.rrcall_base_;
   arch_ = other.arch_;
-  explicit_proc_mem_ = other.explicit_proc_mem_;
+  chaos_mode_ = other.chaos_mode_;
+  chaos_mode_known_ = other.chaos_mode_known_;
+  exclusion_range_ = other.exclusion_range_;
+  quirks_ = other.quirks_;
   clear_fip_fdp_ = other.clear_fip_fdp_;
+  required_forward_compatibility_version_ = other.required_forward_compatibility_version_;
 }
 
 TraceReader::~TraceReader() {}
