@@ -14,6 +14,7 @@
 #include "CompressedReader.h"
 #include "CompressedWriter.h"
 #include "Event.h"
+#include "MemoryRange.h"
 #include "TaskishUid.h"
 #include "TraceFrame.h"
 #include "TraceTaskEvent.h"
@@ -21,12 +22,22 @@
 
 namespace rr {
 
+/**
+ * Bump this when rr changes mean that traces produced by new rr can't be replayed by old rr.
+ */
+const int FORWARD_COMPATIBILITY_VERSION = 1;
+
 struct CPUIDRecord;
 struct DisableCPUIDFeatures;
 class KernelMapping;
 class RecordTask;
 struct TraceUuid;
 
+struct WriteHole {
+  uint64_t offset;
+  uint64_t size;
+};
+
 /**
  * TraceStream stores all the data common to both recording and
  * replay.  TraceWriter deals with recording-specific logic, and
@@ -44,6 +55,7 @@ public:
     remote_ptr<void> addr;
     size_t size;
     pid_t rec_tid;
+    std::vector<WriteHole> holes;
   };
 
   /**
@@ -212,8 +224,13 @@ public:
    * 'addr' is the address in the tracee where the data came from/will be
    * restored to.
    */
-  void write_raw(pid_t tid, const void* data, size_t len,
-                 remote_ptr<void> addr);
+  void write_raw(pid_t tid, const void* data, size_t len, remote_ptr<void> addr) {
+    write_raw_data(data, len);
+    write_raw_header(tid, len, addr, std::vector<WriteHole>());
+  }
+  void write_raw_data(const void* data, size_t len);
+  void write_raw_header(pid_t tid, size_t total_len, remote_ptr<void> addr,
+                        const std::vector<WriteHole>& holes);
 
   /**
    * Write a task event (clone or exec record) to the trace.
@@ -245,6 +262,7 @@ public:
   void set_fdp_exception_only_quirk(bool value) { fdp_exception_only_quirk_ = value; }
   void set_clear_fip_fdp(bool value) { clear_fip_fdp_ = value; }
   bool clear_fip_fdp() const { return clear_fip_fdp_; }
+  void set_chaos_mode(bool value) { chaos_mode = value; }
 
   enum CloseStatus {
     /**
@@ -304,13 +322,14 @@ private:
   bool fdp_exception_only_quirk_;
   bool clear_fip_fdp_;
   bool supports_file_data_cloning_;
+  bool chaos_mode;
 };
 
 class TraceReader : public TraceStream {
 public:
   /**
    * A parcel of recorded tracee data.  |data| contains the data read
-   * from |addr| in the tracee.
+   * from |addr| in the tracee. `data` contains zeroes for holes.
    */
   struct RawData {
     std::vector<uint8_t> data;
@@ -318,6 +337,16 @@ public:
     pid_t rec_tid;
   };
 
+  /**
+   * Like RawData, but returns positions of holes. `data` excludes holes.
+   */
+  struct RawDataWithHoles {
+    std::vector<uint8_t> data;
+    remote_ptr<void> addr;
+    pid_t rec_tid;
+    std::vector<WriteHole> holes;
+  };
+
   /**
    * Read relevant data from the trace.
    *
@@ -358,9 +387,18 @@ public:
   /**
    * Reads the next raw data record for last-read frame. If there are no more
    * raw data records for this frame, return false.
+   * Holes are filled with zeroes in the output buffer.
    */
   bool read_raw_data_for_frame(RawData& d);
 
+  /**
+   * Reads the next raw data record for last-read frame. If there are no more
+   * raw data records for this frame, return false.
+   * Returns hole metadata so you can do something smarter with it than
+   * explicitly filling with zeroes.
+   */
+  bool read_raw_data_for_frame_with_holes(RawDataWithHoles& d);
+
   /**
    * Like read_raw_data_for_frame, but doesn't actually read the data bytes.
    * The array is resized but the data is not filled in.
@@ -429,8 +467,25 @@ public:
 
   SupportedArch arch() const { return arch_; }
 
-  // Whether the /proc/<pid>/mem calls were explicitly recorded in this trace
-  bool explicit_proc_mem() const { return explicit_proc_mem_; }
+  bool chaos_mode(bool* known) const {
+    *known = chaos_mode_known_;
+    return chaos_mode_;
+  }
+  MemoryRange exclusion_range() const {
+    return exclusion_range_;
+  }
+
+  enum TraceQuirks {
+    // Whether the /proc/<pid>/mem calls were explicitly recorded in this trace
+    ExplicitProcMem = 0x1,
+    // Whether this trace requires the special librrpage replay behavior
+    // added in 3aaf792 and later removed.
+    SpecialLibRRpage = 0x2
+  };
+
+  int quirks() const { return quirks_; }
+
+  int required_forward_compatibility_version() const { return required_forward_compatibility_version_; }
 
 private:
   CompressedReader& reader(Substream s) { return *readers[s]; }
@@ -443,12 +498,16 @@ private:
   TicksSemantics ticks_semantics_;
   double monotonic_time_;
   std::unique_ptr<TraceUuid> uuid_;
+  MemoryRange exclusion_range_;
   bool trace_uses_cpuid_faulting;
   bool preload_thread_locals_recorded_;
   bool clear_fip_fdp_;
+  bool chaos_mode_known_;
+  bool chaos_mode_;
   int rrcall_base_;
+  int required_forward_compatibility_version_;
   SupportedArch arch_;
-  bool explicit_proc_mem_;
+  int quirks_;
 };
 
 extern std::string trace_save_dir();
