@@ -80,53 +80,10 @@ class AssemblyTemplate(object):
         return bytes
 
 templates = {
-    'X86SysenterVsyscallImplementation': AssemblyTemplate(
-        RawBytes(0x51),         # push %ecx
-        RawBytes(0x52),         # push %edx
-        RawBytes(0x55),         # push %ebp
-        RawBytes(0x89, 0xe5),   # mov %esp,%ebp
-        RawBytes(0x0f, 0x34),   # sysenter
-    ),
-    'X86SysenterVsyscallImplementationAMD': AssemblyTemplate(
-        RawBytes(0x51),         # push %ecx
-        RawBytes(0x52),         # push %edx
-        RawBytes(0x55),         # push %ebp
-        RawBytes(0x89, 0xcd),   # mov %ecx,%ebp
-        RawBytes(0x0f, 0x05),   # syscall
-        RawBytes(0xcd, 0x80),   # int $0x80
-    ),
-    'X86SysenterVsyscallUseInt80': AssemblyTemplate(
-        RawBytes(0xcd, 0x80),   # int $0x80
-        RawBytes(0xc3),         # ret
-    ),
     'X86SysenterVsyscallSyscallHook': AssemblyTemplate(
         RawBytes(0xe9),         # jmp $syscall_hook_trampoline
         Field('syscall_hook_trampoline', 4),
     ),
-    'X86VsyscallMonkeypatch': AssemblyTemplate(
-        RawBytes(0xb8),         # mov $syscall_number,%eax
-        Field('syscall_number', 4),
-        RawBytes(0xe8),         # call $X86VsyscallMonkeypatchShared
-        Field('vsyscall_monkeypatch_shared', 4),
-        RawBytes(0xc3),
-    ),
-    'X86VsyscallMonkeypatchShared': AssemblyTemplate(
-        # __vdso functions use the C calling convention, so
-        # we have to set up the syscall parameters here.
-        # No x86-32 __vdso functions take more than two parameters.
-        RawBytes(0x53),         # push %ebx
-        RawBytes(0x8b, 0x5c, 0x24, 0x0c), # mov 12(%esp),%ebx
-        RawBytes(0x8b, 0x4c, 0x24, 0x10), # mov 16(%esp),%ecx
-        RawBytes(0xcd, 0x80),   # int $0x80
-        # pad with NOPs to make room to dynamically patch the syscall
-        # with a call to the preload library, once syscall buffering
-        # has been initialized.
-        RawBytes(0x90),         # nop
-        RawBytes(0x90),         # nop
-        RawBytes(0x90),         # nop
-        RawBytes(0x5b),         # pop %ebx
-        RawBytes(0xc3),         # ret
-    ),
     'X86SyscallStubExtendedJump': AssemblyTemplate(
         # This code must match the stubs in syscall_hook.S.
         RawBytes(0x89, 0x25, 0x08, 0x10, 0x00, 0x70), # movl %esp,(stub_scratch_1)
@@ -141,23 +98,18 @@ templates = {
         RawBytes(0xe9),                               # jmp $trampoline_relative_addr
         Field('trampoline_relative_addr', 4)
     ),
-
+    'X86SyscallStubRestore': AssemblyTemplate(
+        RawBytes(0xe9),                               # jmp $trampoline_relative_addr
+        Field('trampoline_relative_addr', 4)
+    ),
+    'X64CallMonkeypatch': AssemblyTemplate(
+        RawBytes(0xe8),         # call $relative_addr
+        Field('relative_addr', 4),
+    ),
     'X64JumpMonkeypatch': AssemblyTemplate(
         RawBytes(0xe9),         # jmp $relative_addr
         Field('relative_addr', 4),
     ),
-    'X64VsyscallMonkeypatch': AssemblyTemplate(
-        RawBytes(0xb8),         # mov $syscall_number,%eax
-        Field('syscall_number', 4),
-        RawBytes(0x0f, 0x05),   # syscall
-        # pad with NOPs to make room to dynamically patch the syscall
-        # with a call to the preload library, once syscall buffering
-        # has been initialized.
-        RawBytes(0x90),         # nop
-        RawBytes(0x90),         # nop
-        RawBytes(0x90),         # nop
-        RawBytes(0xc3),         # ret
-    ),
     'X64SyscallStubExtendedJump': AssemblyTemplate(
         # This code must match the stubs in syscall_hook.S.
         RawBytes(0x48, 0x89, 0x24, 0x25, 0x10, 0x10, 0x00, 0x70), # movq %rsp,(stub_scratch_1)
@@ -178,6 +130,10 @@ templates = {
         RawBytes(0xff, 0x25, 0x00, 0x00, 0x00, 0x00),       # jmp *0(%rip)
         Field('jump_target', 8),
     ),
+    'X64SyscallStubRestore': AssemblyTemplate(
+        RawBytes(0xff, 0x25, 0x00, 0x00, 0x00, 0x00),       # jmp *0(%rip)
+        Field('return_addr', 8),
+    ),
     'X64DLRuntimeResolve': AssemblyTemplate(
         RawBytes(0x53),                   # push %rbx
         RawBytes(0x48, 0x89, 0xe3),       # mov %rsp,%rbx
@@ -192,11 +148,13 @@ templates = {
         RawBytes(0xd9, 0x74, 0x24, 0xe0),                               # fstenv -32(%rsp)
         RawBytes(0x48, 0xc7, 0x44, 0x24, 0xf4, 0x00, 0x00, 0x00, 0x00), # movq $0,-12(%rsp)
         RawBytes(0xd9, 0x64, 0x24, 0xe0),                               # fldenv -32(%rsp)
-        RawBytes(0x53),                   # push %rbx
+        RawBytes(0x48, 0x87, 0x1c, 0x24), # xchg (%rsp),%rbx
+        # r11 is destroyed anyways by _dl_runtime_resolve, so we can use it here.
+        RawBytes(0x49, 0x89, 0xdb),       # mov %rbx,%r11
         RawBytes(0x48, 0x89, 0xe3),       # mov %rsp,%rbx
         RawBytes(0x48, 0x83, 0xe4, 0xc0), # and $0xffffffffffffffc0,%rsp
-        RawBytes(0xe9),                   # jmp $relative_addr
-        Field('relative_addr', 4),
+        RawBytes(0x41, 0x53),             # push %r11
+        RawBytes(0xc3),                   # ret
     ),
     'X64EndBr': AssemblyTemplate(
         RawBytes(0xf3, 0x0f, 0x1e, 0xfa)
@@ -214,13 +172,6 @@ templates = {
         Field('syscallno', 4),
         RawBytes(0x0f, 0x05) # syscall
     ),
-    'ARM64VdsoMonkeypatch': AssemblyTemplate(
-        ShiftField(
-            RawBytes(0x08, 0x00, 0x80, 0xd2),   # movz x8, #0
-            5, 'syscall_number', 2),
-        RawBytes(0x01, 0x00, 0x00, 0xd4),   # svc #0
-        RawBytes(0xc0, 0x03, 0x5f, 0xd6),   # ret
-    ),
 }
 
 def byte_array_name(name):
