@@ -25,9 +25,11 @@ static InstructionBuf read_instruction(Task* t, remote_code_ptr ip) {
 
 struct DecodedInstruction {
   int operand_size;
+  int address_size;
   int length;
   bool modifies_flags;
   bool uses_si;
+  bool is_repne;
 };
 
 /**
@@ -37,11 +39,13 @@ struct DecodedInstruction {
 static bool decode_x86_string_instruction(const InstructionBuf& code,
                                           DecodedInstruction* decoded) {
   bool found_operand_prefix = false;
+  bool found_address_prefix = false;
   bool found_REP_prefix = false;
   bool found_REXW_prefix = false;
 
   decoded->modifies_flags = false;
   decoded->uses_si = false;
+  decoded->is_repne = false;
 
   int i;
   bool done = false;
@@ -50,6 +54,9 @@ static bool decode_x86_string_instruction(const InstructionBuf& code,
       case 0x66:
         found_operand_prefix = true;
         break;
+      case 0x67:
+        found_address_prefix = true;
+        break;
       case 0x48:
         if (code.arch == x86_64) {
           found_REXW_prefix = true;
@@ -57,6 +64,8 @@ static bool decode_x86_string_instruction(const InstructionBuf& code,
         }
         return false;
       case 0xF2:
+        decoded->is_repne = true;
+        RR_FALLTHROUGH;
       case 0xF3:
         found_REP_prefix = true;
         break;
@@ -101,6 +110,7 @@ static bool decode_x86_string_instruction(const InstructionBuf& code,
   } else {
     decoded->operand_size = 1;
   }
+  decoded->address_size = found_address_prefix ? 4 : 8;
   return true;
 }
 
@@ -151,47 +161,46 @@ static void bound_iterations_for_watchpoint(Task* t, remote_ptr<void> reg,
   *iterations = min(*iterations, steps);
 }
 
-static bool is_x86ish(Task* t) {
-  return t->arch() == x86 || t->arch() == x86_64;
-}
-
-bool fast_forward_through_instruction(Task* t, ResumeRequest how,
-                                      const vector<const Registers*>& states) {
+FastForwardStatus fast_forward_through_instruction(Task* t, ResumeRequest how,
+                                                   const vector<const Registers*>& states) {
   DEBUG_ASSERT(how == RESUME_SINGLESTEP || how == RESUME_SYSEMU_SINGLESTEP);
+  FastForwardStatus result;
 
   remote_code_ptr ip = t->ip();
 
   t->resume_execution(how, RESUME_WAIT, RESUME_UNLIMITED_TICKS);
   if (t->stop_sig() != SIGTRAP) {
     // we might have stepped into a system call...
-    return false;
+    return result;
   }
 
   if (t->ip() != ip) {
-    return false;
+    return result;
   }
   if (t->vm()->get_breakpoint_type_at_addr(ip) != BKPT_NONE) {
     // breakpoint must have fired
-    return false;
+    return result;
   }
-  if (t->vm()->notify_watchpoint_fired(t->debug_status(),
-          t->last_execution_resume())) {
+  if (t->compute_trap_reasons().watchpoint) { 
     // watchpoint fired
-    return false;
+    return result;
   }
   for (auto& state : states) {
     if (state->matches(t->regs())) {
-      return false;
+      return result;
     }
   }
-  if (!is_x86ish(t)) {
-    return false;
+  if (!is_x86ish(t->arch())) {
+    return result;
   }
 
   InstructionBuf instruction_buf = read_instruction(t, ip);
   DecodedInstruction decoded;
   if (!decode_x86_string_instruction(instruction_buf, &decoded)) {
-    return false;
+    return result;
+  }
+  if (decoded.address_size != 8) {
+    ASSERT(t, false) << "Address-size prefix on string instructions unsupported";
   }
 
   remote_code_ptr limit_ip = ip + decoded.length;
@@ -203,7 +212,6 @@ bool fast_forward_through_instruction(Task* t, ResumeRequest how,
   vector<const Registers*> states_copy;
   auto using_states = &states;
 
-  bool did_execute = false;
   while (true) {
     // This string instruction should execute until CX reaches 0 and
     // we move to the next instruction, or we hit one of the states in
@@ -212,10 +220,9 @@ bool fast_forward_through_instruction(Task* t, ResumeRequest how,
     // already verified there isn't one set here.)
 
     // We'll compute an upper bound on the number of string instruction
-    // iterations to execute, and set a watchpoint on the memory location
-    // accessed through DI in the iteration we want to stop at. We'll also
-    // set a breakpoint after the string instruction to catch cases where it
-    // ends due to a ZF change.
+    // iterations to execute, and execute just that many iterations by
+    // modifying CX, setting a breakpoint after the string instruction to catch it
+    // ending.
     // Keep in mind that it's possible that states in |states| might
     // belong to multiple independent loops of this string instruction, with
     // registers reset in between the loops.
@@ -223,10 +230,12 @@ bool fast_forward_through_instruction(Task* t, ResumeRequest how,
     uintptr_t cur_cx = t->regs().cx();
     if (cur_cx == 0) {
       // Fake singlestep status for trap diagnosis
-      t->set_debug_status(DS_SINGLESTEP);
+      t->set_x86_debug_status(DS_SINGLESTEP);
       // This instruction will be skipped entirely.
-      return did_execute;
+      return result;
     }
+    // There is at least one more iteration to go.
+    result.incomplete_fast_forward = true;
 
     // Don't execute the last iteration of the string instruction. That
     // simplifies code below that tries to emulate the register effects
@@ -282,8 +291,8 @@ bool fast_forward_through_instruction(Task* t, ResumeRequest how,
 
     if (iterations == 0) {
       // Fake singlestep status for trap diagnosis
-      t->set_debug_status(DS_SINGLESTEP);
-      return did_execute;
+      t->set_x86_debug_status(DS_SINGLESTEP);
+      return result;
     }
 
     LOG(debug) << "x86-string fast-forward: " << iterations
@@ -291,114 +300,56 @@ bool fast_forward_through_instruction(Task* t, ResumeRequest how,
 
     Registers r = t->regs();
 
-    int direction = t->regs().df_flag() ? -1 : 1;
-    // Figure out the address to set a watchpoint at. This address must
-    // be accessed at or before the last iteration we want to perform.
-    // We have to account for a CPU quirk: Intel CPUs may coalesce iterations
-    // to write up to 64 bytes at a time (observed for "rep stosb" on Ivy
-    // Bridge). Assume 128 bytes to be safe.
-    static const unsigned BYTES_COALESCED = 128;
-    uintptr_t watch_offset = decoded.operand_size * (iterations - 1);
-    if (watch_offset > BYTES_COALESCED) {
-      watch_offset -= BYTES_COALESCED;
-      t->vm()->save_watchpoints();
-      t->vm()->remove_all_watchpoints();
-      remote_ptr<void> watch_di = t->regs().di() + direction * watch_offset;
-      LOG(debug) << "Set x86-string fast-forward watchpoint at " << watch_di;
-      bool ok = t->vm()->add_watchpoint(watch_di, 1, WATCH_READWRITE);
-      ASSERT(t, ok) << "Can't even handle one watchpoint???";
-      ok = t->vm()->add_breakpoint(limit_ip, BKPT_INTERNAL);
-      ASSERT(t, ok) << "Failed to add breakpoint";
-
-      t->resume_execution(RESUME_CONT, RESUME_WAIT, RESUME_UNLIMITED_TICKS);
-      did_execute = true;
-      ASSERT(t, t->stop_sig() == SIGTRAP);
-      // Grab debug_status before restoring watchpoints, since the latter
-      // clears the debug status
-      bool triggered_watchpoint =
-          t->vm()->notify_watchpoint_fired(t->debug_status(),
-              t->last_execution_resume());
-      t->vm()->remove_breakpoint(limit_ip, BKPT_INTERNAL);
-      t->vm()->restore_watchpoints();
-
-      ASSERT(t, cur_cx > t->regs().cx());
-      uintptr_t iterations_performed = cur_cx - t->regs().cx();
-      // we shoudn't execute more iterations than we asked for.
-      // In Ubuntu-14 4.2.0-27-generic in a KVM guest we have seen watchpoints
-      // failing to fire during string_instructions_replay. We plow through
-      // and hit the backup breakpoint. triggered_watchpoint is true because
-      // the memory has changed. This assertion should catch such errors.
-      ASSERT(t, iterations >= iterations_performed);
-      iterations -= iterations_performed;
-      // instructions that don't modify flags should not terminate too early.
-      // We can terminate prematurely when the watchpoint we set relative
-      // to DI is triggered by a read via SI.
-      ASSERT(t,
-             decoded.modifies_flags || iterations <= BYTES_COALESCED ||
-                 triggered_watchpoint);
-
-      if (!triggered_watchpoint) {
-        // watchpoint didn't fire. We must have exited the loop early and
-        // hit the breakpoint. IP will be after the breakpoint instruction.
-        ASSERT(t,
-               t->ip() == limit_ip.increment_by_bkpt_insn_length(t->arch()) &&
-                   decoded.modifies_flags);
-        // Undo the execution of the breakpoint instruction.
-        Registers tmp = t->regs();
-        tmp.set_ip(limit_ip);
-        t->set_regs(tmp);
-      } else {
-        ASSERT(t, t->ip() == limit_ip || t->ip() == ip);
-        watch_offset = decoded.operand_size * (iterations - 1);
-        if (watch_offset > BYTES_COALESCED) {
-          // Fake singlestep status for trap diagnosis
-          t->set_debug_status(DS_SINGLESTEP);
-          // We fired the watchpoint too early, perhaps because reads through SI
-          // triggered it. Let's just bail out now; better for the caller to
-          // retry fast_forward_through_instruction than for us to try
-          // singlestepping all the rest of the way.
-          LOG(debug) << "x86-string fast-forward: " << iterations
-                     << " iterations to go, but watchpoint hit early; aborted";
-          return did_execute;
-        }
-      }
-    }
-
-    LOG(debug) << "x86-string fast-forward: " << iterations
-               << " iterations to go";
-
-    // Singlestep through the remaining iterations.
-    while (iterations > 0 && t->ip() == ip) {
-      // Don't count ticks here. Reactivating the performance counter can be
-      // expensive and since we know we're just executing the string instruction
-      // we shouldn't miss any ticks here.
-      t->resume_execution(RESUME_SINGLESTEP, RESUME_WAIT, RESUME_NO_TICKS);
-      did_execute = true;
-      ASSERT(t, t->stop_sig() == SIGTRAP);
-      // Watchpoints can fire spuriously because configure_watch_registers
-      // can increase the size of the watched area to conserve watch registers.
-      --iterations;
-    }
-
-    if (t->ip() != ip) {
-      // We exited the loop early due to flags being modified.
-      ASSERT(t, t->ip() == limit_ip && decoded.modifies_flags);
+    Registers tmp = r;
+    tmp.set_cx(iterations);
+    t->set_regs(tmp);
+    bool ok = t->vm()->add_breakpoint(limit_ip, BKPT_INTERNAL);
+    ASSERT(t, ok) << "Failed to add breakpoint";
+    // Watchpoints can fire spuriously because configure_watch_registers
+    // can increase the size of the watched area to conserve watch registers.
+    // So, disable watchpoints temporarily.
+    t->vm()->save_watchpoints();
+    t->vm()->remove_all_watchpoints();
+    t->resume_execution(RESUME_CONT, RESUME_WAIT, RESUME_UNLIMITED_TICKS);
+    t->vm()->restore_watchpoints();
+    t->vm()->remove_breakpoint(limit_ip, BKPT_INTERNAL);
+    result.did_fast_forward = true;
+    // We should have reached the breakpoint
+    ASSERT(t, t->stop_sig() == SIGTRAP);
+    ASSERT(t, t->ip() == limit_ip.increment_by_bkpt_insn_length(t->arch()));
+    uintptr_t iterations_performed = iterations - t->regs().cx();
+    tmp = t->regs();
+    // Undo our change to CX value
+    tmp.set_cx(tmp.cx() + cur_cx - iterations);
+    if (decoded.modifies_flags &&
+        (t->regs().cx() > 0 ||
+         (decoded.is_repne && t->regs().zf_flag()) ||
+         (!decoded.is_repne && !t->regs().zf_flag()))) {
       // String instructions that modify flags don't have non-register side
       // effects, so we can reset registers to effectively unwind the loop.
       // Then we try rerunning the loop again, adding this state as one to
       // avoid stepping into. We shouldn't need to do this more than once!
       ASSERT(t, states_copy.empty());
-      extra_state_to_avoid = t->regs();
+      tmp.set_ip(limit_ip);
+      extra_state_to_avoid = tmp;
       states_copy = states;
       states_copy.push_back(&extra_state_to_avoid);
       using_states = &states_copy;
       t->set_regs(r);
-    } else {
-      LOG(debug) << "x86-string fast-forward done; ip()==" << t->ip();
-      // Fake singlestep status for trap diagnosis
-      t->set_debug_status(DS_SINGLESTEP);
-      return did_execute;
+      continue;
     }
+    // instructions that don't modify flags should not terminate too early.
+    ASSERT(t, t->regs().cx() == 0);
+    ASSERT(t, iterations_performed == iterations);
+    // We always end with at least one iteration to go in the string instruction,
+    // so we must have the IP of the string instruction.
+    tmp.set_ip(r.ip());
+    t->set_regs(tmp);
+
+    LOG(debug) << "x86-string fast-forward done; ip()==" << t->ip();
+    // Fake singlestep status for trap diagnosis
+    t->set_x86_debug_status(DS_SINGLESTEP);
+    return result;
   }
 }
 
@@ -490,7 +441,7 @@ static bool is_string_instruction_before(Task* t, remote_code_ptr ip) {
 }
 
 bool maybe_at_or_after_x86_string_instruction(Task* t) {
-  if (!is_x86ish(t)) {
+  if (!is_x86ish(t->arch())) {
     return false;
   }
 
@@ -499,7 +450,7 @@ bool maybe_at_or_after_x86_string_instruction(Task* t) {
 }
 
 bool at_x86_string_instruction(Task* t) {
-  if (!is_x86ish(t)) {
+  if (!is_x86ish(t->arch())) {
     return false;
   }
 
