@@ -10,7 +10,10 @@
 #include <elf.h>
 #include <fcntl.h>
 #include <linux/capability.h>
+#include <linux/cdrom.h>
 #include <linux/ethtool.h>
+#include <linux/fb.h>
+#include <linux/fiemap.h>
 #include <linux/filter.h>
 #include <linux/futex.h>
 #include <linux/if_bonding.h>
@@ -19,12 +22,15 @@
 #include <linux/msg.h>
 #include <linux/net.h>
 #include <linux/netfilter/x_tables.h>
+#include <linux/seccomp.h>
 #include <linux/sem.h>
+#include <linux/serial.h>
 #include <linux/shm.h>
 #include <linux/sockios.h>
 #include <linux/sysctl.h>
 #include <linux/usbdevice_fs.h>
 #include <linux/videodev2.h>
+#include <linux/vt.h>
 #include <linux/wireless.h>
 #include <poll.h>
 #include <scsi/sg.h>
@@ -34,6 +40,7 @@
 #include <stdint.h>
 #include <sys/epoll.h>
 #include <sys/ioctl.h>
+#include <sys/prctl.h>
 #include <sys/quota.h>
 #include <sys/resource.h>
 #include <sys/socket.h>
