@@ -108,10 +108,12 @@ struct FcntlConstants {
     OFD_SETLK = 37,
     OFD_SETLKW = 38,
     // Other Linux-specific operations
+    NOTIFY = 0x400 + 2,
     DUPFD_CLOEXEC = 0x400 + 6,
     SETPIPE_SZ = 0x400 + 7,
     GETPIPE_SZ = 0x400 + 8,
     ADD_SEALS = 0x400 + 9,
+    GET_SEALS = 0x400 + 10,
     GET_RW_HINT = 0x400 + 11,
     SET_RW_HINT = 0x400 + 12,
     GET_FILE_RW_HINT = 0x400 + 13,
@@ -444,6 +446,11 @@ struct BaseArch : public wordsize,
   };
   RR_VERIFY_TYPE(sockaddr);
 
+  struct sockaddr_storage {
+    char sa_data[128];
+  };
+  RR_VERIFY_TYPE(sockaddr_storage);
+
   struct sockaddr_un {
     unsigned_short sun_family;
     char sun_path[108];
@@ -621,6 +628,36 @@ struct BaseArch : public wordsize,
   };
   RR_VERIFY_TYPE(termio);
 
+  struct seccomp_notif_sizes {
+    uint16_t seccomp_notif;
+    uint16_t seccomp_notif_resp;
+    uint16_t seccomp_data;
+  };
+  // seccomp_notif_sizes is not present in older kernels
+  // RR_VERIFY_TYPE(seccomp_notif_sizes);
+
+  struct serial_struct {
+    signed_int type;
+    signed_int line;
+    unsigned_int port;
+    signed_int irq;
+    signed_int flags;
+    signed_int xmit_fifo_size;
+    signed_int custom_divisor;
+    signed_int baud_base;
+    unsigned_short close_delay;
+    char io_type;
+    char reserved_char[1];
+    signed_int hub6;
+    unsigned_short closing_wait;
+    unsigned_short closing_wait2;
+    ptr<unsigned char> iomem_base;
+    unsigned_short iomem_reg_shift;
+    unsigned_int port_high;
+    unsigned_long iomap_base;
+  };
+  RR_VERIFY_TYPE(serial_struct);
+
   struct winsize {
     unsigned_short ws_row;
     unsigned_short ws_col;
@@ -1666,6 +1703,134 @@ struct BaseArch : public wordsize,
     ptr<link_map> r_map;
     // More fields we don't need (and are potentially libc specific)
   };
+
+  struct prctl_mm_map {
+    __u64 start_code;
+    __u64 end_code;
+    __u64 start_data;
+    __u64 end_data;
+    __u64 start_brk;
+    __u64 brk;
+    __u64 start_stack;
+    __u64 arg_start;
+    __u64 arg_end;
+    __u64 env_start;
+    __u64 env_end;
+    ptr<__u64> auxv;
+    __u32 auxv_size;
+    __u32 exe_fd;
+  };
+  RR_VERIFY_TYPE(prctl_mm_map);
+
+  struct fiemap_extent {
+    __u64 fe_logical;
+    __u64 fe_physical;
+    __u64 fe_length;
+    __u64 fe_reserved64[2];
+    __u32 fe_flags;
+    __u32 fe_reserved[3];
+  };
+  RR_VERIFY_TYPE(fiemap_extent);
+  struct fiemap {
+    __u64 fm_start;
+    __u64 fm_length;
+    __u32 fm_flags;
+    __u32 fm_mapped_extents;
+    __u32 fm_extent_count;
+    __u32 fm_reserved;
+    struct fiemap_extent fm_extents[0];
+  };
+  RR_VERIFY_TYPE(fiemap);
+
+  struct vt_stat {
+    unsigned short v_active;
+    unsigned short v_signal;
+    unsigned short v_state;
+  };
+  RR_VERIFY_TYPE(vt_stat);
+
+  struct fb_fix_screeninfo {
+    char id[16];
+    unsigned long smem_start;
+    __u32 smem_len;
+    __u32 type;
+    __u32 type_aux;
+    __u32 visual;
+    uint16_t xpanstep;
+    uint16_t ypanstep;
+    uint16_t xwrapstep;
+    __u32 line_length;
+    unsigned long mmio_start;
+    __u32 mmio_len;
+    __u32 accel;
+    uint16_t capabilities;
+    uint16_t reserved[2];
+  };
+  RR_VERIFY_TYPE(fb_fix_screeninfo);
+
+  struct fb_bitfield {
+    __u32 offset;
+    __u32 length;
+    __u32 msb_right;
+  };
+  RR_VERIFY_TYPE(fb_bitfield);
+  struct fb_var_screeninfo {
+    __u32 xres;
+    __u32 yres;
+    __u32 xres_virtual;
+    __u32 yres_virtual;
+    __u32 xoffset;
+    __u32 yoffset;
+    __u32 bits_per_pixel;
+    __u32 grayscale;
+    struct fb_bitfield red;
+    struct fb_bitfield green;
+    struct fb_bitfield blue;
+    struct fb_bitfield transp;
+    __u32 nonstd;
+    __u32 active;
+    __u32 height;
+    __u32 width;
+    __u32 accel_flags;
+    __u32 pixclock;
+    __u32 left_margin;
+    __u32 right_margin;
+    __u32 upper_margin;
+    __u32 lower_margin;
+    __u32 hsync_len;
+    __u32 vsync_len;
+    __u32 sync;
+    __u32 vmode;
+    __u32 rotate;
+    __u32 colorspace;
+    __u32 reserved[4];
+  };
+  RR_VERIFY_TYPE(fb_var_screeninfo);
+
+  struct cdrom_tochdr {
+    uint8_t cdth_trk0;
+    uint8_t cdth_trk1;
+  };
+  RR_VERIFY_TYPE(cdrom_tochdr);
+
+  struct cdrom_msf0 {
+    uint8_t minute;
+    uint8_t second;
+    uint8_t frame;
+  };
+  union cdrom_addr {
+    struct cdrom_msf0 msf;
+    int lba;
+  };
+  struct cdrom_tocentry {
+    uint8_t cdte_track;
+    uint8_t cdte_adr : 4;
+    uint8_t cdte_ctrl : 4;
+    uint8_t cdte_format;
+    union cdrom_addr cdte_addr;
+    uint8_t cdte_datamode;
+  };
+  RR_VERIFY_TYPE(cdrom_tocentry);
 };
 
 struct X64Arch : public BaseArch<SupportedArch::x86_64, WordSize64Defs> {
@@ -1843,6 +2008,30 @@ struct X64Arch : public BaseArch<SupportedArch::x86_64, WordSize64Defs> {
     __kernel_ulong_t __unused4;
   };
   RR_VERIFY_TYPE_ARCH(SupportedArch::x86_64, struct ::semid64_ds, struct semid64_ds);
+
+  struct ethtool_rx_flow_spec {
+    uint32_t flow_type;
+    char h_u[52];
+    char h_ext[20];
+    char m_u[52];
+    char m_ext[20];
+    uint64_t ring_cookie;
+    uint32_t location;
+  };
+  RR_VERIFY_TYPE_ARCH(SupportedArch::x86_64, struct ::ethtool_rx_flow_spec, struct ethtool_rx_flow_spec);
+
+  struct ethtool_rxnfc {
+    uint32_t cmd;
+    uint32_t flow_type;
+    uint64_t data;
+    struct ethtool_rx_flow_spec fs;
+    union {
+      uint32_t rule_cnt;
+      uint32_t rss_context;
+    };
+    uint32_t rule_locs[0];
+  };
+  RR_VERIFY_TYPE_ARCH(SupportedArch::x86_64, struct ::ethtool_rxnfc, struct ethtool_rxnfc);
 };
 
 struct X86Arch : public BaseArch<SupportedArch::x86, WordSize32Defs> {
@@ -1883,6 +2072,7 @@ struct X86Arch : public BaseArch<SupportedArch::x86, WordSize32Defs> {
   // PTRACE_ARCH_PRCTL does not exist on x86
   static const int PTRACE_SYSEMU = Arch64::PTRACE_SYSEMU;
   static const int PTRACE_SYSEMU_SINGLESTEP = Arch64::PTRACE_SYSEMU_SINGLESTEP;
+  static const int RR_AT_SYSINFO = 32;
 
   struct user_regs_struct {
     int32_t ebx;
@@ -2035,6 +2225,30 @@ struct X86Arch : public BaseArch<SupportedArch::x86, WordSize32Defs> {
     __kernel_ulong_t __unused4;
   };
   RR_VERIFY_TYPE_ARCH(SupportedArch::x86, struct ::semid64_ds, struct semid64_ds);
+
+  struct __attribute__((packed)) ethtool_rx_flow_spec {
+    uint32_t flow_type;
+    char h_u[52];
+    char h_ext[20];
+    char m_u[52];
+    char m_ext[20];
+    uint64_t ring_cookie;
+    uint32_t location;
+  };
+  RR_VERIFY_TYPE_ARCH(SupportedArch::x86, struct ::ethtool_rx_flow_spec, struct ethtool_rx_flow_spec);
+
+  struct __attribute__((packed)) ethtool_rxnfc {
+    uint32_t cmd;
+    uint32_t flow_type;
+    uint64_t data;
+    struct ethtool_rx_flow_spec fs;
+    union {
+      uint32_t rule_cnt;
+      uint32_t rss_context;
+    };
+    uint32_t rule_locs[0];
+  };
+  RR_VERIFY_TYPE_ARCH(SupportedArch::x86, struct ::ethtool_rxnfc, struct ethtool_rxnfc);
 };
 
 // Archs that inherit Linux's "generic" data structures
@@ -2158,6 +2372,30 @@ struct ARM64Arch : public GenericArch<SupportedArch::aarch64, WordSize64Defs> {
   };
 
   RR_VERIFY_TYPE_ARCH(SupportedArch::aarch64, struct ::semid64_ds, struct semid64_ds);
+
+  struct ethtool_rx_flow_spec {
+    uint32_t flow_type;
+    char h_u[52];
+    char h_ext[20];
+    char m_u[52];
+    char m_ext[20];
+    uint64_t ring_cookie;
+    uint32_t location;
+  };
+  RR_VERIFY_TYPE_ARCH(SupportedArch::aarch64, struct ::ethtool_rx_flow_spec, struct ethtool_rx_flow_spec);
+
+  struct ethtool_rxnfc {
+    uint32_t cmd;
+    uint32_t flow_type;
+    uint64_t data;
+    struct ethtool_rx_flow_spec fs;
+    union {
+      uint32_t rule_cnt;
+      uint32_t rss_context;
+    };
+    uint32_t rule_locs[0];
+  };
+  RR_VERIFY_TYPE_ARCH(SupportedArch::aarch64, struct ::ethtool_rxnfc, struct ethtool_rxnfc);
 };
 
 #define RR_ARCH_FUNCTION(f, arch, args...)                                     \
