@@ -59,12 +59,12 @@ static char simple_to_lower(char ch) {
 }
 
 static string simple_to_lower(const string& s) {
-  char* buf = new char[s.size() + 1];
+  std::unique_ptr<char[]> buf(new char[s.size() + 1]);
   for (size_t i = 0; i < s.size(); ++i) {
     buf[i] = simple_to_lower(s[i]);
   }
   buf[s.size()] = 0;
-  return string(buf);
+  return string(buf.get());
 }
 
 #if __has_attribute(require_constant_initialization)
@@ -118,7 +118,6 @@ static void init_log_globals() {
     log_buffer_size = atoi(buffer);
     if (log_buffer_size) {
       log_buffer = unique_ptr<deque<char>>(new deque<char>());
-      atexit(flush_log_buffer);
     }
   }
 
@@ -347,6 +346,7 @@ CleanFatalOstream::CleanFatalOstream(const char* file, int line,
 CleanFatalOstream::~CleanFatalOstream() {
   cerr << endl;
   flush_log_stream();
+  flush_log_buffer();
   exit(1);
 }
 
