@@ -34,6 +34,7 @@
 #include "ProcFdDirMonitor.h"
 #include "ProcMemMonitor.h"
 #include "ProcStatMonitor.h"
+#include "RRPageMonitor.h"
 #include "ReplaySession.h"
 #include "ReplayTask.h"
 #include "SeccompFilterRewriter.h"
@@ -261,6 +262,7 @@ template <typename Arch> static void prepare_clone(ReplayTask* t) {
   ReplayTask* new_task = static_cast<ReplayTask*>(
       t->session().clone(t, clone_flags_to_task_flags(flags), params.stack,
                          params.tls, params.ctid, new_tid, rec_tid));
+  new_task->own_namespace_rec_tid = tte.own_ns_tid();
 
   if (Arch::clone == t->regs().original_syscallno()) {
     /* FIXME: what if registers are non-null and contain an
@@ -555,14 +557,46 @@ static void finish_anonymous_mmap(ReplayTask* t, AutoRemoteSyscalls& remote,
                            device, inode, nullptr, &recorded_km, emu_file);
 }
 
+static void write_mapped_data_with_holes(ReplayTask* t, const TraceReader::RawDataWithHoles& buf) {
+  unique_ptr<AutoRemoteSyscalls> remote;
+  size_t data_offset = 0;
+  size_t addr_offset = 0;
+  auto holes_iter = buf.holes.begin();
+  while (data_offset < buf.data.size() || holes_iter != buf.holes.end()) {
+    if (holes_iter != buf.holes.end() && holes_iter->offset == addr_offset) {
+      t->write_zeroes(&remote, buf.addr + addr_offset, holes_iter->size);
+      addr_offset += holes_iter->size;
+      ++holes_iter;
+      continue;
+    }
+    size_t data_end = buf.data.size();
+    if (holes_iter != buf.holes.end()) {
+      data_end = data_offset + holes_iter->offset - addr_offset;
+    }
+    t->write_bytes_helper(buf.addr + addr_offset, data_end - data_offset, buf.data.data() + data_offset,
+                          nullptr);
+    addr_offset += data_end - data_offset;
+    data_offset = data_end;
+  }
+}
+
 static void write_mapped_data(ReplayTask* t,
                               remote_ptr<void> rec_addr,
                               size_t size,
                               TraceReader::MappedData& data) {
   switch (data.source) {
-  case TraceReader::SOURCE_TRACE:
-    t->set_data_from_trace();
+  case TraceReader::SOURCE_TRACE: {
+    TraceReader::RawDataWithHoles buf;
+    ASSERT(t, t->trace_reader().read_raw_data_for_frame_with_holes(buf));
+    ASSERT(t, buf.addr == rec_addr);
+    // Note that this gets called for remaps and shared maps that refer to the same pages
+    // as previous maps and so the data we're recording might not be the initial data
+    // for those pages, but it is the inital data *for this mapping*.
+    write_mapped_data_with_holes(t, buf);
+    t->vm()->maybe_update_breakpoints(t, rec_addr.cast<uint8_t>(),
+                                      buf.data.size());
     break;
+  }
   case TraceReader::SOURCE_FILE: {
     ScopedFd file(data.file_name.c_str(), O_RDONLY);
     ASSERT(t, file.is_open()) << "Can't open " << data.file_name;
@@ -693,6 +727,23 @@ static void process_mmap(ReplayTask* t, const TraceFrame& trace_frame,
         TraceReader::VALIDATE, TraceReader::CURRENT_TIME_ONLY, &extra_fds,
         &skip_monitoring_mapped_fd);
 
+      if (t->session().has_trace_quirk(TraceReader::SpecialLibRRpage)) {
+        FileMonitor *fd_monitor = t->fd_table()->get_monitor(fd);
+        if (fd_monitor && fd_monitor->type() == FileMonitor::RRPage) {
+          if (offset_pages == 0 && !(flags & MAP_FIXED) &&
+              length <= 2*page_size() && addr == (RR_PAGE_ADDR - page_size())) {
+            // We only mapped the first page during record. Do the same here
+            length = page_size();
+          }
+          if (offset_pages == 1 && length == page_size() &&
+              addr == RR_PAGE_ADDR && t->vm()->has_rr_page()) {
+            // We skipped this during recording. Setting length to zero here
+            // will have the same effect.
+            length = 0;
+          }
+        }
+      }
+
       if (data.source == TraceReader::SOURCE_FILE &&
           data.file_size_bytes > data.data_offset_bytes) {
         struct stat real_file;
@@ -732,7 +783,7 @@ static void process_mmap(ReplayTask* t, const TraceFrame& trace_frame,
     // tracer and the tracee. Instead, only mappings that have
     // sufficiently many memory access from the tracer to require
     // acceleration should be shared.
-    if (!(MAP_SHARED & flags) && t->session().flags().share_private_mappings) {
+    if (length > 0 && !(MAP_SHARED & flags) && t->session().flags().share_private_mappings) {
       Session::make_private_shared(remote, t->vm()->mapping_of(addr));
     }
 
@@ -1011,7 +1062,7 @@ static void handle_opened_files(ReplayTask* t, int flags) {
     if (emu_file) {
       file_monitor = new MmappedFileMonitor(t, emu_file);
     } else if (o.path == "terminal") {
-      file_monitor = new StdioMonitor(STDERR_FILENO);
+      file_monitor = new StdioMonitor(t->session().tracee_output_fd(STDERR_FILENO));
     } else if (is_proc_mem_file(o.path.c_str())) {
       file_monitor = new ProcMemMonitor(t, o.path);
     } else if (is_proc_fd_dir(o.path.c_str())) {
@@ -1020,6 +1071,8 @@ static void handle_opened_files(ReplayTask* t, int flags) {
       file_monitor = new SysCpuMonitor(t, o.path);
     } else if (is_proc_stat_file(o.path.c_str())) {
       file_monitor = new ProcStatMonitor(t, o.path);
+    } else if (is_rr_page_lib(o.path.c_str())) {
+      file_monitor = new RRPageMonitor();
     } else if (flags & O_DIRECT) {
       file_monitor = new FileMonitor();
     } else {
@@ -1074,6 +1127,7 @@ static void rep_process_syscall_arch(ReplayTask* t, ReplayTraceStep* step,
     switch (non_negative_syscall(sys)) {
       case Arch::madvise:
       case Arch::mprotect:
+      case Arch::pkey_mprotect:
       case Arch::sigreturn:
       case Arch::rt_sigreturn:
         break;
@@ -1161,19 +1215,21 @@ static void rep_process_syscall_arch(ReplayTask* t, ReplayTraceStep* step,
     case Arch::munmap:
     case Arch::mprotect:
     case Arch::modify_ldt:
+    case Arch::pkey_mprotect:
     case Arch::set_thread_area: {
       // Using AutoRemoteSyscalls here fails for arch_prctl, not sure why.
       Registers r = t->regs();
-      r.set_syscallno(sys);
+      int modified_sys = sys == Arch::pkey_mprotect ? Arch::mprotect : sys;
+      r.set_syscallno(modified_sys);
       r.set_ip(r.ip().decrement_by_syscall_insn_length(r.arch()));
       t->set_regs(r);
-      if (sys == Arch::mprotect) {
+      if (modified_sys == Arch::mprotect) {
         t->vm()->fixup_mprotect_growsdown_parameters(t);
       }
       t->enter_syscall();
       t->exit_syscall();
       ASSERT(t, t->regs().syscall_result() == trace_regs.syscall_result());
-      if (sys == Arch::mprotect) {
+      if (modified_sys == Arch::mprotect) {
         Registers r2 = t->regs();
         r2.set_arg1(r.arg1());
         r2.set_arg2(r.arg2());
