@@ -546,7 +546,7 @@ statfs = EmulatedSyscall(x86=99, x64=137, generic=43, arg2="struct Arch::statfs"
 # to a statfs structure defined approximately as follows:
 fstatfs = EmulatedSyscall(x86=100, x64=138, generic=44, arg2="struct Arch::statfs")
 
-ioperm = UnsupportedSyscall(x86=101, x64=173)
+ioperm = EmulatedSyscall(x86=101, x64=173)
 
 #  int socketcall(int call, unsigned long *args)
 #
@@ -570,7 +570,7 @@ stat = EmulatedSyscall(x86=106, x64=4, arg2="struct Arch::stat")
 lstat = EmulatedSyscall(x86=107, x64=6, arg2="struct Arch::stat")
 fstat = EmulatedSyscall(x86=108, x64=5, generic=80, arg2="struct Arch::stat")
 olduname = UnsupportedSyscall(x86=109)
-iopl = UnsupportedSyscall(x86=110, x64=172)
+iopl = EmulatedSyscall(x86=110, x64=172)
 vhangup = UnsupportedSyscall(x86=111, x64=153, generic=58)
 idle = UnsupportedSyscall(x86=112)
 vm86old = UnsupportedSyscall(x86=113)
@@ -1383,8 +1383,8 @@ waitid = IrregularEmulatedSyscall(x86=284, x64=247, generic=95)
 add_key = EmulatedSyscall(x86=286, x64=248, generic=217)
 request_key = UnsupportedSyscall(x86=287, x64=249, generic=218)
 keyctl = IrregularEmulatedSyscall(x86=288, x64=250, generic=219)
-ioprio_set = UnsupportedSyscall(x86=289, x64=251, generic=30)
-ioprio_get = UnsupportedSyscall(x86=290, x64=252, generic=31)
+ioprio_set = EmulatedSyscall(x86=289, x64=251, generic=30)
+ioprio_get = EmulatedSyscall(x86=290, x64=252, generic=31)
 
 #  int inotify_init(void)
 #
@@ -1459,6 +1459,13 @@ fchmodat = EmulatedSyscall(x86=306, x64=268, generic=53)
 # page....
 faccessat = EmulatedSyscall(x86=307, x64=269, generic=48)
 
+#  int faccessat2(int dirfd, const char *pathname, int mode, int flags)
+#
+# The faccessat2() system call operates in exactly the same way as
+# access(2), except for the differences described in this manual
+# page...
+faccessat2 = EmulatedSyscall(x86=439, x64=439, generic=439)
+
 pselect6 = IrregularEmulatedSyscall(x86=308, x64=270, generic=72)
 
 ppoll = IrregularEmulatedSyscall(x86=309, x64=271, generic=73)
@@ -1664,18 +1671,18 @@ arch_prctl = IrregularEmulatedSyscall(x86=384, x64=158)
 
 bpf = IrregularEmulatedSyscall(x86=357, x64=321, generic=280)
 execveat = UnsupportedSyscall(x86=358, x64=322, generic=281)
-userfaultfd = UnsupportedSyscall(x86=374, x64=323, generic=282)
+userfaultfd = IrregularEmulatedSyscall(x86=374, x64=323, generic=282)
 membarrier = EmulatedSyscall(x86=375, x64=324, generic=283)
 mlock2 = UnsupportedSyscall(x86=376, x64=325, generic=284)
 copy_file_range = IrregularEmulatedSyscall(x86=377, x64=326, generic=285)
 preadv2 = UnsupportedSyscall(x86=378, x64=327, generic=286)
 pwritev2 = UnsupportedSyscall(x86=379, x64=328, generic=287)
-pkey_mprotect = UnsupportedSyscall(x86=380, x64=329, generic=288)
-pkey_alloc = UnsupportedSyscall(x86=381, x64=330, generic=289)
-pkey_free = UnsupportedSyscall(x86=382, x64=331, generic=290)
+pkey_mprotect = IrregularEmulatedSyscall(x86=380, x64=329, generic=288)
+pkey_alloc = EmulatedSyscall(x86=381, x64=330, generic=289)
+pkey_free = EmulatedSyscall(x86=382, x64=331, generic=290)
 statx = EmulatedSyscall(x86=383, x64=332, generic=291, arg5="typename Arch::statx_struct")
 io_pgetevents = UnsupportedSyscall(x86=385, x64=333, generic=292)
-rseq = UnsupportedSyscall(x86=386, x64=334, generic=293)
+rseq = IrregularEmulatedSyscall(x86=386, x64=334, generic=293)
 
 clock_gettime64 = EmulatedSyscall(x86=403, arg2="typename Arch::Arch64::timespec")
 clock_settime64 = UnsupportedSyscall(x86=404)
@@ -1700,7 +1707,7 @@ sched_rr_get_interval_time64 = UnsupportedSyscall(x86=423)
 
 # x86-64 decided to skip ahead here to catchup
 pidfd_send_signal = UnsupportedSyscall(x86=424, x64=424, generic=424)
-io_uring_setup = UnsupportedSyscall(x86=425, x64=425, generic=425)
+io_uring_setup = IrregularEmulatedSyscall(x86=425, x64=425, generic=425)
 io_uring_enter = UnsupportedSyscall(x86=426, x64=426, generic=426)
 io_uring_register = UnsupportedSyscall(x86=427, x64=427, generic=427)
 open_tree = UnsupportedSyscall(x86=428, x64=428, generic=428)
@@ -1710,9 +1717,17 @@ fsconfig = UnsupportedSyscall(x86=431, x64=431, generic=431)
 fsmount = UnsupportedSyscall(x86=432, x64=432, generic=432)
 fspick = UnsupportedSyscall(x86=433, x64=433, generic=433)
 pidfd_open = EmulatedSyscall(x86=434, x64=434, generic=434)
-clone3 = UnsupportedSyscall(x86=435, x64=435, generic=435)
+clone3 = IrregularEmulatedSyscall(x86=435, x64=435, generic=435)
 openat2 = UnsupportedSyscall(x86=437, x64=437, generic=437)
 pidfd_getfd = UnsupportedSyscall(x86=438, x64=438, generic=438)
+process_madvise = UnsupportedSyscall(x86=440, x64=440, generic=440)
+epoll_pwait2 = UnsupportedSyscall(x86=441, x64=441, generic=441)
+mount_setattr = UnsupportedSyscall(x86=442, x64=442, generic=442)
+quotactl_fd = UnsupportedSyscall(x86=443, x64=443, generic=443)
+landlock_create_ruleset = UnsupportedSyscall(x86=444, x64=444, generic=444)
+landlock_add_rule = UnsupportedSyscall(x86=445, x64=445, generic=445)
+landlock_restrict_self = UnsupportedSyscall(x86=446, x64=446, generic=446)
+memfd_secret = UnsupportedSyscall(x86=447, x64=447, generic=447)
 
 # restart_syscall is a little special.
 restart_syscall = RestartSyscall(x86=0, x64=219, generic=128)
@@ -1727,6 +1742,8 @@ rrcall_reload_auxv = IrregularEmulatedSyscall(x86=1004, x64=1004, generic=1004)
 rrcall_mprotect_record = IrregularEmulatedSyscall(x86=1005, x64=1005, generic=1005)
 rrcall_notify_stap_semaphore_added = IrregularEmulatedSyscall(x86=1006, x64=1006, generic=1006)
 rrcall_notify_stap_semaphore_removed = IrregularEmulatedSyscall(x86=1007, x64=1007, generic=1007)
+rrcall_check_presence = IrregularEmulatedSyscall(x86=1008, x64=1008, generic=1008)
+rrcall_detach_teleport = IrregularEmulatedSyscall(x86=1009, x64=1009, generic=1009)
 
 # These syscalls also appear under `socketcall` on x86.
 socket = EmulatedSyscall(x86=359, x64=41, generic=198)
