@@ -1,6 +1,6 @@
 /* -*- Mode: C; tab-width: 8; c-basic-offset: 2; indent-tabs-mode: nil; -*- */
 
-#include "rrutil.h"
+#include "util.h"
 
 #ifndef ALSA_DEVICE_DIRECTORY
 #define ALSA_DEVICE_DIRECTORY "/dev/snd/"
