@@ -17,13 +17,17 @@ stopped_locations = {
     'i386': ['(0x[0-9a-f]+ in )?__kernel_vsyscall',
              '(0x[0-9a-f]+ in )?_traced_raw_syscall',
              '0x[0-9a-f]+ in \?\?',
+             '(0x[0-9a-f]+ in )?syscall_traced',
+             '(0x[0-9a-f]+ in )?rr_page_start',
              '(0x[0-9a-f]+ in )?__lll_lock_wait',
              '(0x[0-9a-f]+ in )?pthread_barrier_wait',
              '(0x[0-9a-f]+ in )?futex_wait'],
     'i386:x86-64': ['(0x[0-9a-f]+ in )?__lll_lock_wait',
                     '(0x[0-9a-f]+ in )?pthread_barrier_wait',
                     '(0x[0-9a-f]+ in )?futex_wait',
-                    '0x0*70000002 in \?\?'],
+                    '0x0*70000002 in \?\?',
+                    '(0x[0-9a-f]+ in )?syscall_traced',
+                    '(0x[0-9a-f]+ in )?rr_page_start'],
     'aarch64': ['(0x[0-9a-f]+ in )?futex_wait']
 }
 
