@@ -9,7 +9,7 @@ int main(void) {
   ALLOCATE_GUARD(cpu, -1);
   ALLOCATE_GUARD(node, -1);
   /* The 'tcache' parameter is unused in all kernels rr works on */
-  test_assert(0 == syscall(SYS_getcpu, cpu, node, NULL));
+  test_assert(0 == syscall(SYS_getcpu, cpu, node, (void*)0));
   test_assert(*cpu <= 0xffffff);
   test_assert(*node <= 0xffffff);
   VERIFY_GUARD(cpu);
