@@ -83,7 +83,8 @@ void munmap_remote(pid_t child, uintptr_t start, size_t size) {
 static void remote_unmap_callback(uint64_t child, char* name,
                                   map_properties_t* props) {
   if ((props->start <= child_syscall_addr && child_syscall_addr < props->end) ||
-      props->start == RR_PAGE_ADDR || strcmp(name, "[vsyscall]") == 0) {
+      (props->start <= RR_PAGE_ADDR && props->end > RR_PAGE_ADDR) ||
+      strcmp(name, "[vsyscall]") == 0) {
     return;
   }
 
