@@ -1,30 +1,7 @@
 /* -*- Mode: C; tab-width: 8; c-basic-offset: 2; indent-tabs-mode: nil; -*- */
 
 #include "util.h"
-
-#if defined(__i386__)
-#define ORIG_SYSCALLNO orig_eax
-#elif defined(__x86_64__)
-#define ORIG_SYSCALLNO orig_rax
-#else
-#error unknown architecture
-#endif
-
-#if defined(__i386__)
-#define SYSCALL_RESULT eax
-#elif defined(__x86_64__)
-#define SYSCALL_RESULT rax
-#else
-#error unknown architecture
-#endif
-
-#if defined(__i386__)
-#define IP eip
-#elif defined(__x86_64__)
-#define IP rip
-#else
-#error unknown architecture
-#endif
+#include "ptrace_util.h"
 
 #ifdef SYS_geteuid32
 #define SYSCALLNO SYS_geteuid32
@@ -32,30 +9,7 @@
 #define SYSCALLNO SYS_geteuid
 #endif
 
-extern char syscall_addr;
-
-/* Make a syscallbuf-patchable syscall to check that syscallbuf patching
-   doesn't happen when we are emulating a ptracer --- which can be
-   potentially confused by it. */
-static uid_t my_geteuid(void) {
-  int r;
-#ifdef __i386__
-  __asm__ __volatile__("syscall_addr: int $0x80\n\t"
-                       "nop\n\t"
-                       "nop\n\t"
-                       "nop\n\t"
-                       : "=a"(r)
-                       : "a"(SYS_geteuid32));
-#elif defined(__x86_64__)
-  __asm__ __volatile__("syscall_addr: syscall\n\t"
-                       "nop\n\t"
-                       "nop\n\t"
-                       "nop\n\t"
-                       : "=a"(r)
-                       : "a"(SYS_geteuid));
-#endif
-  return r;
-}
+extern char syscall_addr __attribute__ ((visibility ("hidden")));
 
 int main(void) {
   pid_t child;
@@ -80,23 +34,27 @@ int main(void) {
   test_assert(0 == ptrace(PTRACE_SYSCALL, child, NULL, (void*)0));
   test_assert(child == waitpid(child, &status, 0));
   test_assert(status == (((0x80 | SIGTRAP) << 8) | 0x7f));
-  test_assert(0 == ptrace(PTRACE_GETREGS, child, NULL, &regs));
+  ptrace_getregs(child, &regs);
   /* This assert will fail if we patched the syscall for syscallbuf. */
-  test_assert(&syscall_addr + 2 == (char*)regs.IP);
+  test_assert(&syscall_addr + SYSCALL_SIZE == (char*)regs.IP);
   test_assert(SYSCALLNO == regs.ORIG_SYSCALLNO);
+#if !defined(__aarch64__)
   test_assert(-ENOSYS == (int)regs.SYSCALL_RESULT);
-  regs.ORIG_SYSCALLNO = SYS_gettid;
-  test_assert(0 == ptrace(PTRACE_SETREGS, child, NULL, &regs));
+#endif
+  ptrace_change_syscall(child, &regs, SYS_gettid);
 
   test_assert(0 == ptrace(PTRACE_SYSCALL, child, NULL, (void*)0));
   test_assert(child == waitpid(child, &status, 0));
   test_assert(status == (((0x80 | SIGTRAP) << 8) | 0x7f));
-  test_assert(0 == ptrace(PTRACE_GETREGS, child, NULL, &regs));
+  ptrace_getregs(child, &regs);
+#if !defined(__aarch64__)
+  // On aarch64 we're only allowed to ask this during a syscall-entry stop
   test_assert(SYS_gettid == regs.ORIG_SYSCALLNO);
+#endif
   test_assert(child == (int)regs.SYSCALL_RESULT);
-  test_assert(&syscall_addr + 2 == (char*)regs.IP);
+  test_assert(&syscall_addr + SYSCALL_SIZE == (char*)regs.IP);
   regs.SYSCALL_RESULT = uid + 1;
-  test_assert(0 == ptrace(PTRACE_SETREGS, child, NULL, &regs));
+  ptrace_setregs(child, &regs);
 
   test_assert(0 == ptrace(PTRACE_CONT, child, NULL, (void*)0));
   test_assert(child == waitpid(child, &status, 0));
