@@ -1,29 +1,14 @@
 /* -*- Mode: C; tab-width: 8; c-basic-offset: 2; indent-tabs-mode: nil; -*- */
 
 #include "util.h"
+#include "ptrace_util.h"
 
 /* This test tests the interaction between PTRACE_SYSEMU and PTRACE_SYSCALL.
  * In addition, it also tests the behavior of PTRACE_SYSEMU when the entering
  * syscall number is invalid (it will be -ENOSYS for the second syscall).
  */
 
-#ifndef PTRACE_SYSEMU
-#define PTRACE_SYSEMU 31
-#endif
-
-#if defined(__i386__)
-#define SYSCALL_RESULT eax
-#define SYSCALL_GETEUID SYS_geteuid32
-#define IP eip
-#elif defined(__x86_64__)
-#define SYSCALL_RESULT rax
-#define SYSCALL_GETEUID SYS_geteuid
-#define IP rip
-#else
-#error unknown architecture
-#endif
-
-extern char syscall1_addr;
+extern char syscall1_addr __attribute__ ((visibility ("hidden")));
 int main(void) {
   pid_t child;
   int status;
@@ -37,14 +22,22 @@ int main(void) {
                          "nop\n\t"
                          "int $0x80\n\t"
                          "nop\n\t"
-                         "int $3\n\t" ::"a"(SYSCALL_GETEUID));
+                         "int $3\n\t" ::"a"(SYS_geteuid32));
 #elif defined(__x86_64__)
     __asm__ __volatile__("int $3\n\t"
                          "syscall1_addr: syscall\n\t"
                          "nop\n\t"
                          "syscall\n\t"
                          "nop\n\t"
-                         "int $3\n\t" ::"a"(SYSCALL_GETEUID));
+                         "int $3\n\t" ::"a"(SYS_geteuid));
+#elif defined(__aarch64__)
+    register long x8 __asm__("x8") = SYS_geteuid;
+    __asm__ __volatile__("brk #0\n\t"
+                         "syscall1_addr: svc #0\n\t"
+                         "nop\n\t"
+                         "svc #0\n\t"
+                         "nop\n\t"
+                         "brk #0\n\t" :: "r"(x8));
 #else
 #error Add support for new architecture here
 #endif
@@ -56,6 +49,8 @@ int main(void) {
   test_assert(WIFSTOPPED(status) && WSTOPSIG(status) == SIGTRAP);
   test_assert(0 == ptrace(PTRACE_SETOPTIONS, child, 0, PTRACE_O_TRACESYSGOOD));
 
+  skip_breakpoint(child);
+
   /* Should step to syscall entry */
   test_assert(0 == ptrace(PTRACE_SYSEMU, child, 0, 0));
   test_assert(child == waitpid(child, &status, 0));
@@ -68,9 +63,11 @@ int main(void) {
 
   /* If we hadn't skipped the syscall the register would now contain the result
    */
-  test_assert(0 == ptrace(PTRACE_GETREGS, child, NULL, &regs));
+  ptrace_getregs(child, &regs);
+#if !defined(__aarch64__)
   test_assert((uintptr_t)regs.SYSCALL_RESULT == (uintptr_t)-ENOSYS);
-  test_assert((uintptr_t)regs.IP == (uintptr_t)&syscall1_addr + 2);
+#endif
+  test_assert((uintptr_t)regs.IP == (uintptr_t)&syscall1_addr + SYSCALL_SIZE);
 
   /* Should step to syscall entry */
   test_assert(0 == ptrace(PTRACE_SYSEMU, child, 0, 0));
