@@ -12,7 +12,7 @@ int main(void) {
   test_assert(sizeof(buf) == write(fd, buf, sizeof(buf)));
 
   test_assert(0 == lseek(fd, 0, SEEK_SET));
-  ret = read(fd, buf, UINTPTR_MAX - 0xfff);
+  ret = syscall(RR_read, fd, buf, UINTPTR_MAX - 0xfff);
   if (ret < 0) {
     /* x86-64 returns EFAULT here. I'm not sure why. */
     test_assert(EFAULT == errno);
