@@ -9,8 +9,9 @@ static char syscall_bytes[] =
     { 0x0f, 0x05, 0xc3 }
 #elif __i386__
     { 0xcd, 0x80, 0xc3 }
-#else
-#error Unknown architecture
+#elif defined(__aarch64__)
+    { 0x01, 0x00, 0x00, 0xd4,   // svc #0
+      0xc0, 0x03, 0x5f, 0xd6 } // ret
 #endif
 ;
 
@@ -25,6 +26,16 @@ static void do_write(int fd, const char* p) {
   __asm__ __volatile__("call *%%esi\n\t"
                        : "=a"(ret)
                        : "a"(SYS_write), "S"(ptr), "b"(fd), "c"(p), "d"(len));
+#elif __aarch64__
+  register long x8 __asm__("x8") = SYS_write;
+  register long x7 __asm__("x7") = (long)ptr;
+  register long x0 __asm__("x0") = (long)fd;
+  register long x1 __asm__("x1") = (long)p;
+  register long x2 __asm__("x2") = (long)len;
+  __asm__ __volatile__("blr x7\n\t"
+                       : "+r"(x0)
+                       : "r"(x1), "r"(x2), "r"(x7), "r"(x8));
+  ret = x0;
 #else
 #error Unknown architecture
 #endif
