@@ -29,13 +29,17 @@
 #include <linux/aio_abi.h>
 #include <linux/audit.h>
 #include <linux/capability.h>
+#include <linux/cdrom.h>
 #include <linux/ethtool.h>
+#include <linux/fb.h>
+#include <linux/fiemap.h>
 #include <linux/filter.h>
 #include <linux/fs.h>
 #include <linux/futex.h>
 #include <linux/if.h>
 #include <linux/if_packet.h>
 #include <linux/if_tun.h>
+#include <linux/kd.h>
 #include <linux/limits.h>
 #include <linux/mtio.h>
 #include <linux/netlink.h>
@@ -46,6 +50,7 @@
 #include <linux/sockios.h>
 #include <linux/unistd.h>
 #include <linux/videodev2.h>
+#include <linux/vt.h>
 #include <linux/wireless.h>
 #include <mqueue.h>
 #include <poll.h>
@@ -61,6 +66,7 @@
 #include <stdint.h>
 #include <stdio.h>
 #include <string.h>
+#include <sys/auxv.h>
 #include <sys/epoll.h>
 #include <sys/eventfd.h>
 #include <sys/fanotify.h>
@@ -106,6 +112,7 @@
 // X86 specific headers
 #if defined(__i386__) || defined(__x86_64__)
 #include <asm/prctl.h>
+#include <sys/io.h>
 #include <x86intrin.h>
 #endif
 
@@ -389,6 +396,12 @@ inline static SyscallWrapper get_spurious_desched_syscall(void) {
 #ifndef SECCOMP_FILTER_FLAG_TSYNC
 #define SECCOMP_FILTER_FLAG_TSYNC 1
 #endif
+#ifndef SECCOMP_GET_ACTION_AVAIL
+#define SECCOMP_GET_ACTION_AVAIL 2
+#endif
+#ifndef SECCOMP_GET_NOTIF_SIZES
+#define SECCOMP_GET_NOTIF_SIZES 3
+#endif
 
 /* Old systems don't have linux/kcmp.h */
 #define RR_KCMP_FILE 0
