@@ -20,6 +20,7 @@
 #include <sys/socket.h>
 #include <sys/syscall.h>
 #include <sys/time.h>
+#include <sys/uio.h>
 #include <sys/vfs.h>
 #include <unistd.h>
 
@@ -101,14 +102,39 @@ vector<uint8_t> read_auxv(Task* t) {
   RR_ARCH_FUNCTION(read_auxv_arch, t->arch(), t);
 }
 
-template <typename Arch> void patch_auxv_vdso_arch(RecordTask* t) {
+remote_ptr<void> read_interpreter_base(std::vector<uint8_t> auxv) {
+  if (auxv.size() == 0 || (auxv.size() % sizeof(uint64_t) != 0)) {
+    // Corrupted or missing auxv
+    return nullptr;
+  }
+  remote_ptr<void> interpreter_base = nullptr;
+  for (size_t i = 0; i < (auxv.size() / sizeof(uint64_t)) - 1; i += 2) {
+    uint64_t* entry = ((uint64_t*)auxv.data())+i;
+    uint64_t kind = entry[0];
+    uint64_t value = entry[1];
+    if (kind == AT_BASE) {
+      interpreter_base = value;
+      break;
+    }
+  }
+  return interpreter_base;
+}
+
+std::string read_ld_path(Task* t, remote_ptr<void> interpreter_base) {
+  if (!interpreter_base || !t->vm()->has_mapping(interpreter_base)) {
+    return {};
+  }
+  return t->vm()->mapping_of(interpreter_base).map.fsname();
+}
+
+template <typename Arch> void patch_auxv_vdso_arch(RecordTask* t, uintptr_t search, uintptr_t new_entry_native) {
   auto stack_ptr = auxv_ptr<Arch>(t);
   std::vector<uint8_t> v = read_auxv_arch<Arch>(t, stack_ptr);
   size_t wsize = sizeof(typename Arch::unsigned_word);
   for (int i = 0; (i + 1)*wsize*2 <= v.size(); ++i) {
-    if (*((typename Arch::unsigned_word*)(v.data() + i*2*wsize)) == AT_SYSINFO_EHDR) {
-      auto entry_ptr = stack_ptr + i*2;
-      typename Arch::unsigned_word new_entry = AT_IGNORE;
+    if (*((typename Arch::unsigned_word*)(v.data() + i*2*wsize)) == search) {
+      auto entry_ptr = stack_ptr + i*2 + 1;
+      typename Arch::unsigned_word new_entry = new_entry_native;
       t->write_mem(entry_ptr, new_entry);
       t->record_local(entry_ptr, &new_entry);
       return;
@@ -117,8 +143,8 @@ template <typename Arch> void patch_auxv_vdso_arch(RecordTask* t) {
   return;
 }
 
-void patch_auxv_vdso(RecordTask* t) {
-  RR_ARCH_FUNCTION(patch_auxv_vdso_arch, t->arch(), t);
+void patch_auxv_vdso(RecordTask* t, uintptr_t search, uintptr_t new_entry) {
+  RR_ARCH_FUNCTION(patch_auxv_vdso_arch, t->arch(), t, search, new_entry);
 }
 
 template <typename Arch> static vector<string> read_env_arch(Task* t) {
@@ -451,6 +477,9 @@ static void iterate_checksums(Task* t, ChecksumMode mode,
         FATAL() << "Segment " << parsed.start << "-" << parsed.end
                 << " changed to " << m.map << "??";
       }
+      // As |m| may have changed in the above for loop we need to update the
+      // |raw_map_line| too.
+      raw_map_line = m.map.str();
       ASSERT(t, m.map.end() == parsed.end)
           << "Segment " << parsed.start << "-" << parsed.end
           << " changed to " << m.map << "??";
@@ -852,6 +881,15 @@ static CPUIDRecord cpuid_record(uint32_t eax, uint32_t ecx) {
   return result;
 }
 
+bool is_cpu_vendor_amd(CPUIDData vendor_string)
+{
+  char vendor[12];
+  memcpy(&vendor[0], &vendor_string.ebx, 4);
+  memcpy(&vendor[4], &vendor_string.edx, 4);
+  memcpy(&vendor[8], &vendor_string.ecx, 4);
+  return strncmp(vendor, "AuthenticAMD", sizeof(vendor)) == 0;
+}
+
 static vector<CPUIDRecord> gather_cpuid_records(uint32_t up_to) {
   vector<CPUIDRecord> results;
   CPUIDRecord vendor_string = cpuid_record(CPUID_GETVENDORSTRING, UINT32_MAX);
@@ -974,11 +1012,30 @@ static vector<CPUIDRecord> gather_cpuid_records(uint32_t up_to) {
     return results;
   }
 
+  bool is_amd = is_cpu_vendor_amd(vendor_string.out);
+
   CPUIDRecord extended_info = cpuid_record(CPUID_INTELEXTENDED, UINT32_MAX);
   results.push_back(extended_info);
-  int extended_info_max = min(up_to, extended_info.out.eax);
-  for (int extended = CPUID_INTELEXTENDED + 1; extended <= extended_info_max;
+  uint32_t extended_info_max = min(up_to, extended_info.out.eax);
+  for (uint32_t extended = CPUID_INTELEXTENDED + 1; extended <= extended_info_max;
        ++extended) {
+    if (is_amd) {
+      if (extended == CPUID_AMD_CACHE_TOPOLOGY) {
+        for (int level = 0;; ++level) {
+          CPUIDRecord rec = cpuid_record(extended, level);
+          results.push_back(rec);
+          if (!(rec.out.eax & 0x1f)) {
+            // CacheType: null, no more caches
+            break;
+          }
+        }
+        continue;
+      } else if (extended == CPUID_AMD_PLATFORM_QOS) {
+        cpuid_record(extended, 0);
+        cpuid_record(extended, 1);
+        continue;
+      }
+    }
     results.push_back(cpuid_record(extended, UINT32_MAX));
   }
 
@@ -1005,7 +1062,7 @@ bool cpuid_faulting_works() {
   did_check_cpuid_faulting = true;
 
   // Test to see if CPUID faulting works.
-  if (RR_ARCH_PRCTL(ARCH_SET_CPUID, 0) != 0) {
+  if (RR_ARCH_PRCTL(ARCH_SET_CPUID, 0L) != 0) {
     LOG(debug) << "CPUID faulting not supported by kernel/hardware";
     return false;
   }
@@ -1032,7 +1089,7 @@ bool cpuid_faulting_works() {
   if (sigaction(SIGSEGV, &old_sa, NULL) < 0) {
     FATAL() << "Can't restore sighandler";
   }
-  if (RR_ARCH_PRCTL(ARCH_SET_CPUID, 1) < 0) {
+  if (RR_ARCH_PRCTL(ARCH_SET_CPUID, 1L) < 0) {
     FATAL() << "Can't restore ARCH_SET_CPUID";
   }
   return cpuid_faulting_ok;
@@ -1064,6 +1121,10 @@ bool cpuid_compatible(const vector<CPUIDRecord>& trace_records) {
 }
 
 bool cpu_has_xsave_fip_fdp_quirk() {
+  CPUIDData features = cpuid(CPUID_GETFEATURES, 0);
+  if ((features.ecx & XSAVE_FEATURE_FLAG) == 0) {
+    return false;
+  }
 #if defined(__i386__) || defined(__x86_64__)
   uint64_t xsave_buf[576/sizeof(uint64_t)] __attribute__((aligned(64)));
   xsave_buf[1] = 0;
@@ -1221,13 +1282,33 @@ bool running_under_rr(bool cache) {
   static bool rr_check_done = false;
   if (!rr_check_done || !cache) {
     rr_check_done = true;
-    int ret = syscall(SYS_rrcall_check_presence, 0, 0, 0, 0, 0, 0);
-    DEBUG_ASSERT(ret == 0 || (ret == -1 && errno == ENOSYS));
+    int ret = syscall(SYS_rrcall_check_presence, (uintptr_t)0, (uintptr_t)0,
+      (uintptr_t)0, (uintptr_t)0, (uintptr_t)0, (uintptr_t)0);
+    if (ret > 0 || (ret < 0 && errno != ENOSYS)) {
+      FATAL() << "Unexpected result for rrcall_check_presence: " << ret;
+    }
     rr_under_rr = ret == 0;
   }
   return rr_under_rr;
 }
 
+string find_helper_library(const char *basepath)
+{
+  string lib_path = resource_path() + "lib64/rr/";
+  string file_name = lib_path + basepath;
+  if (access(file_name.c_str(), F_OK) == 0) {
+    return lib_path;
+  }
+  lib_path = resource_path() + "lib/rr/";
+  file_name = lib_path + basepath;
+  if (access(file_name.c_str(), F_OK) == 0) {
+    return lib_path;
+  }
+  // File does not exist. Assume install put it in LD_LIBRARY_PATH.
+  lib_path = "";
+  return lib_path;
+}
+
 vector<string> read_proc_status_fields(pid_t tid, const char* name,
                                        const char* name2, const char* name3) {
   vector<string> result;
@@ -1278,6 +1359,173 @@ bool uses_invisible_guard_page() {
   return !is_pax_kernel;
 }
 
+bool read_proc_net_socket_addresses(Task* t, int fd,
+                                    std::array<typename NativeArch::sockaddr_storage, 2> &out) {
+  char buf[1000];
+
+  auto stat = t->stat_fd(fd);
+  ino_t inode = stat.st_ino;
+
+  // Check tcp first.
+  if (FILE* f = fopen("/proc/net/tcp", "r")) {
+    // Skip the first line, which contains human-readable column labels.
+    if (!fgets(buf, sizeof(buf) - 1, f)) {
+      LOG(warn) << "Can't read /proc/net/tcp";
+    }
+    while (fgets(buf, sizeof(buf) - 1, f)) {
+      // Ignore the first field.
+      strtok(buf, " ");
+      // Then come the local and remote addresses.
+      char* local_addr_str = strtok(NULL, " ");
+      char* remote_addr_str = strtok(NULL, " ");
+      // Then six more fields to ignore.
+      strtok(NULL, " ");
+      strtok(NULL, " ");
+      strtok(NULL, " ");
+      strtok(NULL, " ");
+      strtok(NULL, " ");
+      strtok(NULL, " ");
+      // Finally the inode
+      char* ino = strtok(NULL, " ");
+      ino_t inod = strtoull(ino, NULL, 10);
+      if (inod != inode) {
+        continue;
+      }
+
+      // We have a match.
+      char* next = NULL;
+      unsigned long laddr = strtoul(local_addr_str, &next, 16);
+      if ((laddr == ULONG_MAX && errno == ERANGE) || *next != ':') {
+        LOG(warn) << "Local address not in expected format";
+        break;
+      }
+      ++next;
+      unsigned long lport = strtoul(next, NULL, 16);
+      if (lport > USHRT_MAX) {
+        LOG(warn) << "Local address not in expected format";
+        break;
+      }
+      struct sockaddr_in* local_addr_in = (struct sockaddr_in*)&out[0];
+      local_addr_in->sin_family = AF_INET;
+      local_addr_in->sin_addr.s_addr = laddr;
+      local_addr_in->sin_port = lport;
+      unsigned long raddr = strtoul(remote_addr_str, &next, 16);
+      if ((raddr == ULONG_MAX && errno == ERANGE) || *next != ':') {
+        LOG(warn) << "Remote address not in expected format";
+        break;
+      }
+      ++next;
+      unsigned long rport = strtoul(next, NULL, 16);
+      if (rport > USHRT_MAX) {
+        LOG(warn) << "Remote address not in expected format";
+        break;
+      }
+      struct sockaddr_in* remote_addr_in = (struct sockaddr_in*)&out[1];
+      remote_addr_in->sin_family = AF_INET;
+      remote_addr_in->sin_addr.s_addr = raddr;
+      remote_addr_in->sin_port = rport;
+      fclose(f);
+      return true;
+    }
+    fclose(f);
+  } else {
+    LOG(warn) << "Can't open /proc/net/tcp";
+  }
+
+  // Then tcp6
+  if (FILE* f = fopen("/proc/net/tcp6", "r")) {
+    // Skip the first line, which contains human-readable column labels.
+    if (!fgets(buf, sizeof(buf) - 1, f)) {
+      LOG(warn) << "Can't read /proc/net/tcp6";
+    }
+    while (fgets(buf, sizeof(buf) - 1, f)) {
+      // Ignore the first field.
+      strtok(buf, " ");
+      // Then come the local and remote addresses.
+      char* local_addr_str = strtok(NULL, " ");
+      char* remote_addr_str = strtok(NULL, " ");
+      // Then six more fields to ignore.
+      strtok(NULL, " ");
+      strtok(NULL, " ");
+      strtok(NULL, " ");
+      strtok(NULL, " ");
+      strtok(NULL, " ");
+      strtok(NULL, " ");
+      // Finally the inode
+      char* ino = strtok(NULL, " ");
+      ino_t inod = strtoull(ino, NULL, 10);
+      if (inod != inode) {
+        continue;
+      }
+
+      // We have a match.
+      struct sockaddr_in6* local_addr_in6 = (struct sockaddr_in6*)&out[0];
+      local_addr_in6->sin6_family = AF_INET6;
+
+      if (strlen(local_addr_str) != 37) {
+        break;
+      }
+      char buf[9];
+      for (int i = 0; i < 4; ++i) {
+        char* c = local_addr_str + i * 8;
+        strncpy(buf, c, 8);
+        buf[8] = '\0';
+        unsigned long local_addr_component = strtoul(buf, NULL, 16);
+        if ((local_addr_component == ULONG_MAX && errno == ERANGE)) {
+          LOG(warn) << "Local address not in expected format";
+          break;
+        }
+        local_addr_in6->sin6_addr.s6_addr32[i] = local_addr_component;
+      }
+      if (*(local_addr_str + 32) != ':') {
+        LOG(warn) << "Local address not in expected format";
+        break;
+      }
+      unsigned long lport = strtoul(local_addr_str + 33, NULL, 16);
+      if (lport > USHRT_MAX) {
+        LOG(warn) << "Local address not in expected format";
+        break;
+      }
+      local_addr_in6->sin6_port = lport;
+
+      struct sockaddr_in6* remote_addr_in6 = (struct sockaddr_in6*)&out[1];
+      remote_addr_in6->sin6_family = AF_INET6;
+
+      if (strlen(remote_addr_str) != 37) {
+        break;
+      }
+      for (int i = 0; i < 4; ++i) {
+        char* c = remote_addr_str + 24 - i * 8;
+        strncpy(buf, c, 8);
+        buf[8] = '\0';
+        unsigned long remote_addr_component = strtoul(buf, NULL, 16);
+        if ((remote_addr_component == ULONG_MAX && errno == ERANGE)) {
+          LOG(warn) << "Remote address not in expected format";
+          break;
+        }
+        remote_addr_in6->sin6_addr.s6_addr32[i] = remote_addr_component;
+      }
+      if (*(remote_addr_str + 32) != ':') {
+        LOG(warn) << "Remote address not in expected format";
+        break;
+      }
+      uint16_t rport = strtoul(remote_addr_str + 33, NULL, 16);
+      if (rport > USHRT_MAX) {
+        LOG(warn) << "Remote address not in expected format";
+        break;
+      }
+      remote_addr_in6->sin6_port = rport;
+      fclose(f);
+      return true;
+    }
+    fclose(f);
+  } else {
+    LOG(warn) << "Can't open /proc/net/tcp6";
+  }
+
+  return false;
+}
+
 static bool try_copy_file_by_copy_all(int dest_fd, int src_fd)
 {
   static bool should_try_copy_all = true;
@@ -1681,6 +1929,9 @@ bool is_advanced_pc_and_signaled_instruction(Task* t, remote_code_ptr ip) {
       !memcmp(insn, int3_insn, sizeof(int3_insn))) {
     return true;
   }
+#else
+  UNUSED(t);
+  UNUSED(ip);
 #endif
   return false;
 }
@@ -1915,6 +2166,7 @@ ssize_t pwrite_all_fallible(int fd, const void* buf, size_t size, off64_t offset
     }
     buf = static_cast<const char*>(buf) + ret;
     written += ret;
+    offset += ret;
     size -= ret;
   }
   return written;
@@ -2058,6 +2310,7 @@ std::vector<int> read_all_proc_fds(pid_t tid)
   while (struct dirent *dir = readdir(fddir)) {
     ret.push_back(atoi(dir->d_name));
   }
+  closedir(fddir);
   return ret;
 }
 
@@ -2082,4 +2335,27 @@ int pop_count(uint64_t v) {
   return ret;
 }
 
+void SAFE_FATAL(int err, const char *msg)
+{
+  static char prefix[] = "FATAL (errno = ";
+  const char *errstr = errno_name_cstr(err);
+  char buf[100];
+  if (errstr == NULL) {
+    snprintf(buf, sizeof(buf), "errno(%d)", err);
+    errstr = buf;
+  }
+  static char bridge[] = "): ";
+  static char nl[] = "\n";
+  struct iovec out[5] = {
+    {.iov_base = prefix, .iov_len=sizeof(prefix)},
+    {.iov_base = (char*)errstr, .iov_len=strlen(errstr)},
+    {.iov_base = bridge, .iov_len=sizeof(errstr)},
+    {.iov_base = (char*)msg, .iov_len=strlen(msg)},
+    {.iov_base = nl, .iov_len=sizeof(nl)}
+  };
+  (void)::writev(STDERR_FILENO, out, sizeof(out)/sizeof(struct iovec));
+  abort();
+}
+
+
 } // namespace rr
