@@ -28,6 +28,13 @@
 #define SOL_NETLINK 270
 #endif
 
+#define UNUSED(expr)     \
+  do {                   \
+    if (expr) {          \
+      (void)0;           \
+    }                    \
+  } while (0)
+
 namespace rr {
 
 /*
@@ -51,12 +58,22 @@ enum Completion { COMPLETE, INCOMPLETE };
  */
 std::vector<uint8_t> read_auxv(Task* t);
 
+/**
+ * Returns the base address where the interpreter is mapped.
+ */
+remote_ptr<void> read_interpreter_base(std::vector<uint8_t> auxv);
+
+/**
+ * Returns a string containing the file name of the interpreter.
+ */
+std::string read_ld_path(Task* t, remote_ptr<void> interpreter_base);
+
 /**
  * Returns a vector containing the environment strings.
  */
 std::vector<std::string> read_env(Task* t);
 
-void patch_auxv_vdso(RecordTask* t);
+void patch_auxv_vdso(RecordTask* t, uintptr_t search, uintptr_t new_entry);
 
 /**
  * Create a file named |filename| and dump |buf_len| words in |buf| to
@@ -181,8 +198,11 @@ enum cpuid_requests {
   CPUID_INTELBRANDSTRING,
   CPUID_INTELBRANDSTRINGMORE,
   CPUID_INTELBRANDSTRINGEND,
+  CPUID_AMD_CACHE_TOPOLOGY = 0x8000001D,
+  CPUID_AMD_PLATFORM_QOS = 0x80000020
 };
 
+const int XSAVE_FEATURE_FLAG = 1 << 26;
 const int OSXSAVE_FEATURE_FLAG = 1 << 27;
 const int AVX_FEATURE_FLAG = 1 << 28;
 const int HLE_FEATURE_FLAG = 1 << 4;
@@ -199,6 +219,12 @@ struct CPUIDData {
 };
 CPUIDData cpuid(uint32_t code, uint32_t subrequest);
 
+/**
+ * Check whether the given result of cpuid(CPUID_GETVENDORSTRING) indicates
+ * an AMD processor.
+ */
+bool is_cpu_vendor_amd(CPUIDData vendor_string);
+
 /**
  * Check OSXSAVE flag.
  */
@@ -299,6 +325,13 @@ std::vector<std::string> read_proc_status_fields(pid_t tid, const char* name,
  */
 bool uses_invisible_guard_page();
 
+/**
+ * Search /proc/net/ for a socket of the correct family matching the provided fd.
+ * If found, returns the local and remote addresses in out and returns true.
+ * Otherwise, returns false.
+ */
+bool read_proc_net_socket_addresses(Task* t, int fd, std::array<typename NativeArch::sockaddr_storage, 2>& out);
+
 bool copy_file(int dest_fd, int src_fd);
 
 #if defined(__has_feature)
@@ -502,11 +535,14 @@ void normalize_file_name(std::string& s);
 enum NestedBehavior {
   NESTED_ERROR,
   NESTED_IGNORE,
-  NESTED_DETACH
+  NESTED_DETACH,
+  NESTED_RELEASE,
 };
 
 std::string find_exec_stub(SupportedArch arch);
 
+std::string find_helper_library(const char* basepath);
+
 static inline struct timeval to_timeval(double t) {
   struct timeval v;
   v.tv_sec = (time_t)floor(t);
@@ -517,6 +553,10 @@ static inline struct timeval to_timeval(double t) {
 /* Slow but simple pop-count implementation. */
 int pop_count(uint64_t v);
 
+/* A version of fatal that uses no allocation/thread resource and is thus
+  safe to use in volatile contexts */
+void SAFE_FATAL(int err, const char *msg);
+
 } // namespace rr
 
 #endif /* RR_UTIL_H_ */
